/*
 * ProgramMain.java
 *
 * Created on 23 febbraio 2009, 18.09
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package romas;

import java.util.Vector;
import java.util.Hashtable;
import java.util.logging.Level;
import java.io.File;

import romas.com.data.tables.NOTIFICATIONS_MAP;
import romas.services.*;
import romas.services.actuator.ActuatorService;
import romas.services.videostream.VideoStreamService;
import romas.services.voice.SpeakerService;
import romas.services.speakrecognition.SRService;
import romas.services.sensor.SensorService;
import romas.services.movement.MovementService;
import romas.services.comunications.ComunicationService;
import romas.com.data.*;
import romas.util.LogWriter;

/**
 *
 * @author APaliani
 */
public class ProgramMain {

    static ProcessManager processManager;

    static Hashtable boards;
    static romas.com.boards.Arduino2009 arduino2009;
    
    /** Creates a new instance of ProgramMain */
    public ProgramMain() {
    }

    
    /**
     *Inizializzazione del sistema
     */
    private static void initSystem()throws Exception
    {
        String USER_DIR_KEY = "user.dir";
        String currentDir = System.getProperty(USER_DIR_KEY);

        // elimina tutti files del database e del log per evitare corruzioni
        File dir = new File(currentDir);
        try{
            File[] files = dir.listFiles();
            for(int i=0; i< files.length; i++)
            {
                File file = files[i];

                if(file.isFile())
                {
                    String fileName = file.getName();
                    if((!fileName.endsWith(".bat")) && (!fileName.endsWith(".log")))
                    {
                        if(file.delete()){
                            LogWriter.writeLog(Level.CONFIG, "Deleted file: " + file.getName());
                        }
                        else{
                            LogWriter.writeLog(Level.WARNING, "Cannot delete file: " + file.getName());
                        }
                    }                
                }
            }
        }
        catch(Exception ex)
        {
            System.err.println(ex.getMessage());
        }
        
        try
        {
            Configuration config = new Configuration();
            config.loadPropertiesFile("config" + File.separator + "romas.properties");
            String log_Level_name = config.getPropertiesValue("Log_Level_name");
            LogWriter.init("romas.log", log_Level_name);
        }
        catch(Exception ex)
        {
            LogWriter.init("romas.log", "ALL");
            LogWriter.writeLog(Level.WARNING, "Cannot create configured log !");
        }
        
        LogWriter.writeLog(Level.INFO, "\n\n******************************************************************************************************************\n\n" + 
                                       "START ROMAS ON DIR: " + currentDir + " ........................\n\n" + 
                                       "******************************************************************************************************************\n\n"); 
        

        // create all tables
        TableManager.createAllTables();        
    }
    
    
    /**
     *Avvio dei servizi 
     *
     * @param idThreadMain    id del thread principale
     * @throws Exception      Intercetta tutte le eccezioni
     */
    private static void startServices(long idThreadMain)throws Exception
    {
        // File di configurazione
        Configuration config = new Configuration();
        config.loadPropertiesFile("config" + File.separator + "romas.properties");
        int clockAction = Integer.parseInt(config.getPropertiesValue("Clock_Action"));        

        LogWriter.writeLog(Level.INFO, "RUN BASIC SERVICES ........................\n");

        System.out.println("\007\007\007");

        // ProcessManager
        processManager = new ProcessManager(clockAction);
        processManager.setPriority(Thread.MAX_PRIORITY);

        // SpeakerService
        System.out.println("\007");
        processManager.addService(new SpeakerService());


        LogWriter.writeLog(Level.INFO, "INIT COMMON BOARDS ........................\n");

        boards = new Hashtable();

        //Arduino2009
        try{
            String arduinoPortName = config.getPropertiesValue("board_arduino2009_port");
            arduino2009 = new romas.com.boards.Arduino2009(arduinoPortName, 9600);
            boards.put("arduino2009", arduino2009);
            LogWriter.writeLog(Level.INFO, "Added board : Arduino2009 on port "+ arduinoPortName +"\n");
        }catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
            Vector params_ = new Vector();
            params_.add("Errore avviamento scheda : Arduino 2009");
            notifyAction("SPEAKER", "SPEAK", "TEXT", params_, true);
        }


        
        // VideoStreamService
        System.out.println("\007");
        processManager.addService(new VideoStreamService());


        // ComunicationService
        System.out.println("\007");
        processManager.addService(new ComunicationService());



        // Speak_RecognitionService
        System.out.println("\007");
        SRService srService = new SRService();
        Thread srServiceThread = new Thread(srService);
        srServiceThread.setPriority(srService.getPriorityScope());
        srServiceThread.start();
        processManager.addService(srService);



        // Actuator_Service
        try{
            System.out.println("\007");
            ActuatorService actuatorService = actuatorService = new ActuatorService();
            processManager.addService(actuatorService);
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
            Vector params = new Vector();
            params.add("Errore, avviamento servizio: controllo servo motori");
            notifyAction("SPEAKER", "SPEAK", "TEXT", params, true);
        }
        

        // Movement service
        try{
            System.out.println("\007");
            MovementService movementService = new MovementService(arduino2009);
            if(arduino2009 == null) throw new Exception("Arduino2009 board is null !");
            processManager.addService(movementService);
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
            Vector params = new Vector();
            params.add("Errore, avviamento servizio: movimento");
            notifyAction("SPEAKER", "SPEAK", "TEXT", params, true);
        }





        // DirectiveService
        System.out.println("\007");
        processManager.addService(new DirectiveService());
        // caricamento delle direttive principali
        notifyAction("DIRECTIVE", "LOAD", "DEFAULT", null, true);



        // Avvio del Process Manager        
        processManager.start();



        // Sensor service
        try{
            System.out.println("\007");
            SensorService sensorService = new SensorService(arduino2009);
            if(arduino2009 == null) throw new Exception("Arduino2009 board is null !");
            Thread sensorServiceThread = new Thread(sensorService);
            sensorServiceThread.setPriority(sensorService.getPriorityScope());
            sensorServiceThread.start();
            processManager.addService(sensorService);
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
            Vector params = new Vector();
            params.add("Errore, avviamento servizio: sensori");
            notifyAction("SPEAKER", "SPEAK", "TEXT", params, true);
        }

        System.out.println("\007\007\007");
    }
    
    
    /**
     *Notifica la richiesta di una ACTION
     * @param scope
     * @param action
     * @param command
     * @param params
     * @param active
     * @throws Exception
     */
    public static void notifyAction(String scope, String action, String command, Vector params, boolean active)throws Exception
    {
        String actionFULLNAME = action + "_" + command;
        NOTIFICATIONS_MAP notify = new NOTIFICATIONS_MAP();
        notify.FULLNAME_EVENT = actionFULLNAME;
        notify.PARAMS_COMMAND_EVENT = params;
        notify.NOTIFICATOR = scope;
        notify.ACTION_EVENT = action;
        notify.COMMAND_EVENT = command;
        notify.NUMBER_ITERATION_NOTIFY = 1;
        notify.ACTIVE = active;
        
        if(processManager != null)
        {
            if(processManager.services != null)
            {
                if(processManager.services.containsKey(scope))
                {
                    ((NotificationService)processManager.services.get(scope)).notifyAction(notify);
                }
            }
        }
    }
    
    
    private static void start(){
        // Set max priority on thread main
        Thread threadMain = Thread.currentThread();
        threadMain.setPriority(Thread.MAX_PRIORITY); 
        
        //inizializzazione del sistema
        try{
            initSystem();
        }
        catch(Exception ex)
        {
            LogWriter.writeLog(Level.SEVERE, "Init System ERROR: " + ex.getMessage());
            System.exit(0);
        }        
        
        
        // avvio dei servizi
        try{
            startServices(threadMain.getId());
        }
        catch(Exception ex)
        {
            LogWriter.writeLog(Level.SEVERE, "Start services ERROR: " + ex.getMessage());
            System.exit(0);
        }        
    
    }


    /**
     *Metodo principale che avvia il sitema
     * @param args
     */
    public static void main(String[] args)
    {
        start();
    }
    
}
