/*
 * La classe si occupa di interfacciarsi ad Arduino 2009 eseguendo una connessione sulla porta seriale parametrizzata.
 * Deve essere tenuto conto del fatto che a fronte di un comando inviato ad Arduino la classe resta in stato di lock
 * fin quando la scheda Arduino non risponde sulla seriale.
 * Ne deriva che, anche se la scheda non deve restituire un valore, a fronte della richiesta
 * deve comunque rispondere un risultato per sbloccare la classe.
 */

package romas.com.boards;


import romas.util.comunication.port.Serial;
import romas.util.LogWriter;

import java.util.logging.Level;
import java.util.StringTokenizer;


/**
 *
 * @author apaliani
 */
public class Arduino2009{

    Serial serialPort;
   
    // blocca la scheda dall'interno
    public volatile boolean lock = false;
    public volatile boolean busy = false;
    
    public static volatile String sValue = "";
    
    public static volatile String processedSensor = "";
    public static volatile String processedCommand = "";

    public Arduino2009(){}

    public Arduino2009(String portName, int baudRate)throws Exception{
        // create instance of serial port
        serialPort = new Serial(portName, baudRate, false){
            @Override
            public void processMessage(byte[] buffer) {
                processBoardMessage(buffer);
            }
        };

        if(serialPort == null) throw new Exception("Cannot Init Arduino2009 board !");
    }

    /**
     *
     * @param buffer
     *
     *
     */
    private void processBoardMessage(byte[] buffer){
        String sResultString = "";
        try{

            if(buffer.length > 0){
                sValue = "";
                sResultString = new String(buffer);

                processedSensor = "";
                processedCommand = "";


                if(sResultString.indexOf("ALARM")> -1){
                    sResultString = sResultString.substring(sResultString.indexOf("ALARM"), sResultString.length());
                }


                StringTokenizer tokens = new StringTokenizer(sResultString, ":");
                String resultSensor = tokens.nextToken();

                if(resultSensor.startsWith("SR") || resultSensor.startsWith("ALARM")){
                    processedSensor = resultSensor;
                    try{
                        sValue = tokens.nextToken();
                    }catch(Exception ex){
                        sValue = "0";
                    }
                }
                // SR_IR_01; SR_US_01;
                else{
                    processedCommand = sResultString;
                }
            }
            else{
                LogWriter.writeLog(Level.FINER, "Buffer empty or corrupt ["+ sResultString +"] !");
            }

        }catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
        }
        finally{
            lock = false;
        }
    }


    public void sendCommand(String command){
        try{
            lock = true;
            serialPort.writeMessage(command, false);
        }catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
        }
    }


}
