/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package romas.com.data.hmm;

import java.io.*;
import java.text.*;
/**
 *
 * @author apaliani
 */
public class DemoHMM {

public static void main(String[] args) throws IOException
	{
	  	/** DEMO */

	  	InputStreamReader isr = new InputStreamReader(System.in);
	  	BufferedReader reader = new BufferedReader(isr);
	  	DecimalFormat fmt = new DecimalFormat();
	  	fmt.setMinimumFractionDigits(0);
	  	fmt.setMaximumFractionDigits(0);

	  	System.out.println("Urns and Marbles example\n<Press any enter to begin>");
	  	reader.read();reader.read();

	  	System.out.println("Creating a new HMM with 3 states and 3 observation symbols...");
	  	System.out.println("States: 0, 1, 2");
	  	System.out.println("Symbols: 0 (red), 1 (green), 2 (blue)");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	// HMM with 3 states and 3 distinct observation symbols
	  	HMM hmm = new HMM(3, 3);

	  	// initialization (urn example from my HMM presentation)
	  	System.out.println("Setting initial, transition, and emission probabilities...");
	  	// initials
	  	hmm.pi[0] = 0.5;
	  	hmm.pi[1] = 0.25;
	  	hmm.pi[2] = 0.25;
	  	// transitions
	  	hmm.a[0][0] = 0.2;
	  	hmm.a[0][1] = 0.6;
	  	hmm.a[0][2] = 0.2;
	  	hmm.a[1][0] = 0.25;
	  	hmm.a[1][1] = 0.5;
	  	hmm.a[1][2] = 0.25;
	  	hmm.a[2][0] = 0.35;
	  	hmm.a[2][1] = 0.15;
	  	hmm.a[2][2] = 0.5;
	  	// emissions
	  	hmm.b[0][0] = 0.25;
	  	hmm.b[0][1] = 0.25;
	  	hmm.b[0][2] = 0.5;
	  	hmm.b[1][0] = 0.2;
	  	hmm.b[1][1] = 0.3;
	  	hmm.b[1][2] = 0.5;
	  	hmm.b[2][0] = 0.45;
	  	hmm.b[2][1] = 0.05;
	  	hmm.b[2][2] = 0.5;

	  	hmm.print();
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	System.out.println("Storing hmm in hmm.txt...");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();
	 	hmm.write("hmm.txt");

	  	int[] o;
	  	double[][] alpha, beta;
	  	double prob = 0.0;

	  	// observation sequence generation example
	  	System.out.println("Lets generate an observation sequence of length 5 using this hmm...");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	System.out.println("Generating sequence...");
	    	o = hmm.observationGeneration(5);
	  	System.out.println("o = " + o[0] + "," + o[1] + "," + o[2] + "," + o[3] + "," + o[4]);
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	// forward algorithm exmaple
	  	System.out.println("What's the probability of R, G, B?");
	  	System.out.println("O = RGB or 0, 1, 2");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	// create observation sequence o--note that we specify indices
	  	// corresponding to the r, g, b 'bars' of our symbol distributions
	  	o = new int[3];
	  	o[0] = 0;
	  	o[1] = 1;
	  	o[2] = 2;

		System.out.println("Calling the forward procedure with O...");
		System.out.println("<enter>");
		reader.read();reader.read();

		// forwardProc returns an alpha table (all that's
		// left to do is sum up the last column)
	  	alpha = hmm.forwardProc(o);
	  	for(int i=0; i < 3; i++)
	  		prob += alpha[i][2];
	  	System.out.println("The probability of R, G, B is " + fmt.format(100*prob) + "%");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	// backward example
	  	System.out.println("Now lets try it with the backward procedure...");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	// similarly, backwardProc returns a beta table and we
	  	// perform the necessary steps to extract the probability
	  	prob = 0;
	  	beta = hmm.backwardProc(o);
	  	for(int i=0; i < 3; i++)
	  		prob += hmm.pi[i]*hmm.b[i][o[0]]*beta[i][0];
	  	System.out.println("The probability of R, G, B is " + fmt.format(100*prob) + "%");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	// Baum-Welch example
	  	System.out.println("Next lets train this HMM to increase the probability of RGB...");
	  	System.out.println("Calling Baum-Welch with O...");
	  	hmm.baumWelch(o, 9); // 9 = # of steps
	  	System.out.println("<enter>");
	  	reader.read();reader.read();


	  	System.out.println("HMM trained successfully.");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	System.out.println("Once more, calling the forward procedure with O...");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	prob = 0;
	  	alpha = hmm.forwardProc(o);
	  	for(int i=0; i < 3; i++)
	  		prob += alpha[i][2];
	  	fmt.setMaximumFractionDigits(1);
	  	System.out.println("The probability of R, G, B is " + fmt.format(100*prob) + "%");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	System.out.println("The parameters of the trained HMM are...");
	  	hmm.print();
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	System.out.println("Lets load the initial HMM...");
	  	hmm = HMM.load("hmm.txt");

	  	hmm.print();
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	// Viterbi example
	  	o = new int[2];
	  	o[0] = 0;	// change the indices to represent a different sequence
	  	o[1] = 2;
	  	System.out.println("Next, lets try the Viterbi algorithm using O = red, blue...");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	// Viterbi returns a table consisting of a minimized weight, which can be turned
	  	// into the state optimized probability (see below) and an optimized state sequence.
	  	// Structure of answer: answer[0][0] = min weight, answer[1][i] = path, for i=0..T-1.
	  	double[][] answer;
	  	System.out.println("Calling Viterbi with O...");
	  	answer = hmm.viterbi(o);
	  	// recall that if w is our minimized weight, P = e^(-w)
	  	System.out.println("State optimized probability is " + fmt.format(100*Math.exp(-1*answer[0][0])) + "%");
	  	System.out.print("Optimal path is ");
	  	for(int i = 0; i < o.length; i++) {
	  		System.out.print((int)answer[1][i] + "");
	  		if(i != o.length-1)
	  			System.out.print(", ");
	  	}
	  	System.out.println();
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	// k-means example
	  	System.out.println("K-means example using gesture data");
	  	System.out.println("Training file structure:");
	  	System.out.println("Header info: dimensions, length of each sequence, # of sequences.");
	  	System.out.println("Each row represents, in this case, a goodbye gesture (each consists of 3 poses).");
	  	System.out.println("Each column, in this case, represents a particular pose.");
	  	System.out.println("E.g., the first column represents the first pose of a goodbye gesture.");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	// call kmeansLearner and create our new hmm
	  	// # of states = 3, # of steps = 5
	  	HMM newHmm = HMM.kmeansLearner("training.txt", 3, 5);

	  	System.out.println("After K-means, our new HMM is...");
	  	newHmm.print();
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	System.out.println("Storing newHmm in hmm2.txt...");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();
	  	newHmm.write("hmm2.txt");

	  	System.out.println("Note that the first column (pose #1) is grouped into one " +
	  					   "state, \n2nd column in another, and so on...");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	// forward example using newHmm
	  	// convert is used here--input is a 2d array where the rows are symbols
	  	// and the columns are dimensions
	  	double[][] seq = new double[3][3];
	  	seq[0][0] = 90.0;  seq[0][1] =  45.0;  seq[0][2] =  30.0;
	  	seq[1][0] = 135.0; seq[1][1] =  90.0;  seq[1][2] =  45.0;
	  	seq[2][0] = 180.0; seq[2][1] =  135.0; seq[2][2] =  90.0;

		System.out.println("Calling the forward procedure with a certain gesture...");
		System.out.println("<enter>");
		reader.read();reader.read();

		// convert from real values to actual indices before using forwardProc
		prob = 0;
	  	alpha = newHmm.forwardProc(newHmm.convert(seq, 3, 3));
	  	for(int i=0; i < 2; i++)
	  		prob += alpha[i][1];
	  	System.out.println("The probability is " + fmt.format(100*prob) + "%");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	System.out.println("To increase accuracy, we could train the hmm using the\n"
	  	        + "most common gesture or perhaps an average gesture, \nand perform "
	  	        + "pose recognition on all incoming gestures. \n(Exercise left for reader.)");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	System.out.println("Lets go back to the original hmm, the urns and marbles example.");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();



	  	System.out.println("Lets create a new HMM by taking the original urns and marbles "
	  	                   + "HMM and changing some of its values...");

	  	HMM myHmm = new HMM(3, 3);
	  	// initials
	  	myHmm.pi[0] = 0.5;
	  	myHmm.pi[1] = 0.3;
	  	myHmm.pi[2] = 0.2;
	  	// transitions
	  	myHmm.a[0][0] = 0.4;
	  	myHmm.a[0][1] = 0.3;
	  	myHmm.a[0][2] = 0.3;
	  	myHmm.a[1][0] = 0.2;
	  	myHmm.a[1][1] = 0.1;
	  	myHmm.a[1][2] = 0.7;
	  	myHmm.a[2][0] = 0.6;
	  	myHmm.a[2][1] = 0.3;
	  	myHmm.a[2][2] = 0.1;
	  	// emissions
	  	myHmm.b[0][0] = 0.1;
	  	myHmm.b[0][1] = 0.1;
	  	myHmm.b[0][2] = 0.8;
	  	myHmm.b[1][0] = 0.3;
	  	myHmm.b[1][1] = 0.4;
	  	myHmm.b[1][2] = 0.3;
	  	myHmm.b[2][0] = 0.5;
	  	myHmm.b[2][1] = 0.3;
	  	myHmm.b[2][2] = 0.2;

	  	myHmm.print();
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	System.out.print("The distance between this hmm and the original is ");
	  	System.out.println(hmm.hmmDistance(myHmm) + "");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	System.out.println("Now, lets train both of these hmms with r,g,b and see how \n"
	  			+ "the distance between them changes as they learn...");
	  	System.out.println("<enter>");
	  	reader.read();reader.read();

	  	o = new int[3];
	  	o[0] = 0;
	  	o[1] = 1;
	  	o[2] = 2;

	  	for(int i = 0; i < 12; i++) {
		  	System.out.println("iteration #" + (i+1));
		  	hmm.baumWelch(o, 1); // 1 = # of steps
		  	myHmm.baumWelch(o, 1);
		  	System.out.println("Distance is " + hmm.hmmDistance(myHmm));
		  	System.out.println("<enter>");
		  	reader.read();reader.read();
	    }

	  	System.out.println("End of Demo");

	  	/** END OF DEMO */
	}
}
