/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package romas.com.data.hmm;

import java.io.File;
import java.util.logging.Level;
import romas.util.LogWriter;
/**
 *
 * @author apaliani
 */
public class HMM_Manager {


    /**
     * Add new sequence into file training
     * create the file if not exist
     *
     * @param fileTraining
     * @param seqString
     * @param nDimensions
     * @param nSeq
     * @param seqLength
     * @throws java.lang.Exception
     */
    public static void addSequenceTraining(String fileTraining, String seqString, int nDimensions, int nSeq, int seqLength)throws Exception{
        File file = new File(fileTraining);

        if(!file.exists()){
            HMMTraining.creteNew(fileTraining, nDimensions, seqLength, nSeq);
        }

        HMMTraining.addSequence(fileTraining, seqString);
    }


    public static void deleteFileTraining(String fileTraining){
        Runtime rt = Runtime.getRuntime();
            try{
                int returnValue = -1;
                Process process = rt.exec(" del "+ fileTraining);
                returnValue = process.waitFor();
                if(returnValue == -1){
                    System.err.println("Impossibile cancellare file: " + fileTraining);
                }
            }catch(Exception ex){
                System.err.println("Impossibile cancellare file: " + fileTraining + " - Error: " + ex.getMessage());
            }
//        File file = new File(fileTraining);
//        if(!file.exists()){
//            if(!file.delete()){
//                System.err.println("Impossibile cancellare file: " + fileTraining);
//            }
//        }
    }


    /**
     *
     * Add new sequence into file training
     * create the file if not exist
     *
     * @param fileTraining
     * @param sequence
     * @param nDimensions
     * @param nSeq
     * @throws java.lang.Exception
     */
    public static void addSequenceTraining(String fileTraining, double[][] sequence, int nDimensions, int nSeq)throws Exception{
        File file = new File(fileTraining);

        if(!file.exists()){
            HMMTraining.creteNew(fileTraining, nDimensions, sequence.length, nSeq);
        }

        String seqString = "";

        if(sequence != null){
            for(int i=0; i< sequence.length; i++){
                for(int j=0; j<nDimensions; j++){
                    seqString += String.valueOf(sequence[i][j]) + " ";
                }
            }
        }

        HMMTraining.addSequence(fileTraining, seqString);
    }


    /**
     * Trasform the training file in HMM file
     *
     * @param fileTraining
     * @param hmmFileName
     * @param nState
     * @param nIter
     * @throws java.lang.Exception
     */
    public static void trainingToHMM(String fileTraining, String hmmFileName, int nState, int nIter)throws Exception{
        //creazione di un HMM dal file di Training
        HMM hmm = HMM.kmeansLearner(fileTraining, nState, nIter);
        hmm.write(hmmFileName);
    }




    /**
     *
     * Return tthe sequence probability in HMM file
     *
     * @param hmm
     * @param nDimensions
     * @param nSeq
     * @param seq
     * @return
     */
    public static double getProbability(HMM hmm, int nDimensions, int lengthSeq, double[][] seq){
        double[][] alpha;
        double prob = 0.0;

        prob = 0;

        int[] observation = getObservation(hmm, nDimensions, lengthSeq, seq);

        //hmm.baumWelch(observation, nDimensions);

        alpha = hmm.forwardProc(observation);
        for(int i=0; i < alpha.length; i++){
            for(int j=0; j<nDimensions; j++){
                prob += alpha[i][j];
            }
        }

        return (100*prob);         
    }

    public static double getDistance(String hmmFileName, String pathModels, int nDimensions, int lengthSeq, double[][] sequence){
        double distance = -1;
        try{
            String tempHMM_Name = pathModels + File.separator + "temp";
            deleteFileTraining(tempHMM_Name + ".training");
            HMM hmmTemp = createHMM_BySequence(tempHMM_Name , nDimensions, lengthSeq, sequence);
            HMM hmm = HMM.load(hmmFileName);

            distance = distanceHMM(hmm, hmmTemp, getObservation(hmm, nDimensions, lengthSeq, sequence));
        }catch(Exception ex){
            System.err.println("Errore in getDistance : " + ex.getMessage());
        }
        return distance;
    }


    public static double distanceHMM(HMM hmm_A, HMM hmm_B, int[] observation){
	  	double distance = -1;
        hmm_A.baumWelch(observation, 10);
        distance = hmm_A.hmmDistance(hmm_B);
        System.out.println("Distance is " + distance);

        return distance;
    }


    public static HMM createHMM_BySequence(String tempFileName, int nDimensions, int lengthSeq, double[][] sequence){
        HMM tempHMM = null;
        try{
            deleteFileTraining(tempFileName + ".training");

            addSequenceTraining(tempFileName + ".training", sequence, nDimensions, 1);

            trainingToHMM(tempFileName + ".training", tempFileName + ".hmm", lengthSeq, 9);

            tempHMM = HMM.load(tempFileName + ".hmm");

            int[] observation = getObservation(tempHMM, nDimensions, lengthSeq, sequence);

            baumWelch(tempFileName + ".hmm", observation, 9);

            tempHMM = HMM.load(tempFileName + ".hmm");
            
        }catch(Exception ex){
            System.err.println("Errore in createHMM_BySequence : " + ex.getMessage());
        }
        return tempHMM;
    }


    public static double getViterbi(HMM hmm, int nDimensions, int lengthSeq, double[][] seq){
        int[] o = getObservation(hmm, nDimensions, lengthSeq, seq);
        double[][] answer = hmm.viterbi(o);
        
        System.out.print("Optimal path is ");
        for(int i = 0; i < o.length; i++) {
            System.out.print((int)answer[1][i] + "");
            if(i != o.length-1){
                System.out.print(", ");
            }
        }
        
        double result = (100*Math.exp(-1*answer[0][0]));
        return result;
    }


    /**
     * Ritorna l'osservazione risultante dalla sequenza
     * @param hmm
     * @param nDimensions
     * @param lengthSeq
     * @param seq
     * @return
     */
    public static int[] getObservation(HMM hmm, int nDimensions, int lengthSeq, double[][] seq){
        try{
            int[] observation = hmm.convert(seq, nDimensions, lengthSeq);
            return observation;
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, "Errore in getObservetion HMM: " + ex.getMessage());
        }
        return null;
    }


    /**
     * Aumenta le probabilit della sequenza all'interno dell'HMM
     * @param hmm
     * @param observetion
     * @param steps
     */
    public static void baumWelch(String hmmFileName, int[] observetion, int steps){
        try{
            HMM hmm = HMM.load(hmmFileName);
            hmm.baumWelch(observetion, steps);
            hmm.write(hmmFileName);
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, "Errore in baumWelch HMM: " + ex.getMessage());
        }
    }


    public static void main(String args[]){
        try{
            HMM_Manager.trainingToHMM("ciao.training", "ciao.hmm", 256, 100);
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
        }
    }
}
