/*
 * ActionService.java
 *
 * Created on 23 febbraio 2009, 12.15
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package romas.services;

import java.io.File;
import java.util.logging.*;

import romas.com.data.*;
import romas.com.data.tables.TABLE;
import romas.com.data.tables.ACTIONS_PROCESS;
import romas.com.data.tables.foreignkeys.ACTION_SCOPE;
import romas.util.LogWriter;
import romas.services.ActionService.ACTION_STATE;


/**
 *
 * @author APaliani
 */
public abstract class ActionService {
    
    // livello / priorit dei servizi     
    public static class SERVICE_TYPE
    {
        public static final int BASIC_SERVICE = 1;
        public static final int PRIMARY_SERVICE = 2;
        public static final int SECONDARY_SERVICE = Thread.NORM_PRIORITY;
        public static final int ADDITIONAL_SERVICE = (Thread.NORM_PRIORITY + 1);
    }
    
    
    // priorit delle azioni
    public static class ACTION_PRIORITY
    {
        public static final int LOW_PRIORITY_ACTION = 0;
        public static final int MEDIUM_PRIORITY_ACTION = 1;
        public static final int HIGH_PRIORITY_ACTION = 2;
    } 
    
    
    /**
     * Stati delle azioni CREATED, INITIATED, RUNNING, STOPPED, FINISHED, DELETING
     */
    public static class ACTION_STATE
    {
        public static final int CREATED_ACTION_STATE = 0;   //0
        public static final int INITIATED_ACTION_STATE = 1;
        public static final int RUNNING_ACTION_STATE = 2;
        public static final int STOPPED_ACTION_STATE = 3;
        public static final int FINISHED_ACTION_STATE = 4;
        public static final int DELETING_ACTION_STATE = 5;
    }
        
    
    /**
     *Tipo dell'azione
     */
    public static class ACTION_TYPE{
        public static final int ORDINARY_ACTION_TYPE = 0;
        public static final int ALTERNATIVE_ACTION_TYPE = 1;
        public static final int OPPOSITE_ACTION_TYPE = 2;
    }
    
    public Configuration config;
    
    /** Creates a new instance of ActionService */
    public ActionService() {
    }
    
    
    /**
     *Inizializzazione del servizio
     */
    protected void init(String serviceName)throws Exception{
        String USER_DIR_KEY = "user.dir";
        String currentDir = System.getProperty(USER_DIR_KEY);        
        
        config = new Configuration();
        config.loadPropertiesFile("config" + File.separator + "services" + File.separator + serviceName + ".properties");
        
        config.loadNotificationsXMLFile(currentDir + File.separator + "config" + File.separator + "services" + File.separator + serviceName + "_notifications.xml");
        config.loadActionsXMLFile(currentDir + File.separator + "config" + File.separator + "services" + File.separator + serviceName + "_actions.xml");
        LogWriter.writeLog(Level.CONFIG, String.format("Init service [%s] ... ", serviceName));
        
        setActionList(serviceName);
    }
    
            
    /**
     *Metodo da implementare per eseguire il lavoro delle azioni del servizio
     */
    protected abstract boolean actionWork(ACTIONS_PROCESS actionProcess);

    
    /**
     *Ritorna la priorit del Servizio
     *per impostare la priorit del thread
     */
    public abstract int getPriorityScope();

        
    /**
     *Ritorna l'ambiente dove opera il servizio
     */
    public final String getScope()throws Exception{
        String scope = config.getPropertiesValue("SCOPE");
        if(!ACTION_SCOPE.getList().containsKey(scope)){
            String msg = String.format("SCOPE [%s] not managed !", scope);
            //logger.log(Level.SEVERE, msg);
            throw new Exception(msg);
        }
        return scope;
    }
    
    
    /**
     *Imposta la lista delle azioni disponibili 
     *lette dal file di configurazione
     */
    public final void setActionList(String serviceName)throws Exception{
        config.readActions(serviceName);        
    }
        
    
    /**
     *Esegue il metodo actionWork in un thread
     */
    public final void executeAction(ACTIONS_PROCESS actionProcess)throws Exception{
        // esegue soltanto le azioni in stato INITIATED
        if(actionProcess.STATE_ACTION_PROCESS == ACTION_STATE.INITIATED_ACTION_STATE){
            LogWriter.writeLog(Level.INFO, "Exec action : " + TABLE.toString(actionProcess));

            Action actionThread = new Action(actionProcess){
                @Override
                public boolean executeWork(ACTIONS_PROCESS actionP){
                    return actionWork(actionP);
                }
            };     

            // essendo le priorit da 0 a 10 nel sistema assegno priorit MAX al servizio con priorit 0
            actionThread.setPriority(Thread.MAX_PRIORITY - this.getPriorityScope());
            actionProcess.ID_PROCESS_ACTIONS_PROCESS = actionThread.getThreadId();
            actionProcess.ACTION = actionThread;
            actionProcess.STATE_ACTION_PROCESS = ACTION_STATE.RUNNING_ACTION_STATE;

            // update database
            TableManager.updateACTION_PROCESS(actionProcess);

            // avvio del thread
            actionThread.start();        
        }
        else{
            LogWriter.writeLog(Level.WARNING, "Cannot execute ACTION on STATE: " + actionProcess.STATE_ACTION_PROCESS);
        }
    }
   
}