/*
 * DirectiveService.java
 *
 * Created on 2 marzo 2009, 18.09
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package romas.services;

import java.util.Vector;
import romas.ProgramMain;
import romas.com.data.tables.*;
import romas.com.data.TableManager;
import romas.util.LogWriter;
import romas.util.Utility;
import romas.services.ActionService.SERVICE_TYPE;

import eve.database.*;

import java.util.Hashtable;
import java.util.List;
import java.util.logging.*;

/**
 *
 * @author APaliani
 */
public class DirectiveService extends NotificationService{

    private volatile boolean locked = false;

    /** Creates a new instance of DirectiveService
     * @throws Exception
     */
    public DirectiveService() throws Exception{
        
        super("DirectiveService");
        
        // legge le direttive di default dal file di configurazione
        super.config.readDefaultDirectives();
    }

    
    /**
    * Avvia una direttiva rilevandola dal nome
    *
     *@param directiveName nome della direttiva
     * @throws Exception
    */
    public void startDirective(String directiveName)throws Exception
    {
        try{

            while(locked){
                Thread.sleep(50);
            }
            locked = true;

            LogWriter.writeLog(Level.INFO, String.format("Start directive [%s] ...\n", directiveName));
            int i=0;

            EntriesView list = TableManager.searchData(TableManager.dataDIRECTIVES_TABLE, "By Name", directiveName, false);

            // ordinamento per priorit e sequenza
            if((list != null) && (list.size() > 0)){
                EntriesView orderList = TableManager.searchSubData(TableManager.dataDIRECTIVES_TABLE, list, "By Priority", null, false);

                if((orderList != null) && (orderList.size() > 0)){
                    while(i<orderList.size()){
                        DIRECTIVES directive = (DIRECTIVES)list.getData(i);
                        LogWriter.writeLog(Level.FINE, "Start action directive :" + TABLE.toString(directive));
                        loadActionDirective(directive);
                        i++;
                    }
                }
            }
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, "CARICAMENTO DIRETTIVA : "+ directiveName +", FALLITO! " + ex.getMessage());

            Vector params = new Vector();
            params.add("CARICAMENTO DIRETTIVA : "+ directiveName +", FALLITO!");
            romas.ProgramMain.notifyAction("SPEAKER", "SPEAK", "TEXT", params, true);
        }
        finally{
            locked = false;
        }
    }
    
    
    /**
     *Avvia le direttive di default
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    public void startOnRun()throws Exception
    {
        LogWriter.writeLog(Level.INFO, "Start ON RUN - ENABLED directives ... \n");
        
        EntriesView list = TableManager.searchData(TableManager.dataDIRECTIVES_TABLE, "By StartOnRun", true, false);
        
        // notifica l'assenza di directive principali
        if (list == null)
        {
            String msg = super.config.getPropertiesValue("NO_INIT_DIRECTIVES_MESSAGE");
            Vector v_params = new Vector();
            v_params.add(msg);
            ProgramMain.notifyAction("SPEAKER", "SPEAK", "TEXT", v_params, true);
        }
        
        Hashtable hashRunning = new Hashtable();
        for(int i=0; i<list.size(); i++)
        {
            DIRECTIVES directive = (DIRECTIVES)list.getData(i);
            if(directive.ENABLED_DIRECTIVE)
            {
                if(!hashRunning.containsKey(directive.NAME_DIRECTIVE))
                {
                    hashRunning.put(directive.NAME_DIRECTIVE, directive);
                    Vector v_params = new Vector();
                    v_params.add(directive.NAME_DIRECTIVE);
                    // notifica la direttiva da caricare
                    ProgramMain.notifyAction("DIRECTIVE", "LOAD", "DIRECTIVE", v_params, true);
                }
            }            
        }         
    }
    
    
    
    /**
     *Carica la direttiva copiando le azioni della stessa nella ACTION_PROCESS_TABLE
     *
     *@param directive  direttiva da eseguire
     */
    private void loadActionDirective(DIRECTIVES directive)throws Exception
    {

        ACTIONS_PROCESS actionProcess = new ACTIONS_PROCESS(directive.SCOPE_PRIORITY_ACTION_PROCESS, 
                                                            directive.SCOPE_NAME_ACTIONS_PROCESS,
                                                            directive.NAME_ACTION_ACTIONS_PROCESS + "_" + directive.COMMAND_ACTIONS_PROCESS,
                                                            directive.NAME_ACTION_ACTIONS_PROCESS,
                                                            directive.COMMAND_ACTIONS_PROCESS,
                                                            directive.PARAMS_ACTIONS_PROCESS,
                                                            directive.PRIORITY_ACTIONS_PROCESS,
                                                            0,
                                                            ACTION_TYPE.ORDINARY_ACTION_TYPE,
                                                            0,
                                                            ACTION_STATE.CREATED_ACTION_STATE,
                                                            null,
                                                            Utility.getDateTimeID(),
                                                            directive.NUMBER_ITERATION_ACTION_DIRECTIVE
                                                            );
        
        TableManager.addNew(actionProcess, TableManager.dataACTIONS_PROCESS_TABLE);

        LogWriter.writeLog(Level.FINER, "Action loaded: " + TABLE.toString(actionProcess));
    }
    
    public int getPriorityScope() {
        return SERVICE_TYPE.BASIC_SERVICE;
    }

   
    public List<NOTIFICATIONS_MAP> prepareAction() {
        return null;
    }

    protected boolean actionWork(ACTIONS_PROCESS actionProcess)
    {
        try{
            LogWriter.writeLog(Level.FINE, TABLE.toString(actionProcess));

            if(actionProcess.NAME_ACTION_ACTIONS_PROCESS.equals("LOAD")){
                if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("DEFAULT")){
                    // esegue il caricamento delle direttive iniziali
                    startOnRun();
                }

                if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("DIRECTIVE")){
                    // esegue il caricamento delle direttiva richiesta
                    startDirective(actionProcess.PARAMS_ACTIONS_PROCESS.get(0).toString());                
                }
            }        
        }
        catch(Exception ex)
        {
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
            return false;
        }
        return true;
    }
    
    
    public void stop()throws Exception{
        stopService();        
    }

}
