/*
 * Head.java
 *
 * Created on 23 marzo 2009, 13.26
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package romas.services.actuator;

import romas.ProgramMain;
import romas.util.LogWriter;

import java.util.logging.Level;
import romas.util.comunication.controller.servo.Servo;
import romas.util.comunication.controller.servo.ServoConfiguration;
import romas.util.comunication.controller.servo.ServoConfiguration.SERVO_TOWARDS;
import romas.util.comunication.port.Serial;

import romas.services.actuator.Definition.*;
import romas.com.data.tables.ACTIONS_PROCESS;

/**
 *
 * @author APaliani
 */
public class Head implements ILimb{
       
    Servo  v_servoHead;
    Servo  h_servoHead;
    Serial serialPort;
    public boolean isBusy = false;
    
    /** Creates a new instance of Head
     * @param serialPort
     * @param h_configuration
     * @param v_configuration 
     */
    public Head(Serial serialPort, ServoConfiguration h_configuration, ServoConfiguration v_configuration) {
        try
        {
            this.serialPort = serialPort;
            
            h_servoHead = new Servo(serialPort, h_configuration);
            v_servoHead = new Servo(serialPort, v_configuration);
        }
        catch(Exception ex)
        {
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
        }
    }


    public int getRequiredPulse(ACTIONS_PROCESS actionProcess){
        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_LEFT")){
            return (int)h_servoHead.configuration.POS_MIN;
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_RIGHT")){
            return (int)h_servoHead.configuration.POS_MAX;
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_CENTER")){
            return (int)h_servoHead.configuration.POS_MEDIUM;
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_UP")){
            return (int)v_servoHead.configuration.POS_MAX;
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_DOWN")){
            return (int)v_servoHead.configuration.POS_MIN;
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_MIDDLE")){
            return (int)v_servoHead.configuration.POS_MEDIUM;
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_LEFT")){
            return (int)h_servoHead.configuration.POS_MIN;
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_RIGHT")){
            return (int)h_servoHead.configuration.POS_MAX;
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_UP")){
            return (int)v_servoHead.configuration.POS_MAX;
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_DOWN")){
            return (int)v_servoHead.configuration.POS_MIN;
        }

        return 0;
    }


    /**
     * Query servo pulse position
     *
     * @param actionProcess
     * @throws java.lang.Exception
     */
    public void sendQueryPosition(ACTIONS_PROCESS actionProcess)throws Exception{
        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_LEFT")){
            h_servoHead.checkPosition();
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_RIGHT")){
            h_servoHead.checkPosition();
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_CENTER")){
            h_servoHead.checkPosition();
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_UP")){
            v_servoHead.checkPosition();
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_DOWN")){
            v_servoHead.checkPosition();
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_MIDDLE")){
            v_servoHead.checkPosition();
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_LEFT")){
            h_servoHead.checkPosition();
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_RIGHT")){
            h_servoHead.checkPosition();
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_UP")){
            v_servoHead.checkPosition();
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_DOWN")){
            v_servoHead.checkPosition();
        }
    }


    /**
     * Manage head actions
     *
     * @param actionProcess
     * @param steps
     */
    public void actuateAction(ACTIONS_PROCESS actionProcess, int speed, int steps)throws Exception{
        
        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_LEFT")){
            h_servoHead.prepareRotateTo(SERVO_TOWARDS.MIN, speed);
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_RIGHT")){
            h_servoHead.prepareRotateTo(SERVO_TOWARDS.MAX, speed);
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_CENTER")){
            h_servoHead.prepareRotateTo(SERVO_TOWARDS.CENTER, speed);
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_UP")){
            v_servoHead.prepareRotateTo(SERVO_TOWARDS.MAX, speed);
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_DOWN")){
            v_servoHead.prepareRotateTo(SERVO_TOWARDS.MIN, speed);
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("ROTATE_MIDDLE")){
            v_servoHead.prepareRotateTo(SERVO_TOWARDS.CENTER, speed);
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_LEFT")){
            h_servoHead.prepareMoveTo(SERVO_TOWARDS.MIN, speed, steps);
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_RIGHT")){
            h_servoHead.prepareMoveTo(SERVO_TOWARDS.MAX, speed, steps);
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_UP")){
            v_servoHead.prepareMoveTo(SERVO_TOWARDS.MAX, speed, steps);
        }

        if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("MOVE_TO_DOWN")){
            v_servoHead.prepareMoveTo(SERVO_TOWARDS.MIN, speed, steps);
        }
    }

    /**
     * Delete ALL actions for head
     */
    public void stop(){
        try
        {
            // viene notificata la cancellazione di tutte le azioni in corso per la testa
            ProgramMain.notifyAction("ACTUATOR", "HEAD", "CLEAR_ALL", null, true);
        }
        catch(Exception ex)
        {
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
        }
    }

}
