/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package romas.services.speakrecognition;

import java.io.File;
import java.util.logging.Level;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.ArrayList;

import eve.database.EntriesView;

import romas.com.data.tables.*;
import romas.com.data.Configuration;
import romas.services.*;
import romas.util.LogWriter;

/**
 *
 * @author apaliani
 */
public class SRService extends NotificationService{

    Configuration configWordPackages;
    SamplerManager samplerManager;

    EntriesView notifications;
    EntriesView actionNotifications;
    EntriesView commandNotifications;
    EntriesView paramNotifications;
    Hashtable m_hashScopes;
    Hashtable m_hashActions;
    Hashtable m_hashCommands;
    Hashtable m_hashParams;

    int freq_ampl_min = 0;
    int n_dimensions = 0;
    int sample_rate = 0;
    double recognition_threshold = 0;
    int n_packages = 0;
    int freq_min = 0;

    String hmm_Path = "";

    String gate_word = "";
    int gate_wordPackages = 0;

    String run_word = "";
    int run_wordPackages = 0;

    String skip_word = "";
    int skip_wordPackages = 0;

    /** Creates a new instance of SpeakerService */
    public SRService() throws Exception{
        super("SRService");

        hmm_Path = config.getPropertiesValue("hmm_Path");
        configWordPackages = new Configuration();
        configWordPackages.loadPropertiesFile(hmm_Path + File.separator + "WordPackages.properties");

        gate_word = config.getPropertiesValue("gate_word");
        gate_wordPackages = Integer.valueOf(configWordPackages.getPropertiesValue(gate_word));

        run_word = config.getPropertiesValue("run_word");
        run_wordPackages = Integer.valueOf(configWordPackages.getPropertiesValue(run_word));

        skip_word = config.getPropertiesValue("skip_word");
        skip_wordPackages = Integer.valueOf(configWordPackages.getPropertiesValue(skip_word));

        freq_ampl_min = Integer.valueOf(config.getPropertiesValue("freq_ampl_min")).intValue();
        n_dimensions = Integer.valueOf(config.getPropertiesValue("n_dimensions")).intValue();
        sample_rate = Integer.valueOf(config.getPropertiesValue("sample_rate")).intValue();
        recognition_threshold = Double.valueOf(config.getPropertiesValue("recognition_threshold")).doubleValue();
        n_packages = Integer.valueOf(config.getPropertiesValue("n_packages")).intValue();
        freq_min = Integer.valueOf(config.getPropertiesValue("freq_min")).intValue();

        String infoService = "freq_ampl_min [" + freq_ampl_min + "]";
        infoService += ", n_dimensions [" + n_dimensions + "]";
        infoService += ", sample_rate [" + sample_rate + "]";
        infoService += ", recognition_threshold [" + recognition_threshold + "]";
        infoService += ", n_packages [" + n_packages + "]";
        infoService += ", freq_min [" + freq_min + "]";
        infoService += ", freq_ampl_min [" + freq_ampl_min + "]";

        LogWriter.writeLog(Level.INFO, infoService);

        storeScopes();

        samplerManager = new SamplerManager();
    }


    /**
     * Il servizio SR notifica le action da eseguire a tutti i servizi che
     * possono dipendere dal comando vocale.
     * @return
     */
    public List<NOTIFICATIONS_MAP> prepareAction() {
        List<NOTIFICATIONS_MAP> listNotifications;
        try{

            listNotifications = new ArrayList();
            samplerManager.setup();
            samplerManager.initSessionSampler();
            // costruisce l'azione da notificare in relazione alle parole rilevate
            NOTIFICATIONS_MAP notification = new NOTIFICATIONS_MAP();
           
            LogWriter.writeLog(Level.FINE, "Inizio riconoscimento ...");

            double percGateWord = 0;
            // attende gateWord
            while(percGateWord < recognition_threshold){

                LogWriter.writeLog(Level.FINE, "wait gate_word ...");

                MetaTagAudio gateWordTagAudio = new MetaTagAudio();
                gateWordTagAudio = samplerManager.recognizeWord(freq_min, freq_ampl_min, sample_rate, n_packages, n_dimensions);

                // TEST WORD='romas'                
                //gateWordTagAudio.topRangesPackages = gate_wordPackages;
                //gateWordTagAudio.nDimension = n_dimensions;
                //gateWordTagAudio.stateHMM_SeqString = "320.0  360.0  400.0  440.0  480.0  320.0  360.0  400.0  440.0  480.0  320.0  360.0  400.0  440.0  480.0  320.0  360.0  400.0  440.0  480.0  320.0  360.0  400.0  440.0  480.0  320.0  360.0  400.0  440.0  480.0  320.0  360.0  400.0  440.0  480.0  320.0  360.0  400.0  440.0  480.0  320.0  360.0  400.0  440.0  480.0  320.0  360.0  440.0  480.0  520.0  320.0  360.0  440.0  480.0  520.0  320.0  360.0  440.0  520.0  600.0  320.0  360.0  400.0  440.0  960.0  320.0  360.0  400.0  440.0  480.0  320.0  360.0  400.0  440.0  480.0  320.0  360.0  440.0  480.0  520.0  360.0  600.0  640.0  720.0  760.0  720.0  760.0  1480.0  1560.0  0.0  640.0  3800.0  4440.0  4560.0  4600.0  1480.0  1520.0  4240.0  4280.0  4320.0  1440.0  1480.0  3800.0  3840.0  3880.0  3720.0  3880.0  3920.0  4080.0  4200.0  3720.0  3760.0  3840.0  3880.0  3920.0  4000.0  4040.0  4240.0  4280.0  4320.0  1520.0  1640.0  2600.0  4120.0  4160.0  4360.0  4440.0  4560.0  4600.0  4680.0  0.0  0.0  0.0  0.0  0.0  0.0  0.0  0.0  0.0  0.0";
                //gateWordTagAudio.transfer_SeqString_ToDouble();
                
                if(gateWordTagAudio != null){
                    LogWriter.writeLog(Level.FINE, "gate_word : verify packages sampler["+ gateWordTagAudio.topRangesPackages +"] - packages word_gate ["+ gate_wordPackages +"]");
                    if(gateWordTagAudio.topRangesPackages == gate_wordPackages){
                        LogWriter.writeLog(Level.FINE, "gate_word : verify hmm ...");
                        Object[] results = samplerManager.verifyWord(gateWordTagAudio, gate_word, hmm_Path, n_dimensions);
                        if(results[0] != null) percGateWord = Double.valueOf(results[0].toString());
                    }
                }
            }

            // Notifico accesso
            System.out.println("\007\007\007");

            // scope
            String scope = "";
            int loop = 0;
            while(scope.equals("") && !m_hashScopes.containsKey(scope)){
                if(loop > 10)return null;
                LogWriter.writeLog(Level.FINE, "wait scope ...");
                MetaTagAudio scopeMetaTagAudio = samplerManager.recognizeWord(freq_min, freq_ampl_min, sample_rate, n_packages, n_dimensions);
                if(scopeMetaTagAudio != null){
                    scope = processWordList(scopeMetaTagAudio, m_hashScopes);
                }
                loop++;
            }

            loop = 0;
            if(!scope.equals(skip_word)){
                
                // notifica riconoscimento
                System.out.println("\007");

                notification.SUBSCRIBER = m_hashScopes.get(scope.toUpperCase()).toString();

                // action
                storeActions(notification.SUBSCRIBER);
            
                String action = "";
                while(action.equals("") && !m_hashActions.contains(action)){
                    if(loop > 10)return null;
                    LogWriter.writeLog(Level.FINE, "wait action ...");
                    MetaTagAudio actionMetaTagAudio = samplerManager.recognizeWord(freq_min, freq_ampl_min, sample_rate, n_packages, n_dimensions);
                    action = processWordList(actionMetaTagAudio, m_hashActions);
                    loop++;
                }

                if(!action.equals(skip_word)){
                    // notifica riconoscimento
                    System.out.println("\007");

                    notification.ACTION_EVENT = action.toUpperCase();

                    // command
                    storeCommands(notification.SUBSCRIBER, notification.ACTION_EVENT);

                    String command = "";
                    loop = 0;
                    while(command.equals("") && !m_hashCommands.contains(command)){
                        if(loop > 10)return null;
                        LogWriter.writeLog(Level.INFO, "wait command ...");
                        MetaTagAudio commandMetaTagAudio = samplerManager.recognizeWord(freq_min, freq_ampl_min, sample_rate, n_packages, n_dimensions);
                        command = processWordList(commandMetaTagAudio, m_hashCommands);
                        loop++;
                    }

                    if(!command.equals(skip_word)){
                        // notifica riconoscimento
                        System.out.println("\007");

                        notification.COMMAND_EVENT = command.toUpperCase();

                        // params
                        storeParams(notification.SUBSCRIBER, notification.ACTION_EVENT, notification.COMMAND_EVENT);

                        loop = 0;
                        String param = "";
                        do{
                            LogWriter.writeLog(Level.INFO, "wait param ...");
                            MetaTagAudio paramMetaTagAudio = samplerManager.recognizeWord(freq_min, freq_ampl_min, sample_rate, n_packages, n_dimensions);
                            param = processWordList(paramMetaTagAudio, m_hashParams);
                            if(!param.equals("") && !param.equals(skip_word) && !param.equals(run_word)){
                                notification.PARAMS_COMMAND_EVENT.add(param);
                                loop = 0;
                            }
                            loop++;
                            if(loop > 10)return null;
                        } // aggiunge i parametri richiesti
                        while(param.equals("") && !param.equals(skip_word) && !param.equals(run_word));

                        // verifica comando di esecuzione
                        if(param.equals(run_word)){
                            LogWriter.writeLog(Level.INFO, "RUN ...");
                            // notifica esecuzione comando
                            System.out.println("\007\007");
                            listNotifications.add(notification);
                            return listNotifications;
                        }
                    }
                }
                LogWriter.writeLog(Level.WARNING, "No action retrive !");
            }
            LogWriter.writeLog(Level.WARNING, "No scope retrive !");
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
        }
        finally{
            samplerManager.stopSessionSampler();
            System.out.println("\007\007\007\007\007\007");
        }
        
        return null;
    }



    /**
     * Verifica quale scope  stato richiesto
     * @return
     */
    public String processWordList(MetaTagAudio samplerAudio, Hashtable hashWordList){
        Enumeration e = hashWordList.keys();
        String wordRecognized = "";
        double pR = 0;
        while( e. hasMoreElements() ){
            String word = e.nextElement().toString().toLowerCase();
            try{
                // Verifica se la parola  confrontabile con quella campionata
                int wordPackages = Integer.valueOf(configWordPackages.getPropertiesValue(word));
                if(wordPackages == samplerAudio.topRangesPackages){
                    Object[] results = samplerManager.verifyWord(samplerAudio, word, hmm_Path, n_dimensions);

                    double percentueale = 0;
                    if(results[0] != null) percentueale = Double.valueOf(results[0].toString());

                    if(percentueale > pR){
                        pR = percentueale;
                        wordRecognized = word;
                    }
                }
            }
            catch(Exception ex){
                LogWriter.writeLog(Level.SEVERE, "Errore in esame parola ["+ word +"]: " + ex.getMessage());
            }
        }

        if(!wordRecognized.equals("")) LogWriter.writeLog(Level.INFO,"Riconoscita parola [ " + wordRecognized + "]");

        return wordRecognized;
    }





    /**
     * Memorizza gli scope che hanno una sottoscrizione al servizio SR
     */
    private void storeScopes()throws Exception{
        m_hashScopes = new Hashtable();
        notifications =  this.getNotifications();
        if((notifications != null) && (notifications.size() > 0)){
            for(int i=0; i < notifications.size(); i++){
                NOTIFICATIONS_MAP notify = (NOTIFICATIONS_MAP)notifications.get(i).getData();
                // solo per sottoscrizioni attive e scope non presenti
                String local_scope = config.getPropertiesValue(notify.SUBSCRIBER);
                if(notify.ACTIVE && (!m_hashScopes.containsKey(local_scope))) {
                    m_hashScopes.put(local_scope, notify.SUBSCRIBER);
                    LogWriter.writeLog(Level.INFO, "SCOPE ["+ local_scope +"]["+ notify.SUBSCRIBER +"]");
                }
            }
        }

        if(!m_hashScopes.contains(skip_word)) m_hashScopes.put(skip_word, skip_word);
    }


    private void storeActions(String scope)throws Exception{
        m_hashActions = new Hashtable();
        if((notifications != null) && (notifications.size() > 0)){
            for(int i=0; i < notifications.size(); i++){
                NOTIFICATIONS_MAP notify = (NOTIFICATIONS_MAP)notifications.get(i).getData();
                // solo per sottoscrizioni attive e azioni non presenti
                if(notify.ACTIVE && (!m_hashActions.contains(notify.ACTION_EVENT)) && notify.SUBSCRIBER.equals(scope)) {
                    m_hashActions.put(notify.ACTION_EVENT, notify.ACTION_EVENT);
                    LogWriter.writeLog(Level.INFO, "Action [" + notify.ACTION_EVENT + "] for SCOPE ["+ scope +"]");
                }
            }
        }
        //addGenericWord(m_hashActions);
    }



    private void storeCommands(String scope, String action)throws Exception{
        m_hashCommands = new Hashtable();
        if((notifications != null) && (notifications.size() > 0)){
            for(int i=0; i < notifications.size(); i++){
                NOTIFICATIONS_MAP notify = (NOTIFICATIONS_MAP)notifications.get(i).getData();
                // solo per sottoscrizioni attive e comandi non presenti
                if(notify.ACTIVE && (!m_hashCommands.contains(notify.COMMAND_EVENT)) && notify.SUBSCRIBER.equals(scope) && notify.ACTION_EVENT.equals(action)) {
                    m_hashCommands.put(notify.COMMAND_EVENT, notify.COMMAND_EVENT);
                    LogWriter.writeLog(Level.INFO, "Command [" + notify.COMMAND_EVENT + "] for Action ["+ action +"]");
                }
            }
        }
        if(!m_hashCommands.contains(skip_word)) m_hashCommands.put(skip_word, skip_word);
        //addGenericWord(m_hashCommands);
    }


    private void storeParams(String scope, String action, String command)throws Exception{
        m_hashParams = new Hashtable();
        if((notifications != null) && (notifications.size() > 0)){
            for(int i=0; i < notifications.size(); i++){
                NOTIFICATIONS_MAP notify = (NOTIFICATIONS_MAP)notifications.get(i).getData();
                // solo per sottoscrizioni attive e comandi non presenti
                if(notify.ACTIVE && (!m_hashParams.contains(notify.COMMAND_EVENT)) && notify.SUBSCRIBER.equals(scope) && notify.ACTION_EVENT.equals(action) && notify.PARAMS_COMMAND_EVENT.equals(command)) {
                    m_hashParams.put(notify.COMMAND_EVENT, notify.COMMAND_EVENT);
                    LogWriter.writeLog(Level.INFO, "Command [" + notify.COMMAND_EVENT + "] for Action ["+ action +"]");
                }
            }
        }

        if(!m_hashParams.contains(run_word)) m_hashParams.put(run_word, run_word);
        if(!m_hashParams.contains(skip_word)) m_hashParams.put(skip_word, skip_word);
    }


    private void addGenericWord(Hashtable hashWordList){
        if(!hashWordList.contains(gate_word)) hashWordList.put(gate_word, gate_word);
        if(!hashWordList.contains(run_word)) hashWordList.put(run_word, run_word);
        if(!hashWordList.contains(skip_word)) hashWordList.put(skip_word, skip_word);
    }



    /**
     *Execute work
     */
    protected boolean actionWork(ACTIONS_PROCESS actionProcess) {
        LogWriter.writeLog(Level.FINE, TABLE.toString(actionProcess));

        if(actionProcess == null) return false;

        return true;
    }


    public int getPriorityScope() {
        return SERVICE_TYPE.PRIMARY_SERVICE;
    }

}
