/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package romas.services.speakrecognition;

import romas.com.data.hmm.*;
import java.util.Vector;
import java.io.File;

import java.util.logging.Level;
import romas.util.LogWriter;

/**
 *
 * @author apaliani
 */
public class SamplerManager {

    Vector vValori = null;
    private Sampler sampler;

    public Vector getSamplers(){
        return vValori;
    }

    public int countSamplers(){
        return vValori.size();
    }

    public SamplerManager(){
    }

    public void setup(){
        sampler = new Sampler();
        sampler.setup();
    }


    /**
     * Avvia una sessione di campionamenti
     */
    public void initSessionSampler(){
        sampler.initSessionSampler();
    }


    /**
     * Arreesta una sessione di campionamento
     */
    public void stopSessionSampler(){
        sampler.stopSessionSampler();
    }


    public void dispose(){
        sampler = null;
    }

    /**
     *
     * @param samplerMetaAudio
     */
    public void setDataVector(MetaTagAudio samplerMetaAudio){
        if(vValori == null){
            vValori = new Vector();
        }
        vValori.add(samplerMetaAudio);
    }

    public void removeSampler(MetaTagAudio samplerMetaAudio){
        vValori.remove(samplerMetaAudio);
    }


    /**
     *
     * Effettua il confronto tra la sequenza ed il modello in esame
     *
     * @param word
     * @param pathModels
     * @param nDimensions
     * @param freqMin
     * @param freqAmpliMin
     * @param nSampleRate
     * @param nPackages
     * @param semaforo_jTextField
     * @return Object[4] = 0:double percentueale; 1:MetaTagAudio samplerAudio; 2:double percentueale_viterbi; 3:double distance
     */
    public Object[] recognize(String word, String pathModels, int nDimensions, int freqMin, int freqAmpliMin, int nSampleRate, int nPackages, int maxPackages, javax.swing.JTextField semaforo_jTextField){
        MetaTagAudio samplerAudio = sampler.execute(word, freqMin, freqAmpliMin, nSampleRate, nPackages, maxPackages, nDimensions, semaforo_jTextField);
        return verifyWord(samplerAudio, word, pathModels, nDimensions);
    }


    public Object[] verifyWord(MetaTagAudio samplerAudio, String word, String pathModels, int nDimensions){
        // init result
        Object[] results = new Object[4];
        results[0] = 0;
        results[1] = new MetaTagAudio();
        results[2] = 0;
        results[3] = -1;
        try{
            if (samplerAudio == null) return results;

            results[1] = samplerAudio;

            String sequenceHMM = samplerAudio.stateHMM_ToString();
            LogWriter.writeLog(Level.FINE, "Sequenza: " + sequenceHMM + "\n");

            // for match
            String fileHMM = pathModels + File.separator + word + ".hmm";
            HMM hmm = HMM.load(fileHMM);

            double[][] seq = samplerAudio.stateHMM_ToSequence();

            double perc = HMM_Manager.getProbability(hmm, nDimensions, samplerAudio.topRangesPackages, seq);

            double perc_viterbi = -1;//al momento non utilizzo: HMM_Manager.getViterbi(hmm, nDimensions, samplerAudio.topRangesPackages, seq);

            LogWriter.writeLog(Level.FINE, "\nPercentuale di riconoscimento: viterbi[" + perc_viterbi + "] - seq ["+ perc +"] %\n");

            long percentueale = Math.round(perc);

            long percentueale_viterbi = Math.round(perc_viterbi);

            results[0] = percentueale;

            results[2] = percentueale_viterbi;

            results[3] = -1;//al momento non utilizzo: HMM_Manager.getDistance(fileHMM, pathModels, nDimensions, samplerAudio.topRangesPackages, seq);

            return results;
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
            return results;
        }
    }



    public void setSamplerToFile(String fileTrainingName, MetaTagAudio samplerMetaTag, int nDimensions, int nSeq){
        try{
            String sequenceHMM = samplerMetaTag.stateHMM_ToString();
            LogWriter.writeLog(Level.FINE, "Sequenza: " + sequenceHMM + "\n");
            //For training
            String fileTraining = fileTrainingName + ".training";

            // le dimensioni sono moltiplicate per due in quanto viene rilevata anche l'ampiezza della frequenza
            HMM_Manager.addSequenceTraining(fileTraining, sequenceHMM, nDimensions, nSeq, samplerMetaTag.topRangesPackages);
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
        }        
    }
    
    
    public MetaTagAudio executeSampler(String word, int freqMin, int freqAmpliMin, int nSampleRate, int nPackages, int nMaxPackages, int nDimension, javax.swing.JTextField semaforo_jTextField){
        MetaTagAudio samplerAudio = sampler.execute(word, freqMin, freqAmpliMin, nSampleRate, nPackages, nMaxPackages, nDimension, semaforo_jTextField);
        return samplerAudio;    
    }


    public MetaTagAudio recognizeWord(int freqMin, int freqAmpliMin, int nSampleRate, int nPackages, int nDimension){

        return sampler.execute(0, freqMin, freqAmpliMin, 0, nSampleRate, nPackages, nPackages, nDimension, new javax.swing.JTextField());
    }

}
