/*
 * FrameAccess.java
 *
 * Created on 24 febbraio 2009, 10.57
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package romas.services.videostream;

/**
 *
 * @author APaliani
 */
import java.awt.*;
import javax.swing.JFrame;
import java.io.*;
import java.awt.image.*;
import com.sun.image.codec.jpeg.*;
import javax.media.protocol.*;
import javax.media.control.*;
import javax.media.*;
import javax.media.Format;
import javax.media.format.*;
import java.util.logging.Level;
import romas.util.LogWriter;

/**
 * Sample program to access individual video frames by using a 
 * "pass-thru" codec.  The codec is inserted into the data flow
 * path.  As data pass through this codec, a callback is invoked
 * for each frame of video data.
 */
public class FrameAccess extends JFrame implements ControllerListener {

    public Processor p;
    Object waitSync = new Object();
    boolean stateTransitionOK = true;
    Buffer myFrame = null;
    VideoFormat myFormat = null;

    /**
     * Given a media locator, create a processor and use that processor
     * as a player to playback the media.
     *
     * During the processor's Configured state, two "pass-thru" codecs,
     * PreAccessCodec and PostAccessCodec, are set on the video track.  
     * These codecs are used to get access to individual video frames 
     * of the media.
     *
     * Much of the code is just standard code to present media in JMF.
     */
    
    public FrameAccess(VideoFormat myFormat)
    {
        this.myFormat = myFormat;
    }
    
    public void open(MediaLocator ml) {
        
	try {
	    p = Manager.createProcessor(ml);            
	} catch (Exception e) 
    {
        LogWriter.writeLog(Level.SEVERE, "Failed to create a processor from the given url: " + e);
	}

	p.addControllerListener(this);

	// Put the Processor into configured state.
	p.configure();
	if (!waitForState(p.Configured)) {
        LogWriter.writeLog(Level.SEVERE, "Failed to configure the processor !");
	}

	// So I can use it as a player.
	p.setContentDescriptor(null);
        
	// Obtain the track controls.
	TrackControl tc[] = p.getTrackControls();

	if (tc == null) {
	    LogWriter.writeLog(Level.SEVERE, "Failed to obtain track controls from the processor.");
	}

	// Search for the track control for the video track.
	TrackControl videoTrack = null;

	for (int i = 0; i < tc.length; i++) {
	    if (tc[i].getFormat() instanceof VideoFormat) {
		videoTrack = tc[i];
		break;
	    }
	}

	if (videoTrack == null) {
        LogWriter.writeLog(Level.SEVERE, "The input media does not contain a video track!");
	}

    LogWriter.writeLog(Level.INFO, "Video format: " + videoTrack.getFormat());

	// Instantiate and set the frame access codec to the data flow path.
	try {
	    Codec codec[] = { new PreAccessCodec(),
				new PostAccessCodec()};
            
	    videoTrack.setCodecChain(codec);
	} catch (UnsupportedPlugInException e) {
	    LogWriter.writeLog(Level.SEVERE, "The process does not support effects.");
	}

	// Realize the processor.
	p.prefetch();
	if (!waitForState(p.Prefetched)) {
        LogWriter.writeLog(Level.SEVERE, "Failed to realize the processor.");
	}

	// Display the visual & control component if there's one.        
	this.getContentPane().setLayout(new BorderLayout());

	Component cc;

	Component vc;
	if ((vc = p.getVisualComponent()) != null) {
	    this.getContentPane().add("Center", vc);
	}

	if ((cc = p.getControlPanelComponent()) != null) {
	    this.getContentPane().add("South", cc);
	}

	// Start the processor.
	p.start();

	setVisible(true);
    }

    public void addNotify() {
	super.addNotify();
	pack();
    }
    
    private DataSource getDataSource(MediaLocator ml)
            throws IOException, NoDataSourceException {
            return Manager.createDataSource(ml);
    }
    
    private DataSource setDataSourceFormat(DataSource ds, VideoFormat myFormat)
            throws Exception {

            CaptureDevice cd = (CaptureDevice) ds;
            FormatControl[] fcs = cd.getFormatControls();
            Format setFormat = null;
            for (int i = 0; i < fcs.length; i++) {
                    setFormat = fcs[i].setFormat(myFormat);
            }

            if (setFormat == null)
                    throw new Exception("Could not set Control format");

            return ds;
    }
    
    

    /**
     * Block until the processor has transitioned to the given state.
     * Return false if the transition failed.
     */
    boolean waitForState(int state) {
	synchronized (waitSync) {
	    try {
		while (p.getState() != state && stateTransitionOK)
		    waitSync.wait();
	    } catch (Exception e) {}
	}
	return stateTransitionOK;
    }


    /**
     * Controller Listener.
     */
    public void controllerUpdate(ControllerEvent evt) {

	if (evt instanceof ConfigureCompleteEvent ||
	    evt instanceof RealizeCompleteEvent ||
	    evt instanceof PrefetchCompleteEvent) {
	    synchronized (waitSync) {
		stateTransitionOK = true;
		waitSync.notifyAll();
	    }
	} else if (evt instanceof ResourceUnavailableEvent) {
	    synchronized (waitSync) {
		stateTransitionOK = false;
		waitSync.notifyAll();
	    }
	} else if (evt instanceof EndOfMediaEvent) {
	    p.close();
	    System.exit(0);
	}
    }



    /**
     * Main program

    public static void main(String [] args) {

	if (args.length == 0) {
	    prUsage();
	    System.exit(0);
 	}

	String url = args[0];

	if (url.indexOf(":") < 0) {
	    prUsage();
	    System.exit(0);
	}

	MediaLocator ml;

	if ((ml = new MediaLocator(url)) == null) {
	    System.err.println("Cannot build media locator from: " + url);
	    System.exit(0);
	}

	FrameAccess fa = new FrameAccess();

	if (!fa.open(ml))
	    System.exit(0);
    }
     */
    
    static void prUsage() {
        LogWriter.writeLog(Level.SEVERE, "Usage: java FrameAccess <url>");
    }

    
    public Image getBufferImage()
    {
        return ManageBufferImage.bufferToImage(myFrame);
    }
    
    public javax.media.Buffer getBuffer()
    {
        return myFrame;
    }
    
    /** Send image data. */
    public void sendImage(OutputStream outputStream) 
    {     
        try{
            Image image = getBufferImage();

            //*****Send the Image*****
            JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(outputStream);
            JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam((BufferedImage)image);
            param.setQuality(0.9f, false); // 0.75f = height qualith, 0.5f medium, 0.25f low
            encoder.setJPEGEncodeParam(param);
            encoder.encode((BufferedImage)image);
            
        }
        catch(Exception e){}
    }     


    /*********************************************************
     * Inner class.
     *
     * A pass-through codec to access to individual frames.
     *********************************************************/

    public class PreAccessCodec implements Codec {

	/**
         * Callback to access individual video frames.
         */
	void accessFrame(Buffer frame) {

	    // For demo, we'll just print out the frame #, time &
	    // data length.

	    //long t = (long)(frame.getTimeStamp()/10000000f);

	    //System.err.println("Pre: frame #: " + frame.getSequenceNumber() + ", time: " + ((float)t)/100f + 	", len: " + frame.getLength());
            //myFrame = frame;
        }


	/**
 	 * The code for a pass through codec.
	 */

	// We'll advertize as supporting all video formats.
	protected Format supportedIns[] = new Format [] {
	    new VideoFormat(null)
	};

	// We'll advertize as supporting all video formats.
	protected Format supportedOuts[] = new Format [] {
	    new VideoFormat(null)
	};

	Format input = null, output = null;

	public String getName() {
	    return "Pre-Access Codec";
	}

	// No op.
        public void open() {
	}

	// No op.
	public void close() {
	}

	// No op.
	public void reset() {
	}

	public Format [] getSupportedInputFormats() {
	    return supportedIns;
	}

	public Format [] getSupportedOutputFormats(Format in) {
	    if (in == null)
		return supportedOuts;
	    else {
		// If an input format is given, we use that input format
		// as the output since we are not modifying the bit stream
		// at all.
                
		Format outs[] = new Format[1];
		outs[0] = in;
		return outs;
	    }
	}

	public Format setInputFormat(Format format) {
	    input = format;
	    return input;
	}

	public Format setOutputFormat(Format format) {
	    output = format;
	    return output;
	}

	public int process(Buffer in, Buffer out) {

	    // This is the "Callback" to access individual frames.
	    accessFrame(in);

	    // Swap the data between the input & output.
	    Object data = in.getData();
	    in.setData(out.getData());
	    out.setData(data);

	    // Copy the input attributes to the output
	    out.setFormat(in.getFormat());
	    out.setLength(in.getLength());
	    out.setOffset(in.getOffset());

	    return BUFFER_PROCESSED_OK;
	}

	public Object[] getControls() {
	    return new Object[0];
	}

	public Object getControl(String type) {
	    return null;
	}
    }

    public class PostAccessCodec extends PreAccessCodec {
        
        //int frameRate = 25;
        //int frameNumb = 0;
        
	// We'll advertize as supporting all video formats.
	public PostAccessCodec() {
	    supportedIns = new Format [] {
		new RGBFormat()
	    };
	}

	/**
         * Callback to access individual video frames.
         */
	void accessFrame(Buffer frame) {

	    // For demo, we'll just print out the frame #, time &
	    // data length.

	    //long t = (long)(frame.getTimeStamp()/10000000f);

	    //System.err.println("Post: frame #: " + frame.getSequenceNumber() + ", time: " + ((float)t)/100f + ", len: " + frame.getLength());
            
            //if (frameRate == frameNumb) frameNumb = 0; 

            //frameNumb += 1; 
            myFrame = frame;
            //ManageBufferImage.convertAndSaveFile(frame, "C:\\EasyPHP1-8\\www\\ErpWeb\\stream\\webcamera");
	}

	public String getName() {
	    return "Post-Access Codec";
	}
    }
}

    
