/*
 * VideoStreamServer.java
 *
 * Created on 23 febbraio 2009, 17.07
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package romas.services.videostream;


import java.net.*;
import javax.media.*;
import javax.media.format.*;
import java.util.Vector;

import romas.services.*;
import romas.com.data.tables.TABLE;
import romas.com.data.tables.NOTIFICATIONS_MAP;
import romas.com.data.tables.ACTIONS_PROCESS;

import java.util.logging.Level;
import java.util.List;

import romas.util.LogWriter;
/**
 *
 * @author APaliani
 */
public class VideoStreamService extends NotificationService{

    ServerSocket serverSocket;
    public static boolean done = false;
    FrameAccess frameAccess;
    Vector deviceList;
    int m_serverPort = 0;
    
    /** Creates a new instance of VideoStreamServer */
    public VideoStreamService() throws Exception{
        super("VideoStreamService");
    }
    
    
    public int getPriorityScope()
    {
        return SERVICE_TYPE.PRIMARY_SERVICE;
    }
    
    
    public List<NOTIFICATIONS_MAP> prepareAction()
    {
        return null;
    }
     

    /**
     *Da implementare per eseguire le azioni
     */
    protected boolean actionWork(ACTIONS_PROCESS actionProcess) 
    {    
        LogWriter.writeLog(Level.FINE, TABLE.toString(actionProcess));
        
        if(actionProcess.NAME_ACTION_ACTIONS_PROCESS.equals("SERVER"))
        {                   
            try{
                if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("START"))
                {
                    int serverPort = Integer.valueOf(actionProcess.PARAMS_ACTIONS_PROCESS.get(0).toString()).intValue();
                    startServer(serverPort);
                }

                if(actionProcess.COMMAND_ACTIONS_PROCESS.equals("STOP"))
                {
                    stopServer();
                }
                
            }catch(Exception ex){
                
                LogWriter.writeLog(Level.SEVERE, ex.getMessage());
                return false;
            }            
        }       
        
        return true;
    }
    
    
    /**
     *Avvia il server
     */
    public void startServer(int serverPort)throws Exception{
        
        try{
            m_serverPort = serverPort;
            serverSocket = new ServerSocket(serverPort);
           
            
            LogWriter.writeLog(Level.INFO, "Videostreaming server started on host:"+ serverSocket.getInetAddress() +" port ("+ serverPort + ") ... \n");
            
            if(buildDevices()){
                clientListener();
            }

            close();            
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
            close();
        }
    }

    private void close()throws Exception{
        serverSocket.close();

        LogWriter.writeLog(Level.INFO, "VideoStreamingService STOP !");

        Vector params = new Vector();
        params.add("SERVIZIO VISIVO : ARRESTATO");
        romas.ProgramMain.notifyAction("SPEAKER", "SPEAK", "TEXT", params, true);
    }

    /**
     *Attende la connessione dei clients
     */
    private void clientListener()throws Exception
    {
        while (!done){
            LogWriter.writeLog(Level.INFO, "Server listening ... ");
            Socket clientSocket = serverSocket.accept();
            String msg = clientSocket.getInetAddress() + " : " + clientSocket.getPort();
            LogWriter.writeLog(Level.INFO, "Connected client ... " +  msg);

//            Vector params = new Vector();
//            params.add(msg);
//            romas.ProgramMain.notifyAction("VIDEO", "CLIENT_CONNECTION", "OPEN", params, true);

            ClientStreamSocket clientStreamSocket = new ClientStreamSocket(clientSocket, this);
            clientStreamSocket.start();
        }
    }
    
    
    /**
     *Arresta il server
     */
    public void stopServer()
    {
        try{
            LogWriter.writeLog(Level.INFO, "VideoStreamingService STOP !");
            serverSocket.close();
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
        }
    }
    
    /**
     *Ritorna la lista dei dispositivi
     */
    public java.util.Vector DeviceList(){      
        return deviceList;
    }
    
    
    /**
     *Rileval il numero dei dispositivi connessi
     */
    public String getDeviceNumber(){
        String sNumDevices = String.valueOf(deviceList.size());
        if (deviceList.size() < 10){
            sNumDevices = "0" + String.valueOf(deviceList.size());
        } 
        return sNumDevices;
    }
    
    
    /**
     *Rileva i dispositivi connessi al server
     */
    private boolean buildDevices(){
        
         //VideoFormat format = new VideoFormat(VideoFormat.RGB, new Dimension(176, 144), null, 15);

         VideoFormat format = new VideoFormat(VideoFormat.RGB);
         Vector devices = CaptureDeviceManager.getDeviceList(format);
 
         if (devices.size() > 0) {
             
            deviceList = new Vector();
            
            for(int i=0; i < devices.size(); i++){
                
                Device device = new Device();

                try{                       
                    
                    CaptureDeviceInfo di = (CaptureDeviceInfo) devices.elementAt(i);                    
                    
                    device.name = di.getName();
                    if (!(device.name.indexOf("Microsoft") > 0))
                    {
                        device.frameAccess = new FrameAccess(format);
                        device.frameAccess.open(di.getLocator());

                        deviceList.add(device);
                    }

                    LogWriter.writeLog(Level.INFO, "Recognized " + deviceList.size() + " devices ...\n");

                }catch(Exception ex){
                    LogWriter.writeLog(Level.SEVERE, "Could not connect to capture device  ["+ device.name + "] : " + ex.getMessage());
                    return false;
                }
            }
              
         }
         else {
             // exit if we could not find the relevant capturedevice. 
             LogWriter.writeLog(Level.WARNING, "No devices connected !");
             return false;
         }

         return true;
    }
}
