package romas.mobile;

import java.util.Hashtable;
import eve.sys.*;
import eve.ui.*;
import eve.ui.data.*;
import eve.ui.event.DataChangeEvent;
import eve.ui.event.MenuEvent;
import eve.ui.MultiPanel;

import romas.mobile.video.images.BannerSlider;
import romas.mobile.video.ui.VideoScreen;
import romasBridge.ComunicationClient;


public class RomasPDA extends AppForm{

    public BannerSlider bs; 
    public VideoScreen displayVideo;
    private ComunicationClient comunicationClient;
           
    public int Screen_W;
    public int Screen_H;
    
    
    private Button buttonPlayCommand;
    private Button buttonStopCommand;
    private Button buttonSaveScreenShot;
    private Button buttonRecord;
    
    private Choice webcamChoice;

    private Panel displayPanel;
    private PanelConnection connectionPanel;
    private PanelActuator actuatorPanel;
    private PanelSpeaker speakerPanel;
    private Panel cameraPanel;
    private Hashtable hashAction;
    private String listCameraEmpty = "...";    
    private static String action;

    private ButtonBar buttonVideoBar;

    private MultiPanel mp;
    
    public RomasPDA(){        
        initGUI();
        maximizeOnPDA();
    }
    
    
    private void initGUI(){       
        // Adding menu
        eve.ui.GuiStyle.popupBeep =  eve.fx.Sound.BEEP_NONE;
        
        hashAction = new Hashtable();

        hashAction.put("actPlay", "actPlay");
        hashAction.put("actStop", "actStop");
        hashAction.put("actRecordStart", "actRecordStart");
        hashAction.put("actRecordStop", "actRecordStop");
        hashAction.put("actScreenShot", "actScreenShot");
        
        title = "Ro.Ma.S. mobile - alpha 1.0";
        
        buttonPlayCommand = new Button("Play");
        buttonPlayCommand.setAction("actPlay");

        buttonSaveScreenShot = new Button("Save screen shot");
        buttonSaveScreenShot.setAction("actScreenShot");

        buttonStopCommand = new Button("Stop");
        buttonStopCommand.setAction("actStop");
        
        buttonRecord = new Button("Recorder");
        buttonRecord.setAction("actRecordStart");
        
        // disable menu        
        buttonPlayCommand.modify(Control.Disabled,0);
        buttonSaveScreenShot.modify(Control.Disabled,0);
        buttonStopCommand.modify(Control.Disabled,0);
        buttonRecord.modify(Control.Disabled,0);
        
        buttonVideoBar = new ButtonBar();
        buttonVideoBar.addNext(buttonPlayCommand);
        buttonVideoBar.addNext(buttonSaveScreenShot);
        buttonVideoBar.addNext(buttonRecord);
        buttonVideoBar.addNext(buttonStopCommand);
        
   		TabbedPanel tp = new TabbedPanel();
		mp = tp;

        connectionPanel = new PanelConnection("Settings", hashAction);
        tp.addItem(connectionPanel, "Connection", "Connection");

        // pannello selezione web camera
        cameraPanel = new Panel();
        cameraPanel.setText("Videocamera");
        webcamChoice = new Choice(new String[]{listCameraEmpty}, 0);
        cameraPanel.addLast(webcamChoice,Form.DONTSTRETCH,Form.FILL);
        cameraPanel.addNext(buttonVideoBar,Form.STRETCH,Form.FILL);

        tp.addItem(cameraPanel, "Video", "Video");

        // Movement
        tp.addItem(new PanelMovement("Commands", hashAction), "Movement", "Movement");

        // Elevator
        tp.addItem(new PanelElevator("Commands", hashAction), "Elevator", "Elevator");

        // Actuator
        actuatorPanel = new PanelActuator("Commands", hashAction);
        tp.addItem(actuatorPanel, "Actuator", "Actuator");


        // Speaker
        speakerPanel = new PanelSpeaker("Speak", hashAction);
        tp.addItem(speakerPanel, "Speaker", "Speaker");
        
        addLast(tp, Form.DONTSTRETCH,Form.FILL);

        // pannello immagine
        displayPanel = new Panel();
        displayPanel.setText("Display imagine");
        displayVideo = new VideoScreen();
        displayPanel.addNext(displayVideo,Form.STRETCH,Form.FILL);

        ScrollBarPanel scrolPanel = new ScrollBarPanel(displayPanel);

        addLast(scrolPanel,Form.STRETCH,Form.FILL);
    }    

    @Override
    public boolean handleAction(String sAction){                      
        
        if(hashAction.containsKey(sAction))
        {
            action = sAction;        

            Handle h = doWorkInThread();

            ProgressBarForm pbf = new ProgressBarForm();
            pbf.title = "Loading ...";
            pbf.maximizeOnPDA();
            pbf.setMainTask(action);
            pbf.showStop = true;
            pbf.horizontalLayout = false; //Put cancel below the bar.
            
            Object got = null;

            if(sAction.equals("actReconnect") || sAction.equals("actConnect") || sAction.equals("actPlay") || sAction.equals("actRecordStart") || sAction.equals("actRecordStop"))
            {
                h.start();
                got = "OK";
            }
            else
            {
                got = pbf.executeAndGetReturnValue(h);
            }
            
            if (got == null){
                    if (h.error != null)
                            new ReportException(h.error).execute();
                    else
                            new MessageBox("Abort action","action abort!",MessageBox.MBOK).execute();
            }else{
                    //new MessageBox("Staus","Processing: " + got, MessageBox.MBOK).execute();
            }
            return true;
        }
        
        return super.handleAction(action);
    }

    @Override
    public void onEvent(Event ev){        
        //new MessageBox("Evento","Evento!" + String.valueOf(ev.type), MessageBox.MBOK).execute();
        switch(ev.type)
        {            
            case DataChangeEvent.DATA_CHANGED:               
                if(webcamChoice.selectedIndex > 0)
                {
                    String item = webcamChoice.items.get(webcamChoice.selectedIndex).toString();
                    title = "Video streaming - " + item;
                    handleAction("actPlay");
                }
                break;
                
            //case MenuEvent.CLICKED:
            case MenuEvent.SELECTED:                
                super.onEvent(ev);
                break;
                
            default:              
                super.onEvent(ev); //Make sure you call this.
                break;
        }       
    }
    
    public Handle doWorkInThread(){
        return new Task(){
                protected void doRun(){
                        try{                                    
                                String got = doWork(this);
                                // If doSomething() returned null, then
                                // the stop() method was called on this task
                                // which set the shouldStop field to true.
                                if (got == null) set(Aborted|Stopped);
                                else succeed(got);
                        }catch(Exception e){
                                fail(e);
                        }
                }
        };
    }
       
    public String doWork(Handle h)throws Exception{
	if (h == null) h = new Handle();
        
        int max = 10;

        h.startDoing("Loading ...");
        for (int i = 0; !h.shouldStop && i<max; i++){
             try{Thread.sleep(5);}catch(InterruptedException e){}
             h.setProgress((float)(i+1)/max);
        }        
        
        if (action.equals("actExit") )  
        { 
            Util.storePropertiesFile();
            Application.exit(0);
        }else             

        if (action.equals("actConnect") ) 
        {                
            connect();            
        }else
        
        if (action.equals("actReconnect") ) 
        {                
            reconnect();
        }else

        //actSelection
        if (action.equals("actPlay") )
        {
            selectCameraWeb();            
        }else

        if (action.equals("actScreenShot") )
        {
            saveScreenShot();            
        }else
                
        if (action.equals("actRecordStop") )
        {
            recordStop();
        }else
        
        if (action.equals("actRecordStart") )
        {
            recordStart();
        }else
        
        if (action.equals("actStop") )
        {
            buttonStopCommand.modify(Control.Disabled,0);
            buttonRecord.modify(Control.Disabled,0);
            buttonPlayCommand.modify(0,Control.Disabled);
            buttonSaveScreenShot.modify(0,Control.Disabled);
            bs.shutDown();
        }
        else
        if(action.equals("MOVEMENT_LEFT")) {
            comunicationClient.sendMessage("MOVEMENT,MOV,LEFT");
        }
        else
        if(action.equals("MOVEMENT_RIGHT")) {
            comunicationClient.sendMessage("MOVEMENT,MOV,RIGHT");
        }
        else
        if(action.equals("MOVEMENT_FORWARD")) {
            comunicationClient.sendMessage("MOVEMENT,MOV,FORWARD");
        }
        else
        if(action.equals("MOVEMENT_BACKWARD")) {
            comunicationClient.sendMessage("MOVEMENT,MOV,BACKWARD");
        }
        else
        if(action.equals("MOVEMENT_BREAK")) {
            comunicationClient.sendMessage("MOVEMENT,MOV,BREAK");
        }
        else
        if(action.equals("MOVEMENT_CLEAR_ALL")) {
            comunicationClient.sendMessage("MOVEMENT,MOV,CLEAR_ALL");
        }
        else
        if(action.equals("ELV_UP")) {
            comunicationClient.sendMessage("MOVEMENT,ELV,UP");
        }
        else
        if(action.equals("ELV_DOWN")) {
            comunicationClient.sendMessage("MOVEMENT,ELV,DOWN");
        }

        else
        if(action.equals("ELV_BREAK")) {
            comunicationClient.sendMessage("MOVEMENT,ELV,BREAK");
        }
        else
        if(action.equals("SPEAK_TEXT")) {
            comunicationClient.sendMessage("SPEAKER,SPEAK,TEXT," + speakerPanel.textTextBox.getText());
        }

        else
        if(action.equals("ACTUATOR_HEAD_ROTATE_CENTER")) {
            comunicationClient.sendMessage("ACTUATOR,HEAD,ROTATE_CENTER");
        }
        else
        if(action.equals("ACTUATOR_HEAD_ROTATE_LEFT")) {
            comunicationClient.sendMessage("ACTUATOR,HEAD,ROTATE_LEFT");
        }
        else
        if(action.equals("ACTUATOR_HEAD_ROTATE_RIGHT")) {
            comunicationClient.sendMessage("ACTUATOR,HEAD,ROTATE_RIGHT");
        }
        else
        if(action.equals("ACTUATOR_HEAD_ROTATE_UP")) {
            comunicationClient.sendMessage("ACTUATOR,HEAD,ROTATE_UP");
        }
        else
        if(action.equals("ACTUATOR_HEAD_ROTATE_DOWN")) {
            comunicationClient.sendMessage("ACTUATOR,HEAD,ROTATE_DOWN");
        }
        else
        if(action.equals("ACTUATOR_HEAD_ROTATE_MIDDLE")) {
            comunicationClient.sendMessage("ACTUATOR,HEAD,ROTATE_MIDDLE");
        }

        else
        if(action.equals("RESET_ROMAS")) {
            comunicationClient.sendMessage("ROMAS,RESET,CLEAR_ALL");
        }


        if (h.shouldStop) return null;
	
        return "Ok !";
    }
    
    private void recordStop(){
        buttonRecord.setText("Record");
        buttonRecord.action = "actRecordStart";
        bs.recordStop();             
    }
    
    private void recordStart(){
        buttonRecord.setText("Stop recorder");
        buttonRecord.action = "actRecordStop";
        bs.recordStart();            
    }
    
    private void saveScreenShot(){
        if(webcamChoice.selectedIndex > 0)
        {
            int index = webcamChoice.selectedIndex;
            if(!webcamChoice.items.get(index).equals(listCameraEmpty))
            { 
                try {
                    SimpleDateFormat myDateFormat = new SimpleDateFormat("yyyy-MM-dd HH.mm.ss.S");
                    String dateS = myDateFormat.format(java.util.Calendar.getInstance());
                    bs.saveScreenShot(webcamChoice.items.get(index).toString() + "_" + dateS);
                } catch (Exception e) {
                     new MessageBox("Error save screen shot", e.getMessage(), MessageBox.MBOK).execute();
                     e.printStackTrace();
                }
                
            }
        }        
    }

    private void connect() throws Exception{
        // connessione comunication service
        comunicationClient = new ComunicationClient(connectionPanel.hostServerTextBox.getText(), Integer.parseInt(connectionPanel.portCommunicationTextBox.getText()));
        // connessione video streaming
        bs = new BannerSlider(displayVideo, connectionPanel.hostServerTextBox.getText(), Integer.parseInt(connectionPanel.portVideoTextBox.getText()));
     
        comunicationClient.connectToRemoteServer();
        bs.connectToRemoteServer();

        // gestione pulsanti
        connectionPanel.buttonConnectCommand.setText("Reconnect");
        connectionPanel.buttonConnectCommand.action = "actReconnect";
        Util.putPropertiesValue("ipserver", connectionPanel.hostServerTextBox.getText());
        Util.putPropertiesValue("videoPortServer", connectionPanel.portVideoTextBox.getText());
        Util.putPropertiesValue("commPortServer", connectionPanel.portCommunicationTextBox.getText());

        refreshWebCamera();
    }
    
    private void reconnect()throws Exception{
        comunicationClient.connectToRemoteServer();
        bs.connectToRemoteServer();        
        Util.putPropertiesValue("ipserver", connectionPanel.hostServerTextBox.getText());
        Util.putPropertiesValue("videoPortServer", connectionPanel.portVideoTextBox.getText());
        Util.putPropertiesValue("commPortServer", connectionPanel.portCommunicationTextBox.getText());

        refreshWebCamera();
    }


    private void runCameraOne(){
        if(bs.devices != null)
        {
            if(webcamChoice.items.size() > 0)
            {
                int index = 1;
                webcamChoice.select(index);
                if(!webcamChoice.items.get(index).equals(listCameraEmpty))
                {
                    bs.setDevice(index);
                    buttonStopCommand.modify(0,Control.Disabled);
                    buttonRecord.modify(0,Control.Disabled);
                    buttonPlayCommand.modify(Control.Disabled,0);
                    buttonSaveScreenShot.modify(0,Control.Disabled);
                    try{Thread.sleep(300);}catch(Exception ex){}
                    bs.playVideo();
                }
            }
        }

    }

    private void selectCameraWeb(){       
        if(bs.devices != null)
        {
            if(webcamChoice.selectedIndex > 0)
            {
                int index = webcamChoice.selectedIndex;
                if(!webcamChoice.items.get(index).equals(listCameraEmpty))
                {
                    bs.setDevice(webcamChoice.selectedIndex);
                    buttonStopCommand.modify(0,Control.Disabled);
                    buttonRecord.modify(0,Control.Disabled);
                    buttonPlayCommand.modify(Control.Disabled,0);
                    buttonSaveScreenShot.modify(0,Control.Disabled);
                    try{Thread.sleep(300);}catch(Exception ex){}
                    bs.playVideo();                    
                }
            }
        }
    }
    
    private void refreshWebCamera(){
        bs.buildChoiseDevice(listCameraEmpty);
        webcamChoice.items.clear();
        webcamChoice.items = bs.devices.items;
        if(webcamChoice.items.size()> 1)
        {
            buttonPlayCommand.modify(0,Control.Disabled);
            buttonSaveScreenShot.modify(0,Control.Disabled);
        }

        runCameraOne();
    }
    
    public static void main(String args[]){
        Application.startApplication(args);
        RomasPDA f = new RomasPDA();        
        f.execute();
        f.fullScreenOnPDA();
        Application.exit(0);
    }
}
