/*
 * Servo.java
 *
 * Created on 23 marzo 2009, 12.00
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package romas.util.comunication.controller.servo;


import romas.util.comunication.port.Serial;
import romas.util.comunication.controller.servo.ServoConfiguration.SERVO_TOWARDS;

import romas.util.LogWriter;
import java.util.logging.Level;
/**
 *
 * @author APaliani
 *
 *
 *# <ch> P <pw> S <spd>... # <ch> P <pw> S <spd> T <time><cr>
 *
 *<ch> =Channel number in decimal, 0 - 31.
 *
 *<pw> =Pulse width in microseconds, 500 - 2500.
 *
 *<spd> = Movement speed in uS per second for one channel. (Optional)
 *
 *<time> =Time in mS for the entire move, affects all channels, 65535 max. (Optional)
 *
 */
public class Servo{

    private Serial serialPort;
    public ServoConfiguration configuration;  
    
    /** Creates a new instance of Servo */
    public Servo() {
    }
    
    
    public Servo(Serial serialPort, ServoConfiguration configuration)throws Exception
    {
        this.serialPort = serialPort;
        this.configuration = configuration;
    }

    /**
     *Rotation servo
     *
     * @param towards
     * @param speed
     */
    public void prepareRotateTo(SERVO_TOWARDS towards, int speed)throws Exception {
        // "#%s P%s S%s T%s"
        LogWriter.writeLog(Level.FINE, "TOWARDS [" + towards + "], SPEED [" + speed + "]");

        Object params[] = new Object[4];

        switch(towards)
        {
            case MIN:               
                params[0] = configuration.CHANNEL;
                params[1] = getPulse(configuration.POS_MIN, configuration.POS_ACTUAL, configuration.POS_MIN, false);
                params[2] = speed;
                addCommand(params);
                break;

            case CENTER:
                params[0] = configuration.CHANNEL;
                params[1] = getPulse(configuration.POS_MEDIUM, configuration.POS_ACTUAL, configuration.POS_MEDIUM, (configuration.POS_ACTUAL > configuration.POS_MEDIUM));
                params[2] = speed;
                addCommand(params);
                break;

            case MAX:
                params[0] = configuration.CHANNEL;
                params[1] = getPulse(configuration.POS_MAX, configuration.POS_ACTUAL, configuration.POS_MAX, false);
                params[2] = speed;
                addCommand(params);
                break;

            default:
                LogWriter.writeLog(Level.WARNING, "Rotation is not implemented : [" + towards.toString() + "]");
                break;
        }

        // set actual position
        configuration.POS_ACTUAL = Integer.valueOf(params[1].toString());
        
        // wait
        Thread.sleep(300);
    }


    /**
     * Move servo to
     *
     * @param towards
     * @param speed
     * @param time
     * @param steps
     */
    public void prepareMoveTo(SERVO_TOWARDS towards, int speed, int steps)throws Exception {
        LogWriter.writeLog(Level.FINE, "TOWARDS [" + towards + "], SPEED [" + speed + "], steps [" + steps + "]");
        Object params[] = new Object[4];

        switch(towards)
        {
            case MIN:
                params[0] = configuration.CHANNEL;
                params[1] = getPulse(configuration.POS_MIN, configuration.POS_ACTUAL, steps, false);
                params[2] = speed;
                addCommand(params);
                break;

            case MAX:
                params[0] = configuration.CHANNEL;
                params[1] = getPulse(configuration.POS_MAX, configuration.POS_ACTUAL, steps, true);
                params[2] = speed;
                addCommand(params);
                break;

            default:
                LogWriter.writeLog(Level.WARNING, "Movement not implemented : [" + towards.toString() + "]");
                break;
        }

        // set actual position
        configuration.POS_ACTUAL = Integer.valueOf(params[1].toString());

        // wait
        Thread.sleep(300);
    }


    /**
     * Verify servo position
     */
    public void checkPosition() throws Exception{
        Object paramsQP[] = new Object[1];
        paramsQP[0] = configuration.CHANNEL;
        String command = String.format(this.configuration.QUERY_PULSE, paramsQP);
        serialPort.writeMessage(command, true);
    }
    

    /**
     * Retrive good pulse
     *
     * @param limit
     * @param actual
     * @param numPulse
     * @param toMAX
     * @return
     */
    public int getPulse(int limit, int actual, int numPulse, boolean toMAX){
        int pulse = 0;

        if(toMAX){
            pulse = actual + numPulse;

            if(pulse > limit){
                pulse = limit;
            }
        }
        else{
            pulse = actual - numPulse;
            if(pulse < limit){
                pulse = limit;
            }
        }

        return pulse;
    }

    /**
     *"#%s P%s S%s T%s"
     * @param params
     */
    public void addCommand(Object params[])
    {
        String command = String.format(this.configuration.COMMAND_FORMAT, params);
        serialPort.setCommand(serialPort.getCommand()+ " " + command);
    }

}
