/*
 * Serial.java
 *
 * Created on 23 marzo 2009, 11.46
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package romas.util.comunication.port;

import gnu.io.*;
import java.io.*; 
import java.util.Enumeration;
import java.util.Vector;
import java.util.logging.Level;
import romas.util.LogWriter;

/**
 *
 * @author APaliani
 */
public class Serial implements SerialPortEventListener{ 
    OutputStream outputStream = null;
    InputStream inputStream = null;
    CommPort port = null;
    SerialPort serialPort;
    CommPortIdentifier port_id = null;

    boolean processCR = true; // read serial until CR

    /** Creates a new instance of Serial */
    public Serial() {
    }


    /**
     *Create new instance of Serial
     *
     * @param portName
     * @param baud
     */
    public Serial(String portName, int baud, boolean processCR)throws Exception
    {
        this(portName, baud, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE, processCR);
    }


    /**
     *Creates a new instance of Serial
     *     
     *@param portName  : es. COM1
     *
     *@param baud      : es. 115200
     *@param dataBit   : es. SerialPort.DATABITS_8
     *@param bitStop   : es. SerialPort.STOPBITS_1
     *@param bitParity : es. SerialPort.PARITY_NONE
     *
     */
    public Serial(String portName, int baud, int dataBit, int bitStop, int bitParity, boolean processCR) throws Exception{
        try{ 
            this.processCR = processCR;
            // Prendiamo il descrittore della porta scelta.  
            // Nel caso in cui la porta non esista verr 
            // sollevata una eccezione di tipo 
            // NoSuchPortException. 
            //Enumeration enumPorts = CommPortIdentifier.getPortIdentifiers();
            port_id = CommPortIdentifier.getPortIdentifier(portName);

            // Proviamo a prendere l'ownership della porta data.  
            // Nel caso in cui la porta sia gi occupata verr  
            // sollevata una eccezione 
            // di tipo PortInUseException 
            port = port_id.open("Seriale", 3000);

            serialPort = (SerialPort)port;             
            serialPort.setSerialPortParams(baud, dataBit, bitStop, bitParity); 
                                    
            // config output stream
            outputStream = serialPort.getOutputStream();
            // config input stream
            inputStream = serialPort.getInputStream();
            // add events listener
            serialPort.setDTR(false);
            serialPort.setRTS(false);
            serialPort.addEventListener(this);
            serialPort.notifyOnDataAvailable(true);

            Thread.sleep(300); // waits till ports change state.
            
        } catch(NoSuchPortException ne){ 
            throw new Exception("Port " + portName + " not found. ");
            
        }catch(PortInUseException pe){ 
            throw new Exception("Port " + portName + " is busy, from " + pe.currentOwner);
            
        }catch(UnsupportedCommOperationException ue){ 
            throw new Exception("Inavlid settings port ! " + ue.getMessage());
            
        }catch(IOException ioe){ 
            throw new Exception("IO Error: " + ioe.getMessage());
        
        }catch (Exception e) {
            throw new Exception("Port " + portName + " error: " + e.getMessage());
        }// fine try/catch         
    }


    private String command = "";

    /**
     * Get the value of command
     *
     * @return the value of command
     */
    public String getCommand() {
        return command;
    }

    /**
     * Set the value of command
     *
     * @param command new value of command
     */
    public void setCommand(String command) {
        this.command = command;
    }


    /**
     * Send command to serial port
     *
     * @param withCR
     * @return
     */
    public boolean sendCommand(boolean withCR, int time)throws Exception{
        String sTime = String.valueOf(time);
        writeMessage(command + " T" + sTime , withCR);
        command = "";
        return true;
    }

    
    /**
     *Write message to Serial Port
     * @param message
     * @param withCR 
     */
    public void writeMessage(String message, boolean withCR)throws Exception{        
        LogWriter.writeLog(Level.FINE, message);

        String strMessage = message;

        if(withCR){
            char extChar = (char)13;
            strMessage = message + extChar;
        }

        byte[] outByte = strMessage.getBytes();
        outputStream.write(outByte);
        outputStream.flush();
    }


    /**
     *Write message to Serial Port
     * @param message
     * @param withCR
     */
    public void write(int message){

        try{
            String strMessage = String.valueOf(message);

            LogWriter.writeLog(Level.FINE, strMessage);

    //        char extChar = (char)13;
    //        strMessage = message + extChar;

            byte[] outByte = strMessage.getBytes();
            outputStream.write(outByte);
            outputStream.flush();
        }
        catch(Exception ex){
            LogWriter.writeLog(Level.SEVERE, ex.getMessage());
        }
    }

   

    public void close() {
        serialPort.close();
    }

    
    /**
     *Serial input Event 
     * @param event
     */
    public void serialEvent(SerialPortEvent event) {
        if (event.getEventType() == SerialPortEvent.DATA_AVAILABLE) {            
            try {
                if(inputStream.available() > 0){
                    int numBytes = 0;
                    byte[] readBuffer = new byte[2048];

                    if (this.processCR){
                        int nChar = 0;
                        while (!(nChar == 13)){
                            nChar = inputStream.read();
                            readBuffer[numBytes] = (byte)nChar;
                            numBytes++;
                        }
                    }else{
                        while (inputStream.available() > 0) {
                            numBytes = inputStream.read(readBuffer);
                        }
                    }

                    byte[] buffer = new byte[numBytes];
                    for(int i=0; i<numBytes; i++){
                        buffer[i] = readBuffer[i];
                    }

                    processMessage(buffer);
                }
            } catch (IOException e) {
                LogWriter.writeLog(Level.SEVERE, e.getMessage());
            }
        }
    }


    public static String[] list(){
        Enumeration enumPorts = CommPortIdentifier.getPortIdentifiers();
        Vector vPort = new Vector();

        while(enumPorts.hasMoreElements()){
            CommPortIdentifier portIdent = (CommPortIdentifier)enumPorts.nextElement();
            vPort.add(portIdent);
        }

        String[] list = new String[vPort.size()];

        for(int i=0; i<vPort.size(); i++){
            CommPortIdentifier portIdent = (CommPortIdentifier)vPort.get(i);
            list[i] = portIdent.getName();
        }

        return list;
    }


    public void dispose(){
        this.dispose();
    }

    public void setDTR(boolean dtr) {
            serialPort.setDTR(dtr);
    }

    public String getName() {
            return serialPort.getName();
    }

    /**
     *To do implements
     */
    public void processMessage(byte[] buffer){}
}
