package eve.fx;

import eve.fx.Color;
import eve.sys.Cache;

/**
* A Pen is used with certain draw operations in eve.fx.Graphics. See the
* Graphics.setPen(Pen p) method.
**/
//##################################################################
public class Pen{
//##################################################################
// Do not move these four.
public Color color = new Color(0,0,0);
public int style;
public int thickness;
public float miterLimit = 10; 
//

/**
 * Create a new Pen object.
 * @param color The color for the Pen object.
 * @param style The style for the Pen object. This should be one of the constants SOLID, DASH, DOT, DASHDOT, DASHDOTDOT or NULL,
 * optionally ORed with one of the CAP_XXX values and JOIN_XXX values. 
 * @param thickness The thickness of the Pen (in pixels).
 */
//===================================================================
public Pen(Color color,int style,int thickness)
//===================================================================
{
	this.color.set(color); 
	this.style = style; 
	this.thickness = thickness;
}
/**
 * Create a new Pen object.
 * @param color The color for the Pen object.
 * @param style The style for the Pen object. This should be one of the constants SOLID, DASH, DOT, DASHDOT, DASHDOTDOT or NULL, optionally ORed with
 * one of the CAP_XXX values and JOIN_XXX values.
 * @param thickness The thickness of the Pen (in pixels).
 * @param miterLimit A miterlimit for end joins.
 */
//===================================================================
public Pen(Color color,int style,int thickness,float miterLimit)
//===================================================================
{
	this.color.set(color); 
	this.style = style; 
	this.thickness = thickness; 
	this.miterLimit = miterLimit;
}

public static final int SOLID           = 0;
public static final int DASH            = 1;       /* -------  */
public static final int DOT             = 2;       /* .......  */
public static final int DASHDOT         = 3;       /* _._._._  */
public static final int DASHDOTDOT      = 4;       /* _.._.._  */
public static final int NULL 			= 0xff;
public static final int STYLE_MASK = 0xff;

public static final int CAP_BUTT = 0x100;
public static final int CAP_SQUARE = 0x200;
public static final int CAP_ROUND = 0x300;
public static final int CAP_MASK = 0x300;

public static final int JOIN_BEVEL = 0x1000;
public static final int JOIN_MITER = 0x2000;
public static final int JOIN_ROUND = 0x3000;
public static final int JOIN_MASK = 0x3000;

/**
 * @deprecated - use getCopy().set(differentColor);
 * @param differentColor color to set a new Pen to.
 * @return a copy of the Pen with a different color.
 */
//===================================================================
public Pen change(Color differentColor)
//===================================================================
{
	return new Pen(differentColor,style,thickness);	
}
/**
 * Change the parameters of this Pen and return itself.
 * @param c the new Color.
 * @param style the new style.
 * @param thickness the new thickness.
 * @return this Pen.
 */
public Pen set(Color c, int style, int thickness)
{
	this.color.set(c);
	this.style = style;
	this.thickness = thickness;
	return this;
}

public Pen()
{
	this(Color.Black,SOLID,1);
}
/**
 * Set the color of the Pen and return itself.
 * @param color the new Color.
 * @return this Pen.
 */
public Pen set(Color color)
{
	this.color.set(color);
	return this;
}
public void set(Pen other)
{
	if (other == null) style = NULL;
	else {
		color.set(other.color);
		style = other.style;
		thickness = other.thickness;
	}
}
public Pen getCopy()
{
	Pen p = new Pen();
	p.set(this);
	return p;
}
public static Pen getCached(int r, int g, int b,int style,int thickness)
{
	Pen c = (Pen)Cache.get(Pen.class);
	c.color.set(r,g,b);
	c.style = style;
	c.thickness = thickness;
	return c;
}
public static Pen getCached(Color col, int style,int thickness)
{
	Pen c = (Pen)Cache.get(Pen.class);
	c.color.set(col);
	c.style = style;
	c.thickness = thickness;
	return c;
}
public static Pen getCached(Pen other)
{
	Pen c = (Pen)Cache.get(Pen.class);
	c.set(other);
	return c;
	
}
public static Pen getCached()
{
	return (Pen)Cache.get(Pen.class);
}

public void cache()
{
	Cache.put(this);
}

//##################################################################
}
//##################################################################

