package eve.ui;
import eve.sys.Event;
import eve.sys.EventListener;
import eve.ui.event.ControlEvent;
import eve.util.Intable;
import eve.util.Textable;
import java.util.Vector;
import eve.util.mString;

/**
* A CheckBoxGroup object is used to group together a set of mutually exclusive
* CheckBox objects. In other words, it will turn a set of mCheckBoxes into
* radio buttons. Furthermore, any ControlEvent.PRESSED events which are generated
* by the checkboxes in the group will have its target changed so that it appears
* as if these events are originating from the CheckBoxGroup object (if takeTarget is true).
*/
//##################################################################
public class CheckBoxGroup extends Vector implements EventListener,Textable,Intable{
//##################################################################

public boolean exclusive = true;
/**
* If takeTarget is true then ControlEvent.PRESSED events generated by the individual
* boxes will have its target changed so that it appears to be coming from this group.
**/
public boolean takeTarget = true;
/**
* Returns the index of the CheckBox currently selected.
*/
//==================================================================
public int getSelectedIndex()
//==================================================================
{
	for (int i = 0; i<size(); i++){
		CheckBox cb = (CheckBox)get(i);
		if (cb.state) return i;
	}
	return -1;
}
/**
* Returns the CheckBox currently selected.
*/
//==================================================================
public CheckBox getSelected()
//==================================================================
{
	int i = getSelectedIndex();
	if (i == -1) return null;
	return (CheckBox)get(i);
}
/**
* Select the CheckBox at the given index.
*/
//==================================================================
public void selectIndex(int index)
//==================================================================
{
	CheckBox cb = (CheckBox)get(index);
	if (cb != null) cb.setState(true);
}
//==================================================================
public void onEvent(Event ev)
//==================================================================
{
	if (ev.type == ControlEvent.PRESSED && (indexOf(ev.target) != -1) && takeTarget) {
		ev.target = this;
		//System.out.println("Changing!");
	}
}


/**
 * This calls selectIndex() with the provided value.
 */
//===================================================================
public void setInt(int value) {selectIndex(value);}
//===================================================================
/**
* This returns the same value as getSelectedIndex().
**/
//===================================================================
public int getInt() {return getSelectedIndex();}
//===================================================================
/**
* This will turn on the checkbox in this group with a label the same
* as the provided String value.
**/
//===================================================================
public void setText(String value)
//===================================================================
{
	for (int i = 0; i<size(); i++){
		CheckBox cb = (CheckBox)get(i);
		if (mString.compare(value,cb.getText(),true) == 0) {
			cb.setState(true);
			return;
		}
	}
}
/**
* Return the label of the selected checkbox, or an empty String if none
* is selected.
**/
//===================================================================
public String getText()
//===================================================================
{
	CheckBox cb = getSelected();
	if (cb == null) return "";
	return cb.getText();
}
/**
* This creates a set of new CheckBox controls, each one of which is assigned a label
* as specified in the choices parameter. The CheckBox controls are then added to
* the destination panel in the specified number of columns.
* @param choices a list of lables for each checkbox.
* @param destination a destination panel to add to.
* @param columns the number of columns to add.
*/
//===================================================================
public void makePanel(String[] choices,CellPanel destination,int columns)
//===================================================================
{
	destination.equalWidths = true;
	int col = 0;
	for (int i = 0; i<choices.length; i++) {
		CheckBox cb = new CheckBox(choices[i]);
		destination.addNext(cb);
		cb.setGroup(this);
		col++;
		if (col >= columns) {
			destination.endRow();
			col = 0;
		}
	}
}

/**
 * @deprecated - do not use this, use CheckBox.setGroup() to add an CheckBox.
 */
public CheckBoxGroup addCheck(CheckBox what)
{
	add(what);
	return this;
}

//##################################################################
}
//##################################################################

