/*
 * Created on Aug 13, 2005
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui;

import eve.fx.Color;
import eve.fx.Graphics;
import eve.fx.gui.Cursor;
import eve.sys.Device;
import eve.sys.HandleStoppedException;
import eve.ui.event.ControlEvent;

/**
 * @author Michael L Brereton
 *
 */
//####################################################
public class FormFrame extends Frame {
	
	public Form myForm;
	public boolean hasExtraTitleControls = false;
	public String toString()
	{
		if (myForm != null) return "Frame for: "+myForm.title;
		else return super.toString();
	}
//	==================================================================
	public CellPanel titleBar,middleBar,rightButtons;
	public Label title;
//	==================================================================
	/**
	 * Set this true to show an Eve Size Bar
	 */
	public boolean useSizeBar = !Gui.hasSoftKeys;//true;//false;
	
//	==================================================================
	public FormFrame(Form f,int options)
//	==================================================================
	{
		myForm = f;
		if (f.noBorder || (((options & Gui.NEW_WINDOW) == Gui.NEW_WINDOW) && ((myForm.windowFlagsToClear & Window.FLAG_HAS_TITLE) == 0))) {
			borderWidth = 0;
			borderStyle = BDR_NOBORDER;
		}else{
			borderWidth = 3;
			borderStyle = EDGE_RAISED|BDR_OUTLINE;
		}
		name = myForm.title;
		contents.addLast(f).setCell(STRETCH);
		if (f.hasTopBar){
			titleBar = new DragPanel();
			if (Gui.isSmartPhone) titleBar.setPreferredSize(-1,20);
			((DragPanel)titleBar).canDrag = f.moveable || (((options & Gui.NEW_WINDOW) == Gui.NEW_WINDOW) && !Device.isMobile());
			top.addLast(titleBar).setCell(HSTRETCH);
			Label ml = title = new Label(f.title);
			ml.modify(PenTransparent,0);
			ml.foreGround = Color.White;
			ml.backGround = Color.DarkBlue;
			ml.alignment = Gui.Left;
			titleBar.addNext(ml,STRETCH,FILL|NORTHWEST);
			//mControl c = new mCanvas();
			//titleBar.addLast(c);
			//c.backGround = Color.Black;
			titleBar.addNext(middleBar = new CellPanel()).setCell(VSTRETCH);
			titleBar.addLast(rightButtons = new CellPanel()).setCell(VSTRETCH);
			titleBar.setBorder(BDR_OUTLINE|BF_RECT,1);//EDGE_ETCHED,3);
			middleBar.backGround = Color.Black;
			middleBar.foreGround = Color.White;
			rightButtons.equalWidths = false;
		}
		if (!f.resizable && !f.hasTopBar) contentsOnly = true;
		addListener(f);
		setAlpha(f.frameAlpha);
	}

//	==================================================================
	public void make(boolean remake)
//	==================================================================
	{
		if (made) {
			//System.out.println("Already made!");
			//getPreferredSize();
			return;
		}
		//myForm.checkButtons(); <- This is done in Form now.
		if (myForm.hasTopBar){
			rightButtons.addNext(myForm.titleControls);
			rightButtons.addNext(myForm.titleOK);
			rightButtons.addNext(myForm.titleCancel);
			hasExtraTitleControls = myForm.titleControls != null || 
				(myForm.titleOK != null && myForm.titleCancel != null);
		
		//if (myForm.hasTopBar){ This was here before. I moved it above.
			if (myForm.topControls != null) top.addNext(myForm.topControls,HSTRETCH,HFILL|NORTHWEST);//.setCell(DontStretch).setControl(DontFill,NorthWest);//.setSpan(2);
			top.addChildListener(myForm);
		}else top = null;
		if (bottom != null) bottom.addChildListener(myForm);
		if (myForm.resizable && useSizeBar) {
			DragPanel resizeDragBar;
			trueBottom.addLast(resizeDragBar = new DragPanel());//.setControl(Top|Right);
			resizeDragBar.setCursor(Cursor.RESIZE_CURSOR);
			resizeDragBar.isTopBar = false;
			resizeDragBar.borderStyle = EDGE_SUNKEN;
		}
		super.make(remake);
	}

//	==================================================================
	public void onControlEvent(ControlEvent ev)
//	==================================================================
	{
		if (ev.type == ControlEvent.PRESSED){
			if (ev.target == myForm.titleOK) myForm.exit(Form.IDOK);
			else if (ev.target == myForm.titleCancel) myForm.exit(Form.IDCANCEL);
		}
		super.onControlEvent(ev);
	}

	protected boolean fillSip(int height, boolean unfill)
	{
		try{
			openState.waitOn(SizedFlag);
		}catch(HandleStoppedException e){
			return false;
		}catch(InterruptedException e){
			return false;
		}
		try{
			if (!unfill) myForm.fillSip(height);
			else myForm.unfillSip();
			return true;
		}catch(Exception e){
			return false;
		}
	}

//	==================================================================
	public void doBorder(Graphics g)
//	==================================================================
	{
	/*
		g.setColor(Color.Black);
		g.drawRect(0,0,width,height);
		g.setColor(Color.White);
		g.drawLine(1,1,width-2,1);
		g.drawLine(1,1,1,height-2);
		g.setColor(Color.Black);
		g.drawLine(1,height-2,width-2,height-2);
		g.drawLine(width-2,1,width-2,height-2);
		g.setColor(Color.LightGray);
		g.drawRect(2,2,width-4,height-4);
	*/
		super.doBorder(g);
	}
}
//####################################################
