/*
 * Created on Feb 10, 2007
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui;

import java.util.Vector;

import eve.fx.BufferedGraphics;
import eve.fx.Color;
import eve.fx.Dimension;
import eve.fx.DisplayLineSpecs;
import eve.fx.FontMetrics;
import eve.fx.Graphics;
import eve.fx.IImage;
import eve.fx.IconAndText;
import eve.fx.ImageBuffer;
import eve.fx.ImageTool;
import eve.fx.Metrics;
import eve.fx.OnScreenImage;
import eve.fx.Point;
import eve.fx.Rect;
import eve.fx.gui.IKeys;
import eve.sys.Vm;
import eve.sys.mThread;
import eve.ui.event.ControlEvent;
import eve.ui.event.KeyEvent;
import eve.util.StringList;

/**
 * @author Michael L Brereton
 *
 */
//####################################################
public class IconMenu extends Control implements ScrollClient{

	{
		modify(TakesKeyFocus|WantHoldDown,NoFocus);
	}
	
	public void handleContextMenuItem(MenuItem selected, int selectedItem)
	{
	}
	/**
	 * This sends a Cancel event.
	 *
	 */
	public void doCancel()
	{
		postEvent(new ControlEvent(ControlEvent.CANCELLED,this));
	}
//	===================================================================
	public void popupMenuEvent(Object selectedItem)
//	===================================================================
	{
		if (selectedItem instanceof MenuItem){
			handleContextMenuItem((MenuItem)selectedItem,selectedIndex);
		}
	}
	int indexAtPoint(int x, int y)
	{
		int r = y/itemHeight;
		if (r >= numRows) r = numRows-1;
		int c = x/itemWidth;
		if (c >= numCols) c = numCols-1;
		int which = r*numCols+c;
		if (which < 0) return -1;
		if (which + firstItem >= countItems()) return -1;
		return which+firstItem;
	}
	
	public boolean addToContextMenu(Menu dest, int selectedItem)
	{
		return false;
	}
//	-------------------------------------------------------------------
	protected boolean doShowMenu(Point pen)
//	-------------------------------------------------------------------
	{
		int where = indexAtPoint(pen.x,pen.y);
		Menu m = new Menu();
		if (!addToContextMenu(m,where)) return false;
		setMenu(m);
		menuState.outsideOfControl = false;
		return super.doShowMenu(pen);
	}
	
public int preferredItemWidth = isDoubleSized() ? 180 : 100, preferredItemHeight = isDoubleSized() ? 180 : 100;
public int preferredIconWidth = 48, preferredIconHeight = 48;

public void setDisplayMode(int displayMode,int itemWidth, int itemHeight, int iconWidth, int iconHeight)
{
	this.displayMode = displayMode;
	preferredItemWidth = itemWidth;
	preferredItemHeight = itemHeight;
	preferredIconWidth = iconWidth;
	preferredIconHeight = iconHeight;
	if (displayMode == DISPLAY_LINES)
		highlightMode = HIGHLIGHT_ICON|HIGHLIGHT_TEXT;
	else
		highlightMode = HIGHLIGHT_ICON;
}
public void setDisplayMode(int displayMode)
{
	setDisplayMode(displayMode,null);
}
public void setDisplayMode(int displayMode,FontMetrics fontToUse)
{
	if (fontToUse == null){
		fontToUse = getFontMetrics(Application.findFont("gui"));
	}
	int s = isDoubleSized() ? 2 : 1;
	if (displayMode == DISPLAY_LINES){
		int textSize = fontToUse.getHeight()+textSpacing*2;
		int iconHeight = 24*s;
		if (iconHeight > textSize) textSize = iconHeight;
		setDisplayMode(displayMode,100*s,textSize,iconHeight,iconHeight);
	}else {
		int iconHeight = 48*s;
		int textSize = (fontToUse.getHeight()+textSpacing)*2;
		setDisplayMode(displayMode,100*s,iconHeight+textSize,iconHeight,iconHeight);
	}
}
public IconMenu()
{
	setDisplayMode(Gui.getScreenSize().width <= 176 ? DISPLAY_LINES : DISPLAY_GRID);
}
public static int DISPLAY_GRID = 0;
public static int DISPLAY_LINES = 1;

public int displayMode = DISPLAY_GRID;
/**
 * This is what percent the "preferred" sizes are allowed to vary.
 */
public int variablePercent = 5;

/**
 * An option for highlightMode (the default). If this option is used
 * the icon is highlighted when selected. This may be used with
 * HIGHLIGHT_TEXT.
 */
public static final int HIGHLIGHT_ICON = 0x1;
/**
 * An option for highlightMode (the default). If this option is used
 * the text is highlighted when selected. This may be used with
 * HIGHLIGHT_ICON.
 */
public static final int HIGHLIGHT_TEXT = 0x2;
/**
 * The way that an item is highlighted when selected.
 * It can be a combination of the HIGHLIGHT_XXX values.
 */
public int highlightMode =  HIGHLIGHT_ICON;
	//|HIGHLIGHT_TEXT;
private int itemWidth, itemHeight;
private int numRows, numCols;

public static final int SCROLL_DONT_SCROLL = 0;
public static final int SCROLL_VERTICALLY = 1;
//public static final int SCROLL_HORIZONTALLY = 2;

public int scrollMode = SCROLL_VERTICALLY;

private int firstItem = 0;

private int canFit(int size, int preferred, int percentSmaller)
{
	int lower = preferred-(preferred*percentSmaller)/100;
	int min = size/preferred;
	if (min < 1) return 1;
	if (lower*(min+1) <= size) return min+1;
	return min;
}

public Control getBestScroller(int forScrollMode)
{
	scrollMode = forScrollMode;
	if (scrollMode == SCROLL_DONT_SCROLL) return this;
	//else if (scrollMode == SCROLL_HORIZONTALLY) return new HorizontalScrollPanel(this);
	else return new VerticalScrollPanel(this);
}
public Control getBestScroller(int forWidth, int forHeight)
{
	int vf = Vm.getParameter(Vm.VM_FLAGS);
	if (Gui.hasSoftKeys && ((vf & Vm.VM_FLAG_HAS_BACK_BUTTON) != 0))
		return getBestScroller(SCROLL_DONT_SCROLL);
	//if (forWidth > forHeight) return getBestScroller(SCROLL_HORIZONTALLY);
	else return getBestScroller(SCROLL_VERTICALLY);
}
public Control getBestScroller()
{
	Dimension d = Gui.getScreenSize();
	return getBestScroller(d.width,d.height);
}
public int countTotalRows()
{
	return (countItems()+(numCols-1))/numCols;
}
public int countTotalCols()
{
	return (countItems()+(numRows-1))/numRows;
}
public int countItemsThatFit(int width, int height)
{
	int numRows = canFit(height,preferredItemHeight,variablePercent);
	int numCols = canFit(width,preferredItemWidth,variablePercent);
	return numRows*numCols;
}
public void resizeTo(int width, int height)
{
	super.resizeTo(width,height);
	//System.out.println(preferredItemWidth+", "+preferredItemHeight);
	numRows = canFit(height,preferredItemHeight,variablePercent);
	numCols = displayMode == DISPLAY_LINES ? 1 : canFit(width,preferredItemWidth,variablePercent);
	itemHeight = height/numRows;
	itemWidth = width/numCols;
	notifyStructureChanged();
}
/**
Get the total number of items on the dispay.
**/
//-------------------------------------------------------------------
protected int getTotalItemsOnScreen()
//-------------------------------------------------------------------
{
	return numRows*numCols;
}
/**
 * Get the Rect within the display for the specified item on the screen.
 * The index starts at zero for the top left item, then counts left to right, top
 * down.
 * @param itemOnScreen the index of the item on the screen.
 * @param destination a non-null destination Rect to hold the item rect.
 * @return true if the item is on the screen, false if it is out of range.
 */
public boolean getItemOnScreenRect(int itemOnScreen, Rect destination)
{
	if (itemOnScreen < 0) return false;
	int r = itemOnScreen/numCols;
	int c = itemOnScreen%numCols;
	if (r >= numRows) return false;
	destination.set(c*itemWidth,r*itemHeight,itemWidth,itemHeight);
	if (r == numRows-1) destination.height = height-(numRows-1)*itemHeight;
	if (c == numCols-1) destination.width = width-(numCols-1)*itemWidth;
	return true;
}
/**
 * Get the Rect within the display for the absolute item index.
 * @param absoluteIndex the absolute index of the item.
 * @param destination a non-null destination Rect.
 * @return true if the item is on the screen, false if it is not.
 */
public boolean getItemRect(int absoluteIndex, Rect destination)
{
	return getItemOnScreenRect(getIndexOnScreen(absoluteIndex),destination);
}
public int getIndexOnScreen(int absoluteIndex)
{
	if (absoluteIndex < firstItem) return -1;
	if (firstItem+getTotalItemsOnScreen() < absoluteIndex) return -1;
	return absoluteIndex-firstItem;
}
/** 
Items are added to this as Strings or MenuItem objects. Alternatively, the
method getItemAtIndex() may be overriden.
**/
public Vector items = new Vector();
/**
Create a MenuItem given an Icon and a label, possibly scaling the icon
if necessary.
**/
//===================================================================
public MenuItem makeItem(IImage icon, String text)
//===================================================================
{
	MenuItem mi = new MenuItem();
	mi.label = mi.action = text;
	if (icon != null)
		if (icon.getWidth() != preferredIconWidth || icon.getHeight() != preferredIconHeight)
			icon = ImageTool.scale(ImageTool.toImageData(icon),preferredIconWidth,preferredIconHeight,0);
	if (displayMode == DISPLAY_LINES){
		if (text != null) text = text.replace('\n',' ');
		mi.image = icon;//new IconAndText(icon,text,null,Right);
		mi.label = text;//null;
	}else{
		mi.image = icon;
		/*
		if (mi.image == null){
			mi.image = new IconAndText(null,text,null,Down);
			mi.label = null;
		}
		*/
	}
	if (mi.image instanceof IconAndText)
		((IconAndText)mi.image).textColor = null;
		/*
		int w = icon.getWidth();
		if (useGrid && w < 32 && !dontScaleIcons)
			icon = ImageTool.scale(ImageTool.toImageData(icon),32,32,0);
		else if (!useGrid && w > 16 && !dontScaleIcons)
			icon = ImageTool.scale(ImageTool.toImageData(icon),16,16,0);
		mi.image = text != null ? new IconAndText(icon,text,null,useGrid ? Down : Right) : icon;
		*/
		//mi.image = icon;
	
	return mi;
}
/**
Add an item and return the MenuItem created for it. The "image" field of the 
returned MenuItem will be an IconAndText if <b>both</b> "icon" and "text" are
not null.
**/
//===================================================================
public MenuItem addItem(IImage icon, String text)
//===================================================================
{
	MenuItem mi = makeItem(icon,text);
	items.add(mi);
	return mi;
}
/**
* Return the number of items.
**/
//===================================================================
public int countItems()
//===================================================================
{
	return items == null ? 0 : items.size();
}
/**
 * Get the item at the specified index as a MenuItem.
 * @param index the item index.
 * @return the item at the specified index as a MenuItem.
 */
//===================================================================
public MenuItem getItemAtIndex(int index)
//===================================================================
{
	if (items == null) return null;
	if (index < 0 || index >= items.size()) return null;
	Object got = items.get(index);
	if (got instanceof MenuItem) return (MenuItem)got;
	MenuItem mi = new MenuItem();
	mi.label = got.toString();
	items.setElementAt(mi,index);
	return mi;
}
private int paintSelectedMode = PAINT_SELECTED;

private static final int PAINT_INDIVIDUAL = 0x1;
//protected static final int PAINT_IS_MORE_ITEM = 0x2;
protected static final int PAINT_SELECTED = 0x4;
protected static final int PAINT_FLASH_ON = 0x8;
//protected static final int PAINT_FLASH_OFF = 0x10;

private void getIconRect(MenuItem item, Rect fullItemRect, Rect iconRect)
{
	IImage im = item.image;
	int w = im == null ? 0 : im.getWidth();
	int h = im == null ? 0 : im.getHeight();
	int xx = fullItemRect.x, yy = fullItemRect.y, ww = itemWidth, hh = fullItemRect.height;//fullItemRect.width;
	if (displayMode == DISPLAY_LINES){
		iconRect.set(xx,yy+(hh-h)/2,w,h);
	}else
		iconRect.set(xx+(ww-w)/2, yy, w, h);
}

private int textSpacing = 2;

private boolean getTextRect(MenuItem item, Rect fullItemRect, Rect textRect)
{
	IImage im = item.image;
	int w = im == null ? 0 : im.getWidth();
	int h = im == null ? 0 : im.getHeight();
	int hh = fullItemRect.height, yy = fullItemRect.y, ww = itemWidth;//fullItemRect.width;
	if (displayMode == DISPLAY_LINES)
		iconRect.set(w+textSpacing, textSpacing, ww-w-textSpacing*2, hh-textSpacing*2);
	else
		iconRect.set(textSpacing, yy+h+textSpacing, ww-textSpacing*2, hh-h-textSpacing*2);
	return true;
}
/**
 * Override this to change how highlighting is done.
 * @param mi the MenuItem being highlighted.
 * @param g the Graphics being drawn on. At this point the background has been drawn
 * and you will be able to copy from the Graphics so it is possible to highlight by
 * actually altering the background.
 * @param area the area being highlighted.
 * @param highlightingText true if text is being highlighted. This may be true with
 * highlightingIcon also being true.
 * @param highlightingIcon true if icon is being highlighted. This may be true with
 * highlightingText also being true.
 * @param paintOptions one of the PAINT_XXX values. PAINT_SELECTED means a normal selection. 
 * PAINT_FLASH_ON means the icon is being "flashed" to indicate action.
 * @return the new foreground color to use for the text to be drawn after.
 */
protected Color highlightArea(MenuItem mi,Graphics g, Rect area, boolean highlightingText, boolean highlightingIcon, int paintOptions)
{
	/*
	if (true){
		int nw = (area.width*5)/10;
		if ((nw % 2) != 0) nw--;
		int nh = (area.height*5)/10;
		if ((nh % 2) != 0) nh--;
		Image im2 = new Image(nw,nh);
		Graphics gi = new Graphics(im2);
		gi.copyRect(g,area.x+(area.width-nw)/2,area.y+(area.height-nh)/2,nw,nh,0,0);
		gi.free();
		GraphicsImageData gid = new GraphicsImageData();
		gid.setFor(g,area.x+(area.width-nw)/2,area.y+(area.height-nh)/2,nw,nh,false);
		Image im = new Image(area.width,area.height);
		ImageTool.scale(gid,im,0);
		im2.free();
		//
		//
		//ImageTool.scale(gid,im,0);
		//g.drawImage(im,area.x,area.y);
		//g.setColor(highlightColor);
		//g.fillRect(area.x,area.y,area.width,area.height);
		g.draw(im,area.x,area.y);
		//g.drawRect(area.x,area.y,area.width,area.height);
		return getForeground();
	}
	*/
	if ((mi.modifiers & mi.MODIFIER_DONT_HIGHLIGHT) == 0){
		g.setColor(highlightColor);
		if ((paintOptions & PAINT_SELECTED) != 0){
			g.fillRect(iconRect.x, iconRect.y, iconRect.width, iconRect.height);
			return highlightingText ? getBackground() : getForeground();
		}else if ((paintOptions & PAINT_FLASH_ON) != 0){
			g.drawRect(iconRect.x, iconRect.y, iconRect.width, iconRect.height);
			return getForeground();
		}
	}
	return getForeground();
}
public Color highlightColor = Color.DarkBlue;
//-------------------------------------------------------------------
private Rect itemRect, oldClip, iconRect;
private ImageBuffer itemBuffer;
private synchronized void paintItem(Graphics gr, int indexOnScreen, int options, FontMetrics fm)
//-------------------------------------------------------------------
{
	boolean horizontal = displayMode == DISPLAY_LINES;
	if (itemRect == null){
		itemRect = new Rect();
		oldClip = new Rect();
		iconRect = new Rect();
		itemBuffer = new ImageBuffer(); 
	}
	if (!getItemOnScreenRect(indexOnScreen,itemRect)) return;
	BufferedGraphics buffg = bufferGraphics(gr,itemRect,0);
	if (buffg == null) return;
	buffg.translateToZero();
	Graphics g = buffg.getGraphics();//prepareImageBuffer(itemBuffer,itemRect,false,gr); 
		//itemBuffer.get(itemRect.width,itemRect.height,false);
	//
	// We are drawing into an ImageBuffer, so offset itemRect to 0,0
	// but save the original values.
	//
	int irx = itemRect.x, iry = itemRect.y;
	itemRect.x = itemRect.y = 0;
	//
	boolean amSelected = selectedIndex == firstItem+indexOnScreen;
	if (amSelected) options |= paintSelectedMode;//PAINT_SELECTED;
	//
	MenuItem mi = getItemAtIndex(firstItem+indexOnScreen);
	Color bg = getBackground();
	Color fg = getForeground();
	int anchor = displayMode == DISPLAY_LINES ? WEST : NORTH;
	int alignment = displayMode == DISPLAY_LINES ? LEFT : CENTER;
	if (fm == null) fm = getFontMetrics();
	g.setColor(fg);
	
	if (mi != null){
		//
		// See what area needs highlighting.
		//
		if (((options & (PAINT_SELECTED|PAINT_FLASH_ON)) != 0)){
			boolean hi = (highlightMode & HIGHLIGHT_ICON) != 0;
			boolean ht = (highlightMode & HIGHLIGHT_TEXT) != 0;
			if (hi && ht) iconRect.set(itemRect);
			else if (hi) getIconRect(mi,itemRect,iconRect);
			else if (ht) {
				if (getTextRect(mi,itemRect,iconRect) && ht && !hi){
					//
					// The text rect is now in iconRect.
					//
					if (mi.splitLabel == null){
						if (mi.label == null) mi.splitLabel = new StringList();
						else if (horizontal || mi.label.indexOf(' ') == -1) mi.splitLabel = new StringList(mi.label.replace('\n',' '));
						else mi.splitLabel = new DisplayLineSpecs(mi.label,fm,iconRect.width).split(false,null);
					}
					Metrics.anchorTextIn(fm,mi.splitLabel.toBestText(),iconRect,anchor,null);
					iconRect.x -= textSpacing; iconRect.y -= textSpacing;
					iconRect.width += textSpacing*2; iconRect.height += textSpacing*2;
				}
			}
			g.setColor(highlightArea(mi,g,iconRect,ht,hi,options));
		}	
		//
		//
		//
		// Get icon area alone.
		//
		getIconRect(mi,itemRect,iconRect);
		//
		if (mi.image != null){
			int opts = amSelected ? IImage.ACTIVE : IImage.INACTIVE;
			mi.image.draw(g,iconRect.x,iconRect.y,opts);
		}
		//
		if (getTextRect(mi,itemRect,iconRect)){
			//
			// The text rect is now in iconRect.
			//
			if (mi.splitLabel == null){
				if (mi.label == null) mi.splitLabel = new StringList();
				else if (horizontal || mi.label.indexOf(' ') == -1) mi.splitLabel = new StringList(mi.label.replace('\n',' '));
				else mi.splitLabel = new DisplayLineSpecs(mi.label,fm,iconRect.width).split(false,null);
			}
			Metrics.drawText(g,fm,mi.splitLabel.toBestText(),iconRect,alignment,anchor);
		}
	}
	buffg.release();
	if (mi != null && mi.image != null &&mi.image instanceof OnScreenImage){
		((OnScreenImage)mi.image).setRefresher(this);
	}
}
//-------------------------------------------------------------------
protected void paintItem(Graphics gr, int indexOnScreen)
//-------------------------------------------------------------------
{
	paintItem(gr,indexOnScreen,PAINT_INDIVIDUAL,null);
}
//===================================================================
public void repaintItem(int whichItem)
//===================================================================
{
/*
	repaintItem(whichItem,null);
}
//-------------------------------------------------------------------
protected void repaintItem(int whichItem, Graphics gr)
//-------------------------------------------------------------------
{
*/
	int onScreen = getIndexOnScreen(whichItem);
	if (onScreen == -1) return;
	paintItem(null,onScreen);
}

public void doBackground(Graphics g)
{
	super.doBackground(g);
}
public void doPaint(Graphics gr, Rect area)
{
	if (itemWidth <= 0 || itemHeight <= 0) return;
	FontMetrics fm = getFontMetrics();
	Rect ir = Rect.getCached();
	int hleft = height;
	int idx = 0;
	int yy = 0;
	for (int r = 0; r<numRows; r++){
		int wleft = width;
		int xx = 0;
		for (int c = 0; c<numCols; c++){
			ir.set(xx, yy, c == numCols-1 ? wleft : itemWidth, r == numRows-1 ? hleft : itemHeight);
			if (area == null || ir.intersects(area)){
				paintItem(gr,idx,0,fm);
			}
			idx++;
			xx += itemWidth;
			wleft -= itemWidth;
		}
		yy += itemHeight;
		hleft -= itemHeight;
	}
}

public int selectedIndex = -1;

/**
 * This gets called when the 
 * selection changes. By default it calls notifyDataChange()
 * to send off a DataChangeEvent.
 */
protected void dataChanged()
{
	notifyDataChange();
}
/**
 * Returns if there are more pages to display.
 */
public boolean hasMorePages()
{
	int onScreen = getTotalItemsOnScreen();
	boolean sel = selectedIndex != -1;
	if (onScreen+firstItem >= countItems()) return false;
	return true;
}
/**
 * Returns if there are previous pages to display.
 */
public boolean hasPreviousPages()
{
	return (firstItem > 0);
}
//-------------------------------------------------------------------
public boolean nextPage(boolean forwards,boolean repaint)
//-------------------------------------------------------------------
{
	try{
		int onScreen = getTotalItemsOnScreen();
		boolean sel = selectedIndex != -1;
		if (forwards){
			if (onScreen+firstItem >= countItems()) return false;
			firstItem += onScreen;
		}else{
			if (firstItem <= 0) return false;
			firstItem -= onScreen;
			if (firstItem < 0) firstItem = 0; 
		}
		if (sel) selectedIndex = firstItem;
		if (ss != null)
			ss.checkScrolls();
		if (repaint) repaintNow();
		if (sel) dataChanged();
		return true;
	}finally{
		notifyStructureChanged();
	}
}
int selectedRow()
{
	if (selectedIndex < firstItem) return -1;
	int rw = (selectedIndex-firstItem)/numCols;
	if (rw >= numRows) return -1;
	return rw;
}
void moveSelected(int num)
{
	try{
		int cs = selectedIndex;
		selectedIndex = -1;
		repaintItem(cs);
		selectedIndex = cs+num;
		int tot = countItems();
		if (selectedIndex < 0) selectedIndex = 0;
		else if (selectedIndex >= tot)
			selectedIndex = tot-1;
		repaintItem(selectedIndex);
		dataChanged();
	}finally{
		//notifyStructureChanged();
	}
}
/**
Get which item is selected.
**/
//===================================================================
public int getSelectedItem()
//===================================================================
{
	return selectedIndex;
}
/*
 * Returns true if a repaint of the screen was done, false if not.
 */
public boolean makeVisible(int itemIndex)
{
	try{
		int max = countItems();
		if (itemIndex < 0 || itemIndex >= max) return false;
		int tot = getTotalItemsOnScreen();
		if (itemIndex >= firstItem && itemIndex < firstItem+tot) return false;
		firstItem = ((max+tot)/tot)*tot;
		while(firstItem > itemIndex) firstItem -= tot;
		repaintNow();
		return true;
	}finally{
		notifyStructureChanged();
	}
}
//===================================================================
public void setSelectedItem(int newSelectedItem, boolean makeVisible)
//===================================================================
{
	int was = selectedIndex;
	if (newSelectedItem == was) return;
	selectedIndex = newSelectedItem;
	boolean didRepaint = false;
	if (makeVisible && newSelectedItem != -1) didRepaint = makeVisible(newSelectedItem);
	if (!didRepaint){
		if (was != -1) repaintItem(was);
		repaintItem(newSelectedItem);
	}
	dataChanged();
}
//===================================================================
public void setSelectedItem(int newSelectedItem)
//===================================================================
{
	setSelectedItem(newSelectedItem,false);
}
/**
 * This is called by doSelectedAction() just before it posts an Action event.
 * Override it to provide functionality - by default it does nothing.
 * @param itemIndex the index of the item that was selected with the
 * action button or mouse click.
 */
protected void itemAction(int itemIndex)
{
	
}
/**
 */
//===================================================================
public void doSelectedAction(int itemIndex)
//===================================================================
{
	if (itemIndex == selectedIndex) selectedIndex = -1;
	paintSelectedMode = PAINT_FLASH_ON;
	setSelectedItem(itemIndex);
	mThread.nap(150);
	//paintSelectedMode = PAINT_FLASH_OFF;
	paintSelectedMode = PAINT_SELECTED;
	repaintItem(itemIndex);
	itemAction(itemIndex);
	notifyAction();
}

public void penPressed(Point ev)
{
	int which = indexAtPoint(ev.x,ev.y);
	if (which != -1) setSelectedItem(which,true);
}
public void penClicked(Point ev)
{
	int which = indexAtPoint(ev.x,ev.y);
	if (which != selectedIndex) return;
	doSelectedAction(which);
}
//===================================================================
public void onKeyEvent(KeyEvent ev)
//===================================================================
{
	int cs = selectedIndex;
	int tot = countItems();
	if (ev.type == ev.KEY_PRESS){
		//Device.messageBox("Key",ev.key+" - "+Integer.toHexString(ev.key)+":"+Integer.toHexString(ev.modifiers) , Device.MB_TYPE_OK);
		if (ev.key == IKeys.PAGE_DOWN){
			nextPage(true,true);
		}else if (ev.key == IKeys.PAGE_UP || ev.isBackKey()){
			nextPage(false,true);
		}else if (ev.key == IKeys.UP || ev.key == IKeys.DOWN || ev.key == IKeys.LEFT || ev.key == IKeys.RIGHT){
			int sr = selectedRow();
			if (sr == -1){
				selectedIndex = firstItem;
				repaintItem(firstItem);
				dataChanged();
				return;
			}
			if (ev.key == IKeys.UP){
				if (sr == 0){
					if (selectedIndex < numCols || scrollMode == SCROLL_DONT_SCROLL) 
						return;
					else {
						selectedIndex = -1;
						//repaintItem(cs);
						selectedIndex = cs-numCols;
						doScroll(Vertical,ScrollLower,0);
						dataChanged();
						return;
					}
				}else{
					moveSelected(-numCols);
				}
			}else if (ev.key == IKeys.DOWN){
				if (sr == numRows-1){
					if (((selectedIndex/numCols)+1)*numCols >= tot || scrollMode == SCROLL_DONT_SCROLL)
						return;
					else {
						selectedIndex = -1;
						//repaintItem(cs);
						selectedIndex = cs+numCols;
						doScroll(Vertical,ScrollHigher,0);
						if (selectedIndex >= tot)
							selectedIndex = tot-1;
						repaintItem(selectedIndex);
						dataChanged();
						return;
					}
				}else{
					if (sr != -1){
						if ((sr+1)*numCols+firstItem >= tot) return;
					}
					moveSelected(numCols);
				}
			}else if (ev.key == IKeys.RIGHT){
				if (selectedIndex%numCols == numCols-1) return;
				if (selectedIndex >= tot-1) return;
				moveSelected(1);
			}else if (ev.key == IKeys.LEFT){
				if (selectedIndex%numCols == 0) return;
				moveSelected(-1);
			}
		}else if (ev.isActionKey()){
			doSelectedAction(getSelectedItem());
		}
	}else
		super.onKeyEvent(ev);
}

/* (non-Javadoc)
 * @see eve.ui.ScrollClient#doScroll(int, int, int)
 */
public void doScroll(int which, int action, int value) {
	try{
		int total = countItems();
		int fi = firstItem;
		int maxRow = countTotalRows()-numRows;
		int maxCol = countTotalCols()-numCols;
		int maxFirst = 0;
		if (which == IScroll.Vertical){
			maxFirst = maxRow*numCols;
			switch(action){
			case IScroll.TrackTo:
				firstItem = value*numCols;
				break;
			case IScroll.ScrollHigher:
				firstItem += numCols;
				break;
			case IScroll.ScrollLower:
				firstItem -= numCols;
				break;
			case IScroll.PageHigher:
				firstItem += getTotalItemsOnScreen();
				break;
			case IScroll.PageLower:
				firstItem -= getTotalItemsOnScreen();
				break;
			}
			if (action == IScroll.TrackTo){
				
			}
		}else{
			maxFirst = maxCol*numRows;
		}
		if (firstItem > maxFirst) firstItem = maxFirst;
		if (firstItem < 0) firstItem = 0;
		if (firstItem == fi) return;
		repaintNow();
		if (ss != null)
			ss.checkScrolls();
	}finally{
		notifyStructureChanged();
	}
}

/* (non-Javadoc)
 * @see eve.ui.ScrollClient#getActual(int)
 */
public int getActual(int which) {
	if (which == Vertical)
		return countTotalRows();
	else
		return countTotalCols();
}

/* (non-Javadoc)
 * @see eve.ui.ScrollClient#getVisible(int, int)
 */
public int getVisible(int which, int forVisibleSize) {
	if (which == Vertical){
		return canFit(forVisibleSize,preferredItemHeight,variablePercent);
	}else{
		return canFit(forVisibleSize,preferredItemWidth,variablePercent);
	}
}

/* (non-Javadoc)
 * @see eve.ui.ScrollClient#getCurrent(int)
 */
public int getCurrent(int which) {
	if (which == Vertical){
		return firstItem/numCols;
	}else{
		return firstItem/numRows;
	}
}

/* (non-Javadoc)
 * @see eve.ui.ScrollClient#needScrollBar(int, int)
 */
public boolean needScrollBar(int which, int forVisibleSize) {
	if (scrollMode == SCROLL_VERTICALLY){
		return countItemsThatFit(width,forVisibleSize) <= countItems();
	}else{
		return countItemsThatFit(width,forVisibleSize) <= countItems();
	}
}

/* (non-Javadoc)
 * @see eve.ui.ScrollClient#canGo(int, int, int)
 */
public boolean canGo(int orientation, int direction, int position) {
	// TODO Auto-generated method stub
	return true;
}
protected void doReplaceItemAt(MenuItem it, int atIndex)
{
	items.setElementAt(it,atIndex);
}
protected void doInsertItemAt(MenuItem it, int atIndex)
{
	items.insertElementAt(it,atIndex);
}
protected void doRemoveItemAt(int atIndex)
{
	items.removeElementAt(atIndex);
}
protected void doRemoveAll()
{
	items.removeAllElements();
}
public static final int ADD_UPDATE_SCREEN = 0x1;
public static final int ADD_MAKE_VISIBLE = 0x2|ADD_UPDATE_SCREEN;
public static final int ADD_SELECTED = 0x4|ADD_MAKE_VISIBLE;

public void replaceItem(MenuItem it, int atIndex, boolean updateScreen)
{
	int num = countItems();
	if (atIndex < 0 || atIndex >= num) return;
	doReplaceItemAt(it,atIndex);
	if (!updateScreen) return;
	repaintItem(atIndex);
}
public void removeItem(int atIndex, boolean updateScreen)
{
	int num = countItems();
	if (atIndex < 0 || atIndex >= num) return;
	if (atIndex == selectedIndex) selectedIndex = -1;
	else if (atIndex < selectedIndex) selectedIndex--;
	doRemoveItemAt(atIndex);
	if (!updateScreen) return;
	int tot = getTotalItemsOnScreen();
	num--;
	if (atIndex >= firstItem+tot) return; // After this screen.
	if (firstItem >= num){
		firstItem -= getTotalItemsOnScreen();
		if (firstItem < 0) firstItem = 0;
	}
	repaintNow();
}
/**
 * Add an item at a particular location with options to update the screen.
 * @param it the item to add.
 * @param atIndex the index to add to. If this is greater than the number
 * in the menu or -1, it will be added to the end.
 * @param addOptions any of ADD_UPDATE_SCREEN, ADD_MAKE_VISIBLE, ADD_SELECTED
 * or'ed together. Note that ADD_MAKE_VISIBLE will also do ADD_UPDATE_SCREEN and
 * ADD_SELECTED will also do ADD_MAKE_VISIBLE;
 */
public void addItem(MenuItem it, int atIndex, int addOptions)
{
	if (atIndex > countItems() || atIndex < 0) atIndex = countItems();
	doInsertItemAt(it,atIndex);
	if ((addOptions & ADD_UPDATE_SCREEN) != ADD_UPDATE_SCREEN) return;
	if (atIndex <= selectedIndex) selectedIndex++;
	if (numRows == 0 || numCols == 0) return;
	if (atIndex < firstItem+getTotalItemsOnScreen()){
		// The new one IS visible OR it is before firstItem, so update the screen.
		repaintNow();
	}
	if ((addOptions & ADD_MAKE_VISIBLE) == ADD_MAKE_VISIBLE){
		makeVisible(atIndex);
	}
	if ((addOptions & ADD_SELECTED) == ADD_SELECTED){
		setSelectedItem(atIndex,true);
	}
}
/**
 * Add an item without updating the screen.
 * @param it the item to add.
 */
public void addItem(MenuItem it) 
{
	addItem(it,-1,0);
}
/**
 * This returns true if the menu needs an external "More" press
 * to display another page.
 */
public boolean needsExternalMore()
{
	return scrollMode == SCROLL_DONT_SCROLL;
}
/**
 * This returns true if the menu needs an external "Back" press
 * to display another page.
 */
public boolean needsExternalBack()
{
	return scrollMode == SCROLL_DONT_SCROLL;
}

}
//####################################################
