package eve.ui;
import eve.sys.Event;
import eve.ui.event.MenuEvent;

/**
* A MenuChoice is a type of mChoice which uses a full Menu for the drop
* down slection which can have submenus. This is in contrast to a Choice
* which cannot have submenus in its drop-down selector.
**/
//##################################################################
public class MenuChoice extends Choice{
//##################################################################
{
	useMenuItems = true;
	mustAlwaysDrop = true;
	text = "";
}
/**
* Return the current text. This will be an empty String if nothing is selected.
**/
public String getText() {return text;}
/**
* This sets the text for the MenuChoice, regardless of if it is a valid choice. Use selectItem()
* to select an item only if it is a valid choice.
**/
public void setText(String txt) {text = txt; if (text == null) text = "";repaintNow();}
/**
* This method will select a particular item only if it is a valid choice.
**/
//===================================================================
public void selectItem(String txt) 
//===================================================================
{
	Object obj = findItem(txt,true);
	if (obj instanceof String) setText((String)obj); 
	else if (obj instanceof MenuItem){
		MenuItem mi = (MenuItem)obj;
		if (mi.subMenu == null) setText(mi.label);
	}
}
//public MenuItem selectedItem = null;
/**
* Find the MenuItem that is currently selected. Unlike the ChoiceControl.getSelectedItem()
* this does not use the selectedIndex field and always returns a MenuItem object.
**/
//==================================================================
public Object getSelectedItem()
//==================================================================
{
	if (text == null || text.length() == 0) return null;
	return findItem(text,true);
}
//===================================================================
public void setSelectedItem(MenuItem m)
//===================================================================
{
	if (m == null) setText(null);
	else selectItem(m.label);
}
//==================================================================
protected String getDisplayString() {return text;}
//==================================================================

//==================================================================
//------------------------------------------------------------------
public void onEvent(Event ev)
//------------------------------------------------------------------
{
	//System.out.println("Chosen! "+ev);
	if (ev instanceof MenuEvent && (ev.type == MenuEvent.SELECTED || ev.type == MenuEvent.ABORTED) && ev.target == menu){
		MenuEvent me = (MenuEvent)ev;
		if (ev.type != MenuEvent.ABORTED){
			if (me.selectedItem instanceof String) text = (String)me.selectedItem;
			else text = ((MenuItem)me.selectedItem).label;
		}
		noMenu();
		if (ev.type != MenuEvent.ABORTED) notifyAction();
	}else if (ev.target != menu)
		super.onEvent(ev);
}

//##################################################################
}
//##################################################################


