package eve.ui;

import eve.fx.IImage;

public interface MultiCardPanel extends MultiPanel{

	/**
	 * An option with addCard() that indicates that the icon should be
	 * used alone and not used together with the cardName value.
	 */
	public static final int ADD_OPTION_DONT_SHOW_TEXT_WITH_ICON = 0x1;
	
	public Card addCard(Control toAdd, IImage icon, String cardName, String longCardName, int options);
	public Card addCard(Control toAdd, String cardName, String longCardName);
	public CardPanel getCardPanel();
	/**
	* This returns a Card object holding information about the item added at the
	* specified index.
	**/
	public Card getItem(int index);
	/**
	* This returns a Card object holding information about the item added for the
	* specified control.
	**/
	public Card getItem(Control item);

	/**
	 * Set and/or clear card options - or return the current options  (if
	 * optionsToSet and optionsToClear are zero).
	 * @param optionsToSet the option bits to set - any of the OPTION_XXX values.
	 * @param optionsToClear the option bits to clear - any of the OPTION_XXX values.
	 * @return the new value of the option bits. If both optionsToSet and
	 * optionsToClear are 0, then this will return the current state of the
	 * options without changing any.
	 */
	public int modifyCardOptions(int optionsToSet, int optionsToClear);
	/** A value used with modifyCardOptions() */
	public static final int OPTION_DONT_FOCUS_ON_NEXT_SELECT = 0x1;
	/** A value used with modifyCardOptions() */
	public static final int OPTION_FOCUS_ON_CONTROLS_AFTER_SELECTION_HOT_KEY = 0x2;
	/** A value used with modifyCardOptions() */
	public static final int OPTION_DONT_FOCUS_ON_ANY_SELECT = 0x4;
	/** A value used with modifyCardOptions() */
	public static final int OPTION_AUTO_SCROLL = 0x8;
	/**A value used with getExtraControlArea() **/
	public static int EXTRA_CONTROL_EXTREME = 0x100;
	/**A value used with getExtraControlArea() **/
	public static int EXTRA_CONTROL_LEFT = 0x1;
	/**A value used with getExtraControlArea() **/
	public static int EXTRA_CONTROL_RIGHT = 0x2;
	/**A value used with getExtraControlArea() **/
	public static int EXTRA_CONTROL_TOP = 0x4;
	/**A value used with getExtraControlArea() **/
	public static int EXTRA_CONTROL_BOTTOM = 0x8;
	/**
	 * If this MultiCardPanel supports an extra area for placing controls
	 * which are not within the dynamic display area, then this will return
	 * a CellPanel that you can add controls to which will be displayed in
	 * that area.
	 * @param whichArea one of the EXTRA_CONTROL_XXX values possibly OR'ed with
	 * the EXTRA_CONTROL_EXTREME option.
	 * @return the specified extra area if present - null if not.
	 */
	public CellPanel getExtraControlArea(int whichArea);
	/**
	 * Select the specified card.
	 * @param card
	 */
	public void select(Card card);
}
