package eve.ui;
import eve.sys.Handle;
import eve.sys.HandleStoppedException;
import eve.sys.TimeOut;
import eve.ui.event.ControlEvent;
import eve.util.mString;

//##################################################################
public class ProgressBarForm extends Form implements ProgressDisplay{
//##################################################################

public ProgressBar bar;
public Handle handle;
public TimeOut t = TimeOut.Forever;
Label mainTask;
Label subTask;
Button stop;
public boolean showMainTask = true;
public boolean showSubTask = false;
public boolean horizontalLayout = true;
public boolean showStop = false;
public boolean showBar = true;
public boolean showTaskInBar = false;
public boolean showTimeLeft = false;
public boolean dontExitOnStopPressed = false;
/**
* Set this to false if you don't want the form to close its containing
* Frame upon completion.
**/
public boolean exitOnCompletion = true;

//===================================================================
public ProgressBarForm()
//===================================================================
{
	resizable = false;
	mainTask = new Label(1,30);
	subTask = new Label(1,30);
	bar = new ProgressBar();
	exitButtonDefined = true;
	windowFlagsToClear = Window.FLAG_HAS_CLOSE_BUTTON;
}
public void setMainTask(String mainTask)
{
	this.mainTask.setText(mainTask);
}
//===================================================================
public void make(boolean reMake)
//===================================================================
{
	if (made) return;
	if (showMainTask) 
		if (showSubTask || !showTaskInBar)
			addNext(mainTask).setCell(HSTRETCH);
	endRow();
	if (showSubTask)
		if (!showTaskInBar)
			addNext(subTask).setCell(HSTRETCH);
	endRow();
	CellPanel cp = new CellPanel();
	if (showBar) cp.addNext(bar).setCell(HSTRETCH);
	if (!horizontalLayout) cp.endRow();
	if (showStop){
		cp.addNext((stop = new Button("Cancel")).setHotKey(0,'c'),showBar ? DONTSTRETCH : HSTRETCH,DONTFILL|CENTER);
		Gui.iconize(stop,Form.stop,true,null);
		stop.modify(MouseSensitive|Disabled,0);
		if (handle != null) stop.modify(0,Disabled);
	}
	addLast(cp).setCell(HSTRETCH);
	super.make(reMake);
}
//===================================================================
public synchronized void updateTask()
//===================================================================
{
	//ewe.sys.Vm.debug(""+ewe.sys.Vm.getUsedMemory(false));
	bar.showTimeLeft = showTimeLeft;
	bar.set(handle.progress,handle.startTime);
	if (showSubTask) {
		if (!showTaskInBar) subTask.setText(handle.doing);
		if (showTaskInBar){
			if (mString.compare(bar.doing,handle.doing,false) != 0){
				bar.doing = handle.doing;
				bar.repaint();
			}
		}
	}
}   
//===================================================================
public synchronized void clearTask()
//===================================================================
{
	bar.doing = null;
	bar.set(0,0);
	bar.repaint();
}
//-------------------------------------------------------------------
protected void doSetup(Handle h,String processName,boolean execIt)
//-------------------------------------------------------------------
{
	if (processName == null) processName = h.doing;
	title = processName;
	if (execIt) getTopmostForm().exec();
	handle = h;
	h.start();
	if (stop != null) {
		stop.modify(0,Disabled);
		stop.repaintNow();
	}
	if (showMainTask) {
		mainTask.setText(processName);
		if (!showSubTask && showTaskInBar) {
			bar.doing = processName;
			bar.repaint();
		}
	}
}

//===================================================================
public synchronized void setTask(final Handle h,String processName)
//===================================================================
{
	doSetup(h,processName,false);
	new Thread(){
		public void run(){
			try{
				boolean ch = false;
				synchronized(h){
					while(true){
						updateTask();
						h.changeAccepted();
						try{
							h.waitOn(h.Changed);
						}catch(HandleStoppedException e){
							break;
						}catch(InterruptedException e){}
					}
				}
				updateTask();
			}finally{
				if (((h.check() & h.Stopped) != 0) && exitOnCompletion) 
					getTopmostForm().exit(1);
			}
		}
	}.start();
	//h.callBackOnFlags(this,h.Changed,t.reset());
	//updateTask();
}

/**
 * Execute a ProgressBarForm and wait for it to exit. It will exit when
 * the Handle has stopped.
 * @param h the Handle to monitor.
 * @param processName a name for the process.
 * @return the state of the Handle as returned by h.check().
 */
public int execute(Handle h,String processName)
//===================================================================
{
	doSetup(h,processName,true);
	while(true){
		updateTask();
		if ((h.check() & h.Stopped) != 0) break;
		try{
			h.waitOn(h.Changed);
			h.changeAccepted();
		}catch(Exception e){}
	}	
	if (exitOnCompletion) getTopmostForm().exit(0);
	return h.check();
}
/*
//===================================================================
public void callBack(Object obj)
//===================================================================
{
	if (obj instanceof HandleStatus){
		HandleStatus hs = (HandleStatus)obj;
		updateTask();
		if (hs.stopped){
			if (exitOnCompletion) getTopmostForm().exit(1);
		}else
			hs.handle.callBackOnFlags(this,Handle.Changed,t.reset());
	}
}
*/
//===================================================================
public void onControlEvent(ControlEvent ev)
//===================================================================
{
	if (ev.type == ev.PRESSED && ev.target == stop){
		stopPressed();
	}
	super.onControlEvent(ev);
}
//===================================================================
public void stopPressed()
//===================================================================
{
	handle.stop(0);
	if (!dontExitOnStopPressed)
		exit(IDCANCEL);
}

private static ProgressBarForm displaying = null;
/**
 * This displays a modal progress message without a bar or cancel button. It is a good
 * way of disabling input to a Window while a process is running. To remove the box, simply
 * call clear() or you can call display() again without calling clear to display a different message.
 * @param title The title for the box - if it is null, then no title is displayed.
 * @param doing The message to display.
 * @param parent The parent frame. If it is null then a new window is used.
 * @return 
 */
//===================================================================
public static void display(String title,String doing,Frame parent)
//===================================================================
{
	clear();
	displaying = new ProgressBarForm();
	displaying.moveable = false;
	displaying.showStop = false;
	displaying.showBar = false;
	displaying.mainTask = new Label(doing);
	if (title != null) displaying.title = title;
	else {
		displaying.windowTitle = Form.WINDOW_TITLE_DONT_CHANGE;
		displaying.hasTopBar = false;
		displaying.setBorder(EDGE_SUNKEN,2);
	}
	if (parent != null) displaying.exec(parent,Gui.CENTER_FRAME);
	else displaying.exec();
	displaying.waitUntilShown(500);
}
/**
 * Use this to clear a ProgressBarForm displayed using display().
 */
//===================================================================
public static void clear()
//===================================================================
{
	if (displaying != null) displaying.getTopmostForm().exit(0);
	displaying = null;
}
/**
* A quick way to execute a ProgressBarForm.
**/
//===================================================================
public static Handle execute(String doing,Handle h)
//===================================================================
{
	ProgressBarForm pbf = new ProgressBarForm();
	pbf.bar.setPreferredSize(300,-1);
	pbf.horizontalLayout = false;
	pbf.showStop = 
	pbf.showSubTask =
	pbf.showTaskInBar =
	pbf.showTimeLeft = true;
	pbf.execute(h,doing);
	return h;
}
/**
* A quick way to execute a ProgressBarForm.
**/
//===================================================================
public static Handle executeNoBar(String doing,Handle h,boolean showCancel)
//===================================================================
{
	ProgressBarForm pbf = new ProgressBarForm();
	pbf.bar.setPreferredSize(300,-1);
	pbf.horizontalLayout = false;
	pbf.showStop = showCancel;
	pbf.showBar =
	pbf.showSubTask =
	pbf.showTaskInBar =
	pbf.showTimeLeft = false;
	pbf.execute(h,doing);
	return h;
}

/*
 * This executes the ProgressBarForm and waits until it displays before calling
 * start() on the handle.
 */
public int execute(final Handle h)
{
	this.handle = h;
	bar.monitor(h);
	new Thread(){
		public void run(){
			waitUntilShown(4000);
			h.start();
			h.waitUntilCompletion();
			//System.out.println("Complete!");
			if (true){
				if (h.shouldStop) exit(IDCANCEL);
				else exit(IDOK);
			}
			/*
			else new WindowThreadTask(){
				public void doRun(){
					if (h.shouldStop) exit(IDCANCEL);
					else exit(IDOK);
				}
			}.execute(ProgressBarForm.this);
			*/
		}
	}.start();
	return execute();
}
public Object executeAndGetReturnValue(Handle h)
{
	if (execute(h) == IDCANCEL) return null;
	return h.getFinalReturnValue();
}
//##################################################################
}
//##################################################################

