package eve.ui;
import java.io.IOException;
import java.io.PrintWriter;

import eve.io.Io;
import eve.sys.Time;
import eve.sys.Vm;
import eve.ui.event.ControlEvent;
//##################################################################
public class ReportException extends MessageBox{
//##################################################################
Throwable t;
Button toClip, toFile;
TabbedPanel tp;

public ReportException()
{
	this(new Exception("A Test Exception"));
}
//===================================================================
public ReportException(Throwable t)
//===================================================================
{
	this(t,null,null,false);
}
/**
* "message" is the message that is shown before the exception class name. If this is null
* a standard message is shown.<p>
* "advice" is the message that is shown after the exception class name. If this is null
* no advice message is shown.<p>
* If "showContinue" is true then the buttons "Continue" and "Cancel" are shown, otherwise
* a single "Close" button is shown.
**/
//===================================================================
public ReportException(Throwable t,String message,String advice,boolean showContinue)
//===================================================================
{
	super("Application Error",
	(message == null ? "An error occured in the program:" : message)+"\n\n"+
	t.getClass().getName()+": "+
	t.getMessage()+
	(advice == null ? "" : "\n\n"+advice),0);
	windowFlagsToSet |= Window.FLAG_MAXIMIZE_ON_PDA;
	this.t = t;
	tp = new TabbedPanel();
	tp.modifyCardOptions(0,tp.OPTION_AUTO_SCROLL);
	addLast(tp);
	tp.addItem(addTo = new CellPanel(),"Error",null);
	TextPad ma = new TextPad(10,50);
	ma.wrapToScreenSize = false;
	//ma.modify(DisplayOnly,0);
	ma.setText(Vm.getStackTrace(t).replace('\t',' '));
	//ScrollBarPanel sbp = new ScrollBarPanel(ma);
	tp.addItem(ma.setAsStaticDisplay(true),"Details",null);
	addTo.setBorder(GuiStyle.standardEdge,2);
	if (Gui.hasSoftKeys){
		makeSoftKeys().
			setKey(1,(showContinue ? "Continue|":"Close|")+EXIT_IDOK,tick,null).
			setMenu(2,"Actions",null,(showContinue ? "Cancel|"+EXIT_IDCANCEL+"|" : "")+"Details|DETAILS|Error|ERROR|-|-|To Clipboard|TO_CLIPBOARD");
	}else{
		addButton(toClip = new Button("To Clipboard"));
		addButton(toFile = new Button("To File"));
		if (showContinue){
			addButton(ok = new Button("Continue"));
			addButton(cancel = new Button("Cancel"));
		}else{
			addButton(ok = new Button("Close"));
		}
	}
	windowFlagsToClear = Window.FLAG_HAS_CLOSE_BUTTON;
	resizable = true;
}
public void onControlEvent(ControlEvent ev)
{
	if (ev.type == ev.PRESSED){
		if (ev.target == toClip){
			Vm.setClipboardText(Vm.getStackTrace(t).replace('\t',' '));
			Gui.flashMessage("Saved to clipboard. ",this);
		}else if (ev.target == toFile){
				PrintWriter pw = null;
				String f = "\\EveStackTrace.txt";
				try{pw = new PrintWriter(Io.newFileOutputStream(f,true));
				}catch(IOException e){}
				if (pw == null) {
					f = "EveStackTrace.txt";
					try{pw = new PrintWriter(Io.newFileOutputStream(f,true));
					}catch(IOException e){}
				}
				if (pw == null){
					Gui.flashMessage("Could not save stack trace.",this);
				}else{
					try{
						pw.println("-----------------------");
						pw.println(new Time().format("dd-MMM-yyyy, HH:mm:ss"));
						pw.println(Vm.getStackTrace(t).replace('\t',' '));
						Gui.flashMessage("Saved to: "+f,1500,this,Gui.FLASH_BEEP);
					}
					catch(Exception e)
					{
						Gui.flashMessage("Could not save stack trace.",this);
					}
					finally
					{
						pw.close();
					}
				}
		}else super.onControlEvent(ev);
	}
}
public boolean handleAction(String action)
{
	if (super.handleAction(action)) return true;
	if (action.equals("ERROR")) {
		tp.select(0);
		Gui.takeFocus(tp,ByRequest);
	}
	else if (action.equals("DETAILS")) {
		tp.select(1);
		Gui.takeFocus(tp,ByRequest);
	}
	else if (action.equals("TO_CLIPBOARD"))Vm.setClipboardText(Vm.getStackTrace(t).replace('\t',' '));
	else return false;
	return true;
}
//##################################################################
}
//##################################################################

