package eve.ui;
import eve.fx.Color;
import eve.fx.Dimension;
import eve.fx.Font;
/**
* This is a CarrierFrame which scrolls its contents.
**/
//##################################################################
public class ScrollableCarrierFrame extends CarrierFrame {
//##################################################################
/**
* This is the ScrollablePanel which the frame contains.
**/
public ScrollablePanel sp;
/**
* This is the ScrollableHolder which is within the ScrollablePanel sp.
**/
public ScrollClient scrollClient;

/**
* If a title is used, this will be the control that holds the title text. Usually it
* will be an mLabel unless makeTitleControl() is changed.
**/
public Control titleControl;
/**
* Create the ScrollableCarrierFrame.
**/
//==================================================================
public ScrollableCarrierFrame(Control client,Control owner)
//==================================================================
{
	this(client,owner,false);
}
//==================================================================
public ScrollableCarrierFrame(Control client,Control owner,boolean useScrollBars)
//==================================================================
{
	this(client,owner,useScrollBars,null);
}
//==================================================================
public ScrollableCarrierFrame(Control client,Control owner,boolean useScrollBars,String title)
//==================================================================
{
	super(owner);
	this.client = client;
	if (client != null) font = client.getFont();
	Control ts = client;
	if (!(ts instanceof ScrollClient)){
		ScrollableHolder sh = new ScrollableHolder(client);
		backGround = client.getBackground();
		foreGround = client.getForeground();
		//sh.stretchComponent = true;
		sh.modify(TakeControlEvents,0);
		ts = sh;
	}
	scrollClient = (ScrollClient)ts;
	sp = useScrollBars ? (ScrollablePanel)new ScrollBarPanel((ScrollClient)ts) : (ScrollablePanel)new UpDownScroller((ScrollClient)ts);
	content = sp;
	sp.modify(TakeControlEvents,0);
	sp.modifyScrollers(DrawFlat,0);
	//sp.backGround = Color.White;
	if (title != null){
		titleControl = makeTitleControl(title);
		if (titleControl != null)
		contents.addLast(titleControl).setCell(HSTRETCH);
	}
	contents.addLast(sp);
}
//-------------------------------------------------------------------
protected Control makeTitleControl(String title)
//-------------------------------------------------------------------
{
	if (title == null) return null;
	Label l = new Label(title);
	l.anchor = CENTER;
	l.backGround = Color.LightGray;
	l.setBorder(BDR_OUTLINE|BF_BOTTOM,1);
	l.font = getFont().changeStyle(Font.BOLD);
	return l;
}
//-------------------------------------------------------------------
protected Dimension getContentsSize(Dimension d)
//-------------------------------------------------------------------
{
	d= super.getContentsSize(d);
	if (allowTitleExpansion && titleControl != null){
		getPreferredSize(d);
	}
	return d;
	/*
		return new Rect(getPreferredSize(null));
	else
		return super.getContentsSize();
		*/
}

/**
Set this true if the addition of a title should allow the entire frame to increase its
size. This is used, for example, if a menu is being shown, but the menu is less than 5 items high.
**/
public boolean allowTitleExpansion = false;

//##################################################################
}
//##################################################################

