/*
 * Created on May 29, 2007
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui.data;

import java.net.Socket;

import eve.io.Io;
import eve.net.Net;
import eve.server.RemoteAppLoader;
import eve.sys.Convert;
import eve.sys.Device;
import eve.sys.Event;
import eve.sys.EventListener;
import eve.sys.Handle;
import eve.sys.TimeOut;
import eve.sys.mThread;
import eve.ui.Application;
import eve.ui.Control;
import eve.ui.Form;
import eve.ui.Gui;
import eve.ui.Label;
import eve.ui.List;
import eve.ui.ProgressBarForm;
import eve.ui.ReportException;
import eve.ui.event.ControlEvent;
import eve.util.mString;

/**
 * @author Michael L Brereton
 *
 */
//####################################################
public class AppLoader extends RemoteAppLoader implements EventListener{

	RemoteAppLoader loader;

	private String configName;
	
	public AppLoader(String configName)
	{
		loader = this;
		this.configName = configName;
		if (configName != null) try{
			String got = Io.getConfigInfo("EweSoft\\Eve\\AppLoader");
			if (got != null) loader.textDecode(got);
		}catch(Exception e){}
	}
	
	protected String selectApplication(String[] availableApps)
	{
		if (availableApps == null || availableApps.length == 0) return null;
		Form f = new Form();
		f.maximizeOnPDA();
		f.title = "Select Application To Run";
		f.addLast(new Label("Select Application To Run")).setCell(f.HSTRETCH);
		List list = new List(10,40,false);
		f.addLast(list.getScrollablePanel());
		list.addItems(availableApps);
		list.select(0);
		list.addListener(this);
		Gui.setOKCancel(f);
		if (f.execute() == f.IDCANCEL || list.selectedIndex == -1) return null;
		return availableApps[list.selectedIndex];
	}
	public void onEvent(Event ev)
	{
		if ((ev.target instanceof List) && (ev instanceof ControlEvent) && ev.type == ControlEvent.PRESSED){
			Control c = (Control)ev.target;
			while(!(c instanceof Form) && (c != null))
				c = c.getParent();
			if (c != null) ((Form)c).exit(Form.IDOK);
		}
	}
	public Editor getEditor(int options)
	{
		Editor ed = new Editor(loader);
		EditorStack es = new EditorStack();
		es.inputLength = 30;
		if (Device.isMobile()) es.doubleLined = true;
		es.addInputs(ed,"hostName,hostPort,applicationName");
		ed.addLast(es);
		return ed;
	}

private ProgressBarForm pbf;

protected void loaderProgress(int progress,Handle h)
{
	switch(progress){
	case PROGRESS_LOADING_EVE_OR_ZIP:
		ProgressBarForm.display("Loading Eve/Zip File","Loading Eve/Zip File",null);
		break;
	case PROGRESS_LOADED_STARTING_CLASS:
	case PROGRESS_LOADED_EVE_OR_ZIP:
		ProgressBarForm.clear();
		break;
	case PROGRESS_LOADING_STARTING_CLASS:
		ProgressBarForm.display("Loading starting Class","Loading starting Class",null);
		mThread.nap(500);
		break;
		//ProgressBarForm.display()
	default:
		break;
	}
}

public boolean runClient(boolean saveConfig,boolean showException) throws Exception
{
	Handle h = Net.newSocket(loader.hostName,loader.hostPort);
	try{
		if (!h.waitUntilStopped(new TimeOut(1000)))
			ProgressBarForm.executeNoBar("Connecting...",
					h.getStoppableWaitOnSuccess(),true);
	}catch(Exception e){}
	//Device.tryMessageBox("One","One",false);
	if ((h.check() & h.Success) == 0) {
	//if (!(h.returnValue instanceof Socket)) {
		if (h.error != null  && showException) 
			new ReportException(h.error,"Could not connect to server.",null,false).execute();
		return false;
	}
	//Device.tryMessageBox("One","RunClient",false);
	return loader.runClient((Socket)h.returnValue,saveConfig ? configName : null);
}
/**
 * Open a dialog box and run the client if selected by the user.
 * @param saveConfig true to save the config info.
 * @throws Exception if an error occurs during execution of the app.
 */
public boolean runLoader(boolean saveConfig) throws Exception
{
	while(true){
		//hostName = "192.168.1.1";
		Editor ed = getEditor(0);
		ed.title = "Connect to Application Server";
		Gui.setOKCancel(ed);
		if (ed.execute() == ed.IDCANCEL) return false;
		if (runClient(saveConfig,true)) return true;
	}
}
public static void main(String args[])
{
	Application.startApplication(args);
	AppLoader rl = new AppLoader("EweSoft\\Eve\\AppLoader");
	
	if (false && args.length == 0){
		try{
			if (rl.runLoader(true))
				return;
		}catch(Throwable e){
			new ReportException(e).execute();
		}
	}else{
		boolean runDialog = args.length == 0;
		int i = 0;
		for (i = 0; i<args.length; i++){
			String a = args[i];
			if (a.charAt(0) != '-' && a.charAt(0) != '/') break;
			if (a.equalsIgnoreCase("-prompt") || a.equalsIgnoreCase("-dialog"))
				runDialog = true;
			if (a.equalsIgnoreCase("-recent"))
				;//runDialog = false;
		}
		boolean showException = runDialog;
		if (i < args.length){
			rl.hostName = args[i++];
			rl.hostPort = 2000;
			if (rl.hostName.indexOf(':') != -1){
				rl.hostPort = Convert.toInt(mString.rightOf(rl.hostName,':'));
				rl.hostName = mString.leftOf(rl.hostName,':');
			}
		}
		//rl.applicationName = "";
		if (i < args.length){
			rl.applicationName = args[i++];
		}
		while(true){
			try{
				if (runDialog){
					Editor ed = rl.getEditor(0);
					ed.title = "Connect to Application Server";
					Gui.setOKCancel(ed);
					if (ed.execute() == ed.IDCANCEL) Application.exit(0);
				}
				runDialog = true;
				if (rl.runClient(true,showException))
					return;
				showException = true;
			}catch(Throwable e){
				if (showException)
					new ReportException(e).execute();
				showException = true;
			}
		}
	}
	Application.exit(0);
}
}
//####################################################
