package eve.ui.data;
import eve.ui.CheckBox;
import eve.ui.CheckBoxGroup;
import eve.ui.InputStack;
import eve.util.mString;

//##################################################################
public class EditorStack extends InputStack {
//##################################################################
//===================================================================
public void addInputs(Editor ed,String promptsAndFields)
//===================================================================
{
	if (promptsAndFields.indexOf('|') == -1) addInputFields(ed,promptsAndFields);
	else{
		String [] pf = mString.split(promptsAndFields,'|');
		for (int i = 0; i<pf.length-1; i+=2)
			ed.addField(addInput(pf[i],""),pf[i+1]);
	}
}
/**
 * Add a set of inputs given the field names. The prompts will be created from the field names.
 * @param ed An editor to add the fields to.
 * @param fields The field names - the prompts will be derived from them.
 */
//===================================================================
public void addInputFields(Editor ed,String fields)
//===================================================================
{
	String [] pf = mString.split(fields,fields.indexOf('|') == -1 ? ',' : '|');
	for (int i = 0; i<pf.length; i++)
		ed.addField(addInput(nameToPrompt(pf[i])+":",""),pf[i]);
}
//===================================================================
public void addChecks(Editor ed,String promptsAndFields,boolean exclusive)
//===================================================================
{
	addChecks(ed,promptsAndFields,exclusive,false);
}
//===================================================================
public void addChecks(Editor ed,String promptsAndFields,boolean exclusive,boolean checkOnLeft)
//===================================================================
{
	String [] pf = mString.split(promptsAndFields,'|');
	CheckBoxGroup cg = exclusive ? new CheckBoxGroup() : null;
	for (int i = 0; i<pf.length-1; i+=2){
		CheckBox mc;
		if (checkOnLeft) add(mc = new CheckBox(pf[i]),null);
		else mc = addCheckBox(pf[i]);
		ed.addField(mc,pf[i+1]);
		if (cg != null) mc.setGroup(cg);
	}
}
//===================================================================
public void addCheckFields(Editor ed,String fields,boolean exclusive,boolean checkOnLeft)
//===================================================================
{
	String [] pf = mString.split(fields,fields.indexOf('|') == -1 ? ',' : '|');
	CheckBoxGroup cg = exclusive ? new CheckBoxGroup() : null;
	for (int i = 0; i<pf.length; i++){
		CheckBox mc;
		String pr = nameToPrompt(pf[i]);
		if (checkOnLeft) add(mc = new CheckBox(pr),null);
		else mc = addCheckBox(pr);
		ed.addField(mc,pf[i]);
		if (cg != null) mc.setGroup(cg);
	}
}
//##################################################################
}
//##################################################################

