package eve.ui.data;
import java.util.Iterator;
import java.util.Vector;

import eve.data.DataUtils;
import eve.data.MutableTreeNode;
import eve.data.TreeNode;
import eve.util.ObjectIterator;

//##################################################################
public class LiveTreeNode extends LiveObject implements MutableTreeNode{
//##################################################################
/**
* The parent of this Node.
**/
protected TreeNode parent;
/**
* The children of this Node. This will be NULL until children are added.
**/
protected Vector children;

//===================================================================
public TreeNode getChild(int index)
//===================================================================
{
	if (children == null) return null;
	if (index >= children.size() || index < 0) return null;
	return (TreeNode)children.get(index);
}
//===================================================================
public Iterator getChildren()
//===================================================================
{
	if (children == null) return new ObjectIterator(null);
	else return children.iterator();
}
//===================================================================
public int getChildCount()
//===================================================================
{
	if (children == null) return 0;
	return children.size();
}
//===================================================================
public TreeNode getParent() {return parent;}
//===================================================================
public void setParent(TreeNode p) {parent = p;}
//===================================================================
public void removeChild(MutableTreeNode ch)
//===================================================================
{
	if (ch == null) return;
	if (children == null) return;
	int idx = children.indexOf(ch);
	if (idx == -1) return;
	children.removeElementAt(idx);
	ch.setParent(null);
	if (children.size() == 0) children = null;
}
//===================================================================
public void addChild(MutableTreeNode ch)
//===================================================================
{
	if (children == null) children = new Vector();
	insertChild(ch,children.size());
}
//===================================================================
public void insertChild(MutableTreeNode ch,int index)
//===================================================================
{
	if (ch == null) return;
	if (children == null) children = new Vector();
	if (children.indexOf(ch) != -1) children.remove(ch);
	if (index < 0 || index > children.size()) index = children.size();
	children.insertElementAt(ch,index);
	ch.setParent(this);
}
//===================================================================
public int indexOfChild(TreeNode child)
//===================================================================
{
	if (children == null) return -1;
	return children.indexOf(child);
}
//===================================================================
public int [] addressOfChild(TreeNode child)
//===================================================================
{
	return DataUtils.addressOfChild(this,child);
}
//===================================================================
public TreeNode getChildAt(int [] address)
//===================================================================
{
	return DataUtils.getChildAt(this,address);
}
/*
//===================================================================
public static TreeNode getChildAt(TreeNode parent,int [] address)
//===================================================================
{
	if (address == null) return null;
	TreeNode n = parent;
	for (int i = 0; i<address.length && n != null; i++)
		n = n.getChild(address[i]);
	return n;
}
//===================================================================
public static int [] addressOfChild(TreeNode parent,TreeNode child)
//===================================================================
{
	TreeNode ch = child;
	int i;
	for (i = 0;child != parent && child != null;i++) child = child.getParent();
	if (child == null) return null; //Child not in parent.
	int [] ret = new int[i];
	child = ch;
	for (int j = 0;child != parent;j++){
		TreeNode p = child.getParent();
		ret[i-j-1] = p.indexOfChild(child);
		child = p;
	}
	return ret;
}
*/
/*
//===================================================================
public static boolean isNamed(TreeNode node,String name)
//===================================================================
{
	if (node == null) return false;
	String nm = mString.removeTrailingSlash((node instanceof LiveData) ? ((LiveData)node).getName() : node.toString());
	if (nm == null) return false;
	return nm.equalsIgnoreCase(name);
}

//===================================================================
public static TreeNode findNamedChild(TreeNode parent,String name)
//===================================================================
{
	if (parent == null) return null;
	int n = parent.getChildCount();
	for (int i = 0; i<n; i++){
		TreeNode c = parent.getChild(i);
		if (isNamed(c,name)) return c;
	}
	return null;
}
*/
/**
* This tries to find the address of a child given its path from the parent.
**/
/*
//===================================================================
public static int [] addressOfChild(TreeNode parent,String childPath)
//===================================================================
{
	if (childPath == null || parent == null) return null;
	childPath = mString.removeTrailingSlash(childPath.replace('\\','/'));
	String [] all = mString.split(childPath,'/');
	if (all != null) if (all.length == 0) all = null;
	if (all == null) all = new String[]{""};
	
	TreeNode cur = parent;
	for (int i = 0; i<all.length; i++){
		cur.expand();
		cur = findNamedChild(cur,all[i]);
		if (cur == null) return null;
	}
	return DataUtils.addressOfChild(parent,cur);
}
//===================================================================
public static TreeNode getSibling(TreeNode child,int change)
//===================================================================
{
	TreeNode p = child.getParent();
	if (p == null) return null;
	return p.getChild(p.indexOfChild(child)+change);
}
//===================================================================
public static TreeNode getParent(TreeNode child,int levels)
//===================================================================
{
	for (int i = 0; i<levels && child != null; i++) child = child.getParent();
	return child;
}
*/

public boolean canExpand() {return getChildCount() != 0;}
public boolean expand(){return true;}
public boolean isLeaf() {return getChildCount() == 0;}
public boolean collapse() {return true;}
//##################################################################
}
//##################################################################


