package eve.ui.data;
import java.io.IOException;

import eve.io.JavaUtf8Codec;
import eve.ui.Application;
import eve.ui.CellPanel;
import eve.ui.CheckBox;
import eve.ui.Control;
import eve.ui.ControlGroup;
import eve.ui.Menu;
import eve.ui.MenuBar;
import eve.ui.PullDownMenu;
import eve.ui.ScrollBarPanel;
import eve.ui.TextPad;
import eve.ui.WindowThreadTask;

//##################################################################
public class Notepad extends AppForm{
//##################################################################

public String theText = "";
/*
new String(new char[]{
0xc0, 0xd0, 0xe0, 0xf0
});
*/
public FontChooser fontChooser = new FontChooser();
public TextPad textPad;
public FileSaver saver = new FileSaver();

protected static String [] toolImages;
public boolean autoWrap = true;
protected Menu fileMenu;

//===================================================================
public Notepad()
//===================================================================
{
	this(new TextPad(20,60));
}
//===================================================================
public Notepad(final TextPad textPad)
//===================================================================
{
	super(true,true);
	this.textPad = textPad;
	textPad.dontWantPopup = true;
	saver.textCodec = new JavaUtf8Codec(JavaUtf8Codec.STRIP_CR);
	saver.dontAppendLineFeed = true;
	acceptsDroppedFiles = true;
	resizeOnSIP = true;
	title = "Eve Notepad";
	saver.setDefaultFileType("txt","Text Files.");
	textPad.font = Application.findFont("fixed");
	textPad.minYScroll = 0;
	textPad.inputState |= textPad.STATE_AUTOTAB;
	addField(textPad,"theText");
	//textPad.modify(DisplayOnly,0);
	ScrollBarPanel sp = new ScrollBarPanel(textPad);
	data.addLast(sp == null ? (Control)textPad : (Control)sp);
	MenuBar mb = menus;
	Menu m = setupStandardFileCommands(SHOW_NEW_BUTTON|SHOW_OPEN_BUTTON|SHOW_SAVE_BUTTON,tools,"Text Document");
	//Menu m2 = new Menu(new String[]{"One","Two"},"What"); m.addItem(m2);
	PullDownMenu pdm = mb.addMenu(m,"File");
	m = new Menu(textPad.getTextPadMenu(null));
	pdm = new PullDownMenu("Edit",m){
		//===================================================================
		public boolean checkMenu(Menu m)
		//===================================================================
		{
			return textPad.checkClipboardOperations(m);
		}
		//===================================================================
		public void popupMenuEvent(Object selectedItem)
		//===================================================================
		{
			textPad.popupMenuEvent(selectedItem);
			//textPad.takeFocus(ByRequest);
		}
		};
	mb.addMenu(pdm);
	tools.modifyAll(NoFocus|MouseSensitive,TakesKeyFocus,true);
	if (true){
		CellPanel tl = new CellPanel();
		tl.addLast(addField(fontChooser.getEditor(0),"fontChooser")).setCell(HSHRINK);
		tl.addLast(addField(new CheckBox("Auto-wrap Lines"),"autoWrap")).setCell(HSHRINK);
		//tl.addLast(new DateTimeInput());
		tabs.addCard(tl,"Tools",null).iconize("eve/SmallConfig.png");
		new ControlGroup().addExposeGroup(tabs);
		tl.modifyAll(KeepSIP,0);
	}
	//firstFocus = textPad;
}
//===================================================================
public void make(boolean reMake)
//===================================================================
{
	fontChooser.fromFont(textPad.getFont());
	super.make(reMake);
}
//===================================================================
public boolean checkChange()
//===================================================================
{
	if (textPad.hasChanged()) {
		fromControls("theText");
		saver.hasChanged = true;
		textPad.updateData();
	}
	return saver.hasChanged;
}
//===================================================================
public void action(String fieldName,Editor ed)
//===================================================================
{
	checkChange();
	try{
	if (fieldName.equals("New")){
		if (saver.checkSave(theText,getFrame())){
			theText = "";
			saver.newData(null);
			toControls("theText");
		}
	//}else if (fieldName.equals("Extra")){
		//fileMenu.addItem(new MenuItem("Another One"));
	}else if (fieldName.equals("Exit")){
			exit(0);
	}else if (fieldName.equals("Save As")){
			saver.save(true,theText,getFrame());
	}else if (fieldName.equals("Save")){
		saver.save(false,theText,getFrame());
	}else if (fieldName.equals("Open")){
		String got = saver.open(theText,getFrame());
		if (got == null) return;
		theText = got;
		toControls("theText");
	}
	}catch(IOException e){}
}

public boolean canExit(int code)
{
	//Vm.debugStackTrace();
	checkChange();
	if (!saver.checkSave(theText,getFrame())) return false;
	return super.canExit(code);
}
			

//===================================================================
public boolean open(String fileName) throws IOException
//===================================================================
{
	String got = saver.openText(fileName,getFrame());
	if (got == null) return false;
	theText = got;
	toControls("theText");
	saver.setHasChanged(false);
	return true;
}
//===================================================================
public void update()
//===================================================================
{
		textPad.update();
		textPad.repaintNow();
}
//===================================================================
public void fieldChanged(String fieldName,Editor ed)
//===================================================================
{
	if (fieldName.equals("theText")){
		saver.hasChanged = true;
	}else if (fieldName.equals("autoWrap")){
		textPad.wrapToScreenSize = autoWrap;
		update();
	}else if (fieldName.equals("fontChooser")){
		textPad.font = fontChooser.toFont();
		update();
	}
}
//===================================================================
public void filesDropped(String[] fileName)
//===================================================================
{
	//ewe.sys.Vm.debug("FD: "+fileName);
	checkChange();
	if (!saver.checkSave(theText,getFrame())) return;
	try{
		open(fileName[0]);
	}catch(IOException e){}
}
/*
public static void eveMain(final String args[])
{
	if (true)
	Application.setApplicationHandler(new IApplicationHandler(){

		public boolean handleMobileAppClosing(Window target, int eventFlags) {
			Device.tryMessageBox("Error","Don't use that to exit!",true);
			return true;
		}
		
	}
			);
	//Device.setScreenRotation(Device.ROTATION_90);
}
*/
//===================================================================
public static void main(final String args[])
//===================================================================
{
	Application.startApplication(args);
	//Vm.modifyVmFlags(Vm.VM_FLAG_SIP_DISABLED);
	//Vm.setParameter(Vm.TURN_ON_VM_FLAG_BITS, Vm.VM_FLAG_SIP_DISABLED);
	String toOpen = null;
	if (args.length != 0) toOpen = args[0];
	final Notepad np = new Notepad();
	np.exitSystemOnClose = true;
	np.execute();
	if (toOpen != null) 
		new WindowThreadTask(){
		public void doRun(){
			try{
				np.open(args[0]);
			}catch(IOException e){
				//new ReportException(e).execute();
			}
		}
	}.run(np,false);
}
/*
public static void eveMain(String args[])
{
	new Application().create(new Rect(10,10,10,10),"My App",0,0,null);
}
*/
//##################################################################
}
//##################################################################

