package eve.ui.data;
import eve.io.SerialPort;
import eve.io.SerialPortSpecs;
import eve.sys.FieldTransfer;
import eve.ui.Button;
import eve.ui.CellPanel;
import eve.ui.Choice;
import eve.ui.ComboBox;
import eve.ui.InputStack;
import eve.ui.IntChoice;
import eve.util.mString;
/**
* SerialPortOptions is a single object that specifies all the currently supported Serial Port
* options. To open a Serial Port, create one of these objects, setup the 
**/
//##################################################################
public class SerialPortOptions extends SerialPortSpecs implements FieldListener{
//##################################################################
/**
* This is an option when calling getEditor(int type) on this object.
**/
public static final int ADVANCED_EDITOR = 0x1;

/**
 * This is initially null. You can call enumerateAvailablePorts() for it enumerate
 * the ports and set this value - or you can set it explicitly. You can leave
 * it as null as well.
 */
public String[] availablePorts;
/**
 * Enumerate the available ports and set the availablePorts member to that
 * value.
 * @param setToFirst if this is true then the "portName" member is set to
 * the first available port.
 * @return the available ports
 */
public String[] enumerateAvailablePorts(boolean setToFirst)
{
	availablePorts = SerialPort.enumerateAvailablePorts();
	if (setToFirst && availablePorts != null && availablePorts.length != 0)
		portName = availablePorts[0];
	return availablePorts;
}
//===================================================================
public Editor getEditor(int whichEditor)
//===================================================================
{
	Editor ed = new Editor();
	ed.objectClass = getClass();
	ed.sampleObject = this;
	ed.setObject(this);
	ed.title = "Serial Port Options";
	InputStack is = new InputStack();
	ed.addLast(is).setCell(ed.HSTRETCH);
	CellPanel cp = new CellPanel();
	ComboBox ports = new ComboBox();
	ed.addField(cp.addNext(ports).setCell(ed.HSTRETCH),"portName");
	if (availablePorts != null){
		ports.choice.items.clear();
		ports.choice.addItems(availablePorts);
	}
	ed.addField(cp.addLast(new Button("Update Ports$u")).setCell(ed.DONTSTRETCH),"update");
	is.add(cp,"Port:$p");
	ComboBox cb = new ComboBox();
	is.add(ed.addField(cb,"baudRate"),"Baud:$b");
	cb.choice.addItems(mString.split("110|300|1200|2400|4800|9600|19200|38400|57600|115200"));
	if ((whichEditor & ADVANCED_EDITOR) != 0){
		Choice mc = new IntChoice(new int[]{4,5,6,7,8},8);
		is.add(ed.addField(mc,"bits"),"Data bits:$d");
		mc = new Choice(mString.split("None|Odd|Even|Mark|Space"),0);
		is.add(ed.addField(mc,"parity"),"Parity:$y");
		mc = new IntChoice(new int[]{1,2},1);
		is.add(ed.addField(mc,"stopBits"),"Stop bits:$s");
	}
	return ed;
	//ed.addField(panel.addLast(new mButton("Connect!")).setCell(HSTRETCH),"connect");
}

//===================================================================
public void action(String field,Editor ed)
//===================================================================
{
	if (field.equals("update")){
		ComboBox cb = (ComboBox)ed.findFieldTransfer("portName").dataInterface;
		Choice ports = cb.choice;
		String all[] = SerialPort.enumerateAvailablePorts();
		ports.items.clear();
		ports.addItems(all);
		ports.updateItems();
		if (all.length != 0) portName = all[0];
		ed.toControls("portName");
	}
}
//===================================================================
public void action(FieldTransfer ft,Editor ed) {action(ft.fieldName,ed);}
//===================================================================
//===================================================================
public void fieldChanged(FieldTransfer ft,Editor ed){}
//===================================================================
//===================================================================
public void fieldEvent(FieldTransfer ft,Editor ed,Object event) {}
//===================================================================

//##################################################################
}
//##################################################################

