/*
 * Created on Feb 12, 2007
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui.launcher;

import eve.fx.IImage;
import eve.io.File;
import eve.io.FileChooserParameters;
import eve.sys.Registry;
import eve.sys.Type;
import eve.sys.Vm;
import eve.ui.Application;
import eve.ui.Form;
import eve.ui.Frame;
import eve.ui.Gui;
import eve.ui.IconMenu;
import eve.ui.Menu;
import eve.ui.MenuItem;
import eve.ui.MessageBox;
import eve.ui.ReportException;
import eve.ui.SoftKeyBar;
import eve.ui.VMOptions;
import eve.ui.data.AppLoader;
import eve.ui.data.Editor;
//import eve.ui.table.data.registry.RegistryViewForm;

/**
 * @author Michael L Brereton
 *
 */
//####################################################
public class AppLauncher extends IconMenu{
	
public static VMApps apps = new VMApps();
public static IImage defaultIcon = loadImage("eve/eve48.png"); 
Type registryView;

protected boolean showHoldDownIndicator(int x, int y)
{
	return true;
}

private MenuItem makeItemFor(VMApp app)
{
	IImage im = app.image;
	if (im == null) im = defaultIcon;
	return makeItem(im,app.name);
}
public AppLauncher(boolean showExtrasInMenu)
{
	//preferredItemHeight = preferredItemWidth = preferredIconHeight = preferredIconWidth = 48;
	if (displayMode != DISPLAY_LINES && Gui.getScreenSize().height <= 240){
		preferredIconHeight = preferredIconWidth = 32;
		preferredItemHeight -= 16;
	}
	apps.read();
	for (int i = 0; i<apps.apps.size(); i++){
		VMApp app = (VMApp)apps.apps.get(i);
		add(app,false);
	}
	this.showExtrasInMenu = showExtrasInMenu;
	registryView = new Type("eve.ui.table.data.registry.RegistryViewForm");
}

public boolean showExtrasInMenu;

private final String server_action = "server";
private final String app_server_action = "appServer";
private final String app_loader_action = "appLoader";
private final String remove_installation_action = "removeInstallation";
private final String add_action = "add";
private final String edit_action = "edit";
private final String delete_action = "delete";
private final String registry_action = "registry";
private final String run_action = "run";
private final String options_action = "options";
private final String about_action = "about";
private final String unload_action = "unload";
private final String browse_action = "browse";
private final String top_action = "top";

protected void itemAction(int sel)
{
	VMApp app = appAt(sel);
	if (app != null) app.run();
}
public boolean addToContextMenu(Menu dest, int selectedItem)
{
	Menu d = dest;
	if (showExtrasInMenu) dest = new Menu();
	if (selectedItem != -1){
		dest.addItem(new MenuItem("Run App",run_action));
		dest.addItem(new MenuItem("Move to Top",top_action));
		dest.addItem(new MenuItem("Edit App",edit_action));
		dest.addItem(new MenuItem("Delete App",delete_action));
		dest.addItem("-");
	}
	dest.addItem(new MenuItem("Add App",add_action));
	Menu apps = dest;
	dest = d;
	if (showExtrasInMenu){
		SoftKeyBar.fixMenu(apps);
		dest.addItem(new MenuItem("Apps",0,apps));
		String s = "File Browser|"+browse_action;
		if (VMLauncher.serverConsole != null) s += "|Server Console|"+server_action;
		if (registryView.exists() && Registry.getLocalRegistry() != null) s += "|Registry View|"+registry_action;
		s += "|Remote App Server|"+app_server_action;
		s += "|Remote App Loader|"+app_loader_action;
		s += "|Remove Installation|"+remove_installation_action;
		SoftKeyBar.insertAsSubMenu(dest,"Extras",s);
		s = "VM Options|"+options_action+
		//"|Unload From Memory|"+unload_action+
		"|About Eve VM|"+about_action;
		SoftKeyBar.insertAsSubMenu(dest,"Eve VM Control",s);
	}
	return true;
}

//===================================================================
boolean editApp(VMApp app,Frame parent)
//===================================================================
{
	if (app == null) return false;
	VMApp ed = (VMApp)app.getCopy();
	Form f = ed.getEditor(0);
	Gui.setOKCancel(f);
	if (f.execute(parent,Gui.CENTER_FRAME) != f.IDOK) return false;
	app.copyFrom(ed);
	return true;
}
//===================================================================
public boolean add(String pathName,Frame parent)
//===================================================================
{
	VMApp app = new VMApp();
	if (!app.makeFrom(pathName,parent)) return false;
	if (!editApp(app,parent)) return false;
	add(app,true);
	apps.apps.add(app);
	return true;
}
public void doDelete(VMApp app, int selectedIndex)
{
	if ((new MessageBox("Delete App?","Delete this app?\n"+app.name,MessageBox.MBYESNO)).execute() != MessageBox.IDYES)
		return;
	removeItem(selectedIndex,true);
	app.deleting();
	apps.apps.removeElementAt(selectedIndex);
	apps.save();
}
public boolean doEdit(VMApp app, int selectedIndex)
{
	if (editApp(app,getFrame())){
		if (selectedIndex != -1)
			replaceItem(makeItemFor(app),selectedIndex,true);
		apps.save();
	}
	return true;
}
public boolean doTop(VMApp app, int selectedIndex)
{
	if (selectedIndex == 0) return true;
	MenuItem it = getItemAtIndex(selectedIndex);
	removeItem(selectedIndex,true);
	addItem(it,0,ADD_MAKE_VISIBLE|ADD_SELECTED|ADD_UPDATE_SCREEN);
	apps.apps.removeElementAt(selectedIndex);
	apps.apps.insertElementAt(app,0);
	apps.save();
	return true;
}
//===================================================================
public boolean doAdd()
//===================================================================
{
	VMApp newOne = new VMApp();
	Editor ed = newOne.getEditor(newOne.ADD_NEW);
	Gui.setOKCancel(ed);
	int got = ed.execute(getFrame(),Gui.CENTER_FRAME);
	if (got == Form.IDCANCEL) return false;
	boolean added = false;
	if (got == 100){
		File cf = Gui.chooseFile(false,"Choose Eve Application","*.eve;*.zip;*.class - Eve Executable Files",Vm.getEnvOrProperty("DOCUMENTS_DIR",File.getProgramDirectory()),FileChooserParameters.OPTION_QUICK_SELECT,null);
		if (cf == null) return false;
		added = add(cf.toString(),getFrame());
		if (added) apps.save();
		return added;
	/*
		FileChooser fc = new FileChooser(FileChooser.OPEN|FileChooser.QUICK_SELECT,);
		fc.title =;
		fc.addMask();
		//fc.addMask("*.class - Class Files");
		if (fc.execute() == IDCANCEL) return false;
		String f = fc.getChosen();
		return add(f,getFrame());
		*/
	}
	add(newOne,true);
	apps.apps.add(newOne);
	apps.save();
	return true;
}

VMApp appAt(int index)
{
	if (index < 0 || index >= apps.apps.size()) return null;
	return (VMApp)apps.apps.get(index);
}
int runRegistry()
{
	Form f = (Form)registryView.newInstance("(Leve/sys/IRegistryKey;)", 
			new Object[]{Registry.getLocalRegistry()});
	if (f != null) return f.execute();
	return Form.IDCANCEL;
}
public void handleContextMenuItem(MenuItem it, int selectedIndex)
{
	String act = it.action;
	VMApp app = appAt(selectedIndex);
	if (add_action.equals(act)) doAdd();
	else if (app != null && edit_action.equals(act)) doEdit(app,selectedIndex);
	else if (app != null && delete_action.equals(act)) doDelete(app,selectedIndex);
	else if (app != null && run_action.equals(act)) app.run();
	else if (app != null && top_action.equals(act)) doTop(app,selectedIndex);
	else if (options_action.equals(act))VMLauncher.execOptions(this);
	else if (about_action.equals(act))VMLauncher.execAbout(this);
	else if (browse_action.equals(act))VMLauncher.execBrowse(this);
	else if (unload_action.equals(act))VMOptions.getVMOptions().unloadVM(getFrame());
	else if (registry_action.equals(act)){
		runRegistry();
		//new RegistryViewForm(Registry.getLocalRegistry()).execute();
	}else if (server_action.equals(act)){
		VMLauncher.execServer(this);
	}else if (app_server_action.equals(act)){
		apps.runAppServer();
	}else if (app_loader_action.equals(act)){
		loadAppAction();
	}else if (remove_installation_action.equals(act)){
		new RemoveInstallation().execute();
	}
		//VMOptions.getVMOptions().unloadVM(getFrame());
}

void loadAppAction()
{
	String got = Registry.getVmCommandLine("eve.ui.data.AppLoader",null);
	if (got != null){
		try{
			Registry.execCommandLine(got);
			return;
		}catch(Exception e){}
	}
	AppLoader rl = new AppLoader("EweSoft\\Eve\\AppLoader");
	try{
		if (!rl.runLoader(true))
			return;
	}catch(Throwable e){
		new ReportException(e).execute();
	}
	Application.exit(0);
}

public void add(VMApp app,boolean updateScreen)
{
	addItem(makeItemFor(app),-1,ADD_SELECTED);
}
/*
public static void main(String args[])
{
	Application.startApplication(args);
	AppLauncher al = new AppLauncher();
	IconMenuForm f = new IconMenuForm(al,0);
	f.cancelButtonName = "Exit";
	f.windowFlagsToSet |= WindowConstants.FLAG_MAXIMIZE_ON_PDA;
	f.title = "App Launcher";
	f.execute();
	System.exit(0);
}
*/
}
//####################################################
