/*
 * Created on May 8, 2006
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui.print;

import java.util.Vector;

import eve.fx.IImage;
import eve.fx.gui.WindowConstants;
import eve.fx.points.PageFormat;
import eve.fx.points.PointDrawable;
import eve.fx.print.Printable;
import eve.fx.print.RandomPrintable;
import eve.ui.Button;
import eve.ui.CellPanel;
import eve.ui.Gui;
import eve.ui.Input;
import eve.ui.Label;
import eve.ui.data.Editor;

/**
 * @author Michael L Brereton
 * This Class previews pages in a Printable class. It will only work on a Printable
 * which supports the printing of pages accessed randomly.
 */
//####################################################
public class PrintPageViewer extends Editor {

	//public static boolean dontMaximizeByDe;
	public int numPages;
	public int gotoWhichPage = 1;
	public String viewing = "Page 1";
	public PageFormat pageFormat = new PageFormat();
	//ScaledImageViewer siv = new ScaledImageViewer();
	PointGraphicsViewer siv = new PointGraphicsViewer();
	
	private int curIndex;
	
	protected RandomPrintable printable;
	
	public void setPrintable(RandomPrintable p, PageFormat pf)
	{
		printable = p;
		pageFormat = pf == null ? new PageFormat() : (PageFormat)pf.getCopy();
		numPages = p == null ? 0 : p.countPages(pageFormat);
		setPage(curIndex = 0);
		if (numPages == Printable.UNKNOWN_NUMBER_OF_PAGES) numPages = 0;
	}
	protected Button addButton(String name, String action)
	{
		return addButton(name,action,(IImage)null);
	}
	protected Button addButton(String name, String action, IImage image)
	{
		Button b = new Button(name);
		if (image != null) Gui.iconize(b,image,true,null);
		if (buttons == null) buttons = new Vector();
		buttons.add(b);
		addField(b,action);
		return b;
	}
	/**
	 * 
	 */
	public PrintPageViewer() {
		setPreferredSize(640,480);
		windowFlagsToSet |= WindowConstants.FLAG_MAXIMIZE;
		Label l;
		addLast(addField(l = new Label(" "),"viewing")).setCell(HSTRETCH);
		l.anchor = CENTER;
		addLast(siv.getScrollablePanel());
		setPage(0);
		addButton("<< Prev","previous");
		addButton("Next >>","next");
		addButton("Whole Page","whole");
		CellPanel cp = new CellPanel();
		Input in = new Input();
		addField(in,"gotoWhichPage");
		cp.addNext(in);
		addField(cp.addNext(new Button("<-Goto Page")),"gotoPage");
		addButton(cp);
		addButton("Close","doClose");
	}

	public void doClose_action(Editor ed)
	{
		exit(IDCANCEL);
	}
	public void whole_action(Editor ed)
	{
		siv.fit();
	}
	public void gotoPage_action(Editor ed)
	{
		if (gotoWhichPage < 1) {
			Gui.flashMessage("No such page!",ed);
			return;
		}
		if (numPages > 0 && gotoWhichPage > numPages-1)
			gotoWhichPage = numPages-1;
		if (setPage(gotoWhichPage-1))
			curIndex = gotoWhichPage-1;
		else
			Gui.flashMessage("No such page!",ed);
	}
	public void next_action(Editor ed)
	{
		if (numPages <= 0 || curIndex < numPages-1){
			if (setPage(curIndex+1))
				curIndex++;
			else
				Gui.flashMessage("No more pages!",ed);
		}
	}
	public void previous_action(Editor ed)
	{
		if (curIndex > 0){
			setPage(--curIndex);
		}
	}
	public boolean setPage(final int index)
	{
		try{
			PageFormat pf = (PageFormat)pageFormat.getCopy();
			if (printable == null) return false;
			PointDrawable pd = printable.getPage(pf,index);
			if (pd == null) return false;
			PointDrawable opd = siv.getDrawable();
			if (opd != pd && opd != null){
				printable.releasePage(opd);
			}
			siv.setDrawable(pd,pf,-1);
			viewing = "Page "+(index+1);
			if (numPages > 0) viewing += " of "+numPages;
			toControls("viewing");
			return true;
		}catch(Throwable t){
			t.printStackTrace();
			return false;
		}
	}
	private void imagesSet()
	{
		curIndex = 0;
		if (numPages == 0) {
			viewing = "No Pages";
			toControls("viewing");
		}
		else setPage(0); 
	}
	/**
	 * This executes a PrintPageViewer in a new window.
	 * @param title the title to display.
	 * @param printable the RandomPrintable to preview.
	 * @param pageFormat the PageFormat to use - this may be null, in which
	 * case a default PageFormat is used.
	 * @return the return value from the execute() method.
	 */
	public static int preview(String title, RandomPrintable printable, PageFormat pageFormat)
	{
		PrintPageViewer pv = new PrintPageViewer();
		pv.title = title;
		pv.setPrintable(printable,pageFormat);
		return pv.execute();
	}
}

//####################################################
