package eve.ui.print;

import eve.fx.print.PrinterControl;
import eve.fx.print.PrinterProperties;
import eve.ui.InputStack;
import eve.ui.Label;
import eve.ui.MessageArea;
import eve.ui.data.Editor;
import eve.util.mString;

public class PrinterDialog extends Editor{
	
	String[] printers;
	String[] displayNames;
	PrinterProperties props;
	
	public int chosenPrinter = -1;
	
	
	public int duplex;
	public int printQuality;
	public int colorMode;
	public int paperSize;
	
	private int whichPrinter(String name)
	{
		if (name == null) return -1;
		for (int i = 0; i<printers.length; i++){
			if (printers[i].equals(name)) return i;
		}
		return -1;
	}
	private static int indexOf(int value, int[] values)
	{
		for (int i = 0; i<values.length; i++){
			if (values[i] == value) return i;
		}
		return 0;
	}
	public void fromPrinterControl(PrinterControl pc)
	{
		duplex = indexOf(pc.duplexMode,PrinterControl.duplexValues);
		printQuality = indexOf(pc.printQuality,PrinterControl.qualityValues);
		paperSize = indexOf(pc.paperSize,PrinterControl.paperValues);
		colorMode = indexOf(pc.colorMode,PrinterControl.colorValues);
	}
	public void toPrinterControl(PrinterControl pc)
	{
		pc.setDuplexMode(PrinterControl.duplexValues[duplex]);
		pc.setPrintQuality(PrinterControl.qualityValues[printQuality]);
		pc.setPaperSize(PrinterControl.paperValues[paperSize]);
		pc.setColorMode(PrinterControl.colorValues[colorMode]);
	}
	public PrinterDialog(PrinterProperties pl)
	{
		title = pl.getString(PrinterProperties.PRINT_DIALOG_TITLE, "Select Printer");
		props = pl;
		String printerList = pl.getString(PrinterProperties.AVAILABLE_PRINTERS, "");
		if (printerList == null) printerList = "";
		printers = mString.split(printerList,',');
		PrinterControl pc = PrinterControl.getOrCreatePrinterControl(props);
		fromPrinterControl(pc);
		if (printers.length == 0){
			Label tm = new MessageArea("There are no available printers!\nPrinting cannot continue.");
			addLast(tm);
			doButtons(CANCELB);
		}else{
			if (!printers[0].equalsIgnoreCase(PrinterProperties.VALUE_AVAILABLE_PRINTERS_DONT_SHOW_LIST)){
				displayNames = new String[printers.length];
				for (int i = 0; i<printers.length; i++){
					String p = printers[i];
					int s = p.indexOf('='); 
					if (s == -1) displayNames[i] = p;
					else{
						displayNames[i] = p.substring(s+1);
						p = p.substring(0,s);
						printers[i] = p;
						displayNames[i] += " ("+p+")";
					}
				}
				chosenPrinter = whichPrinter(pl.getString(PrinterProperties.CHOSEN_PRINTER, null));
				if (chosenPrinter == -1) chosenPrinter = whichPrinter(pl.getString(PrinterProperties.DEFAULT_PRINTER, null));
				if (chosenPrinter == -1) chosenPrinter = 0;
			}
			InputStack is = new InputStack();
			if (chosenPrinter >= 0){
				addField(is.addChoice("Printer:", displayNames, 0),"chosenPrinter");
			}
			addField(is.addChoice("Paper Size:", PrinterControl.paperNames, 0),"paperSize");
			addField(is.addChoice("Print Quality:", PrinterControl.qualityNames, 0),"printQuality");
			addField(is.addChoice("Color Mode:", PrinterControl.colorNames, 0),"colorMode");
			addField(is.addChoice("Two-Sided:", PrinterControl.duplexNames, 0),"duplex");
			addLast(is,HSTRETCH);
			doButtons(OKB|CANCELB);
		}
	}
	public boolean canExit(int retCode)
	{
		if (retCode == IDCANCEL) return true;
		if (chosenPrinter != -1) props.set(PrinterProperties.CHOSEN_PRINTER,printers[chosenPrinter]);
		PrinterControl pc = PrinterControl.getOrCreatePrinterControl(props);
		toPrinterControl(pc);
		return true;
	}
}
