/*
 * Created on May 8, 2006
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui.print;

import eve.fx.Graphics;
import eve.fx.Point;
import eve.fx.Rect;
import eve.sys.ImageData;
import eve.sys.ImageDataInfo;
import eve.ui.Canvas;
import eve.ui.DragContext;
import eve.ui.event.PenEvent;

/**
 * @author Michael L Brereton
 *
 */
//####################################################
public class ScaledImageViewer extends Canvas {
	
public ImageData source;
public int newWidth, newHeight;
public double zoom;
ImageDataInfo info;
{
	modify(WantDrag,0);
	if (origin == null) origin = new Point();
}

public void doPaint(Graphics g, Rect area)
{
	if (info == null) info = ImageDataInfo.toImageDataInfo(source);
	int newWidth = (int)(info.width*zoom), newHeight = (int)(info.height*zoom);
	g.setColor(getBackground());
	g.fillRect(area.x,area.y,area.width,area.height);
	/*
	int sz = (int)zoom; if (sz < 0) sz = 1;
	g.changePen(Color.Black,Pen.SOLID,sz);
	g.drawLine(-origin.x,-origin.y,newWidth-origin.x,newHeight-origin.y);
	g.drawLine(newWidth-origin.x,-origin.y,-origin.x,newHeight-origin.y);
	*/
	g.drawImageData(source,-origin.x,-origin.y,newWidth,newHeight,area,0);
}

private void zoomChanged()
{
	int i = modify(Invisible,0);
	if (source != null)
		virtualSize = new Rect(0,0,(int)(info.width*zoom),(int)(info.height*zoom));
	checkScrolls();
	restore(i,Invisible);
}
public void setImage(ImageData im)
{
	setImage(im,0,true);
}
public void setImage(ImageData im, double zoom, boolean repaint)
{
	info = ImageDataInfo.toImageDataInfo(im);
	if (zoom <= 0 && width > 0){
		double xs = (double)width/info.width;
		double ys = (double)height/info.height;
		zoom = xs < ys ? xs : ys;
	}
	if (zoom <= 0) zoom = 1;
	this.zoom = zoom;
	source = im;
	int i = modify(Invisible,0);
	setOrigin(0,0,null);
	zoomChanged();
	restore(i,Invisible);
	if (repaint) repaintNow();
}

public void setZoom(double newZoom)
{
	if (newZoom <= 0) newZoom = 1;
	zoom = newZoom;
	zoomChanged();
	repaintNow();
}
public void fit()
{
	double xs = (double)width/info.width;
	double ys = (double)height/info.height;
	zoom = xs < ys ? xs : ys;
	zoomChanged();
	repaintNow();
}
public void onPenEvent(PenEvent ev)
{
	if (ev.type == PenEvent.SCROLL_UP){
		zoom *= 2;
		int i = modify(Invisible,0);
		zoomChanged();
		restore(i,Invisible);
		repaintNow();
	}else if (ev.type == PenEvent.SCROLL_DOWN){
		zoom /= 2;
		int i = modify(Invisible,0);
		zoomChanged();
		restore(i,Invisible);
		repaintNow();
	}else
		super.onPenEvent(ev);
}

public void startDragging(DragContext dc)
{
	oldOrigin.set(origin);
}
Point moved = new Point();
Point oldOrigin = new Point();
Rect screenRect = new Rect();
public void dragged(DragContext dc)
{
	int dx = dc.curPoint.x-dc.prevPoint.x;
	int dy = dc.curPoint.y-dc.prevPoint.y;
	
	scroll(-dx,-dy,moved);
}
public void stopDragging(DragContext dc)
{
	
}
/*
protected void pointerEvent(SurfacePointerEvent ev)
{
	if (ev.type == ev.SCROLL_DOWN){
		zoom /= 2;
		repaintNow();
	}else if (ev.type == ev.SCROLL_UP){
		zoom *= 2;
		repaintNow();
	}else if (ev.type == ev.PEN_DOWN){
		dragging = null;
		penPressed(ev);
		if (dragging != null)
			startDragging(dragging);
	}else if (ev.type == ev.PEN_UP){
		if (dragging != null) {
			dragging.nowAt(ev.x,ev.y);	
			stopDragging(dragging);
			dragging = null;
		}
		else penReleased(ev);
	}else if (ev.type == ev.PEN_MOVE){
		if (dragging != null){
			if (!dragging.nowAt(ev.x,ev.y)) return;
			dragged(dragging);
		}else penMoved(ev);
	}
}
*/
}
//####################################################
