package eve.ui.print;

import eve.fx.Color;
import eve.fx.points.DataCell;
import eve.fx.points.FontManager;
import eve.fx.points.PageFormat;
import eve.fx.points.PointFont;
import eve.fx.points.PointGraphics;
import eve.fx.points.PrintCellPanel;
import eve.fx.points.FontManager.ActiveFonts;
import eve.fx.print.PrintPages;
import eve.fx.print.Printable;
import eve.fx.print.PrintablePage;
import eve.fx.print.PrinterControl;
import eve.fx.print.PrinterJob;
import eve.fx.print.PrinterProperties;
import eve.sys.Vm;
import eve.sys.Wrapper;
import eve.ui.Button;
import eve.ui.InputStack;
import eve.ui.data.Editor;

public class TestPrintForm extends PrintForm{

	public int numberOfPages = 2;
	PrinterControl overridePrintControl = new PrinterControl(false);
	
	public TestPrintForm()
	{
		super("Test Print Form",OPTION_EXTRA_PDF_PREVIEW);
		dontExitAfterPrint = dontExitOnPrint = true;
		InputStack is = new InputStack();
		addField(is.addCheckBox("Use Non Native Dialog"),
				"useNonNativeDialog");
		addField(is.addInput("Number of Pages:",""),
				"numberOfPages");
		addLast(is,HSTRETCH);
		addField(addLast(new Button("Adjust Printer Control"),HSTRETCH,DONTFILL),"adjustControl");
	}
	public void adjustControl_action(Editor ed)
	{
		PrinterProperties pp = 
			overridePrintControl.addToProperties(null);
		pp.set(PrinterProperties.AVAILABLE_PRINTERS,PrinterProperties.VALUE_AVAILABLE_PRINTERS_DONT_SHOW_LIST);
		pp.set(PrinterProperties.PRINT_DIALOG_TITLE,"Select Print Options");
		PrinterJob.printerDialog(pp).waitUntilCompletion();
	}
	class testPrintPage extends PrintablePage{

		Wrapper w;
		testPrintPage(Wrapper w){this.w = w;}
		
		protected void composePage(PointGraphics pg, PageFormat format,
				int pageNumber) {
			
			String msg = Vm.isJavaVM() ? "Java " : "Native ";
			msg += "Eve Test Print - Page: "+(pageNumber);
			PrintCellPanel pcp = addMarginedPanel(72,72,72,72);
			ActiveFonts af = fontManager.assignTo(pg);
			pcp.font = af.get("serif", PointFont.PLAIN, 40); 
			PrintCellPanel p = new PrintCellPanel();
			DataCell dc = new DataCell(msg);
			dc.foreGround = Color.DarkBlue;
			dc.backGround = Color.LightGreen;
			dc.borderColor = Color.LightBlue;
			dc.setSpacing(72/16);
			dc.borderRect = dc.RECT_SIDE_FULL;
			dc.anchor = CENTER;
			p.addLast(dc,DONTSTRETCH);
			PrinterControl pc = (PrinterControl)w.getObject(PrinterControl.class);
			if (pc != null){
				String specs = 
					 "Quality: "+pc.getPrinterQuality(pc.printQuality)+"\n"
					+"Color: "+pc.getColorMode(pc.colorMode)+"\n"
					+"Sides: "+pc.getDuplexMode(pc.duplexMode)+"\n"
					+"Paper: "+pc.getPaperSize(pc.paperSize);
				dc = new DataCell(specs);
				dc.font = af.get("sans",PointFont.ITALIC,20);
				p.addLast(dc,DONTSTRETCH);
			}
			pcp.addLast(p);
		}
		
	}
	
	static FontManager fontManager;
	
	protected Printable getPrintable()
	{
		final PrinterControl pc = new PrinterControl();
		pc.copyFrom(overridePrintControl);
		//
		if (fontManager == null) fontManager = new FontManager();
		fontManager.addTimes("serif");
		fontManager.addSansSerif("sans");
		//
		final Wrapper w = new Wrapper();
		PrintPages pp = new PrintPages(){
			public void adjustPrinterProperties(PrinterProperties p)
			{
				pc.overrideInProperties(p);
				w.setObject(pc);
			}
			
		};
		for (int i = 0; i<numberOfPages; i++)
			pp.addFullPage(new testPrintPage(w));
		return pp;
		
	}
	
}
