package eve.ui.table.data.registry;
import eve.fx.Color;
import eve.fx.Graphics;
import eve.fx.Rect;
import eve.fx.gui.IKeys;
import eve.sys.FieldTransfer;
import eve.sys.IRegistryKey;
import eve.ui.Button;
import eve.ui.CellPanel;
import eve.ui.Gui;
import eve.ui.Menu;
import eve.ui.MenuItem;
import eve.ui.MessageBox;
import eve.ui.ScrollBarPanel;
import eve.ui.SoftKeyBar;
import eve.ui.SplittablePanel;
import eve.ui.TextPad;
import eve.ui.data.Editor;
import eve.ui.event.KeyEvent;
import eve.ui.table.TableControl;
import eve.ui.table.TableEvent;
import eve.ui.table.TreeControl;
import eve.ui.table.TreeEvent;

//##################################################################
public class RegistryViewForm extends Editor{
//##################################################################

TreeControl tree;
RegistryTreeModel treeModel;
TableControl table;
RegistryTableModel tableModel;

boolean amMaxed = false;
boolean onTree = true;
SplittablePanel sp;
int oldSize = -1;
Object treeCursor, tableCursor;

static final String switchPane = "Switch";
static final String exitButton = "Exit";
static final String maxButton = "Maximize Pane";
static final String splitButton = "Split View";
static final String autoUpdateButton = "Auto Update";
static final String searchButton = "Search...";
boolean autoUpdate = true;//!Device.isMobile() || !Gui.hasSoftKeys;

void selectionToTable()
{
	if (treeModel.selectedLine < 0) tableModel.setKey(null);
	else tableModel.setKey(treeModel.getKeyAt(treeModel.selectedLine));
}
public RegistryViewForm(IRegistryKey root)
{
	if (!Gui.isPDA) setPreferredSize(640,480);
	title = "Registry";
	tree = new TreeControl(){
		//===================================================================
		public void doPaint(Graphics g,Rect area)
		//===================================================================
		{
			//if (remote) Gui.showWait(true);
			super.doPaint(g,area);
			//if (remote) Gui.showWait(false);
		}
		public void onKeyEvent(KeyEvent ev){
			super.onKeyEvent(ev);
			if (ev.type != ev.KEY_PRESS) return;
			if (ev.key == IKeys.TAB)
				switchPane();
			else if (!ev.isActionKey() && !autoUpdate){
				tableModel.needAction();
			}
		}
	};
	if (!Gui.hasSoftKeys){
		CellPanel c = new CellPanel();
		c.backGround = Color.White;
		Button b = new Button("Search","eve/ChooseFile.png");
		b.modify(NoFocus,TakesKeyFocus);
		addField(b,"search");
		c.addLast(b).setControl(DONTFILL|WEST);
		b.setBorder(b.BDR_NOBORDER,0);
		c.setBorder(((b.EDGE_ETCHED & ~BF_RECT)|BF_BOTTOM),2);
		addLast(c).setCell(HSTRETCH);
	}
	//tree.multiSelect = true;
	//tree.modify(WantDrag,0);
	tree.backGround = Color.White;
	treeModel = new RegistryTreeModel(root);//tc.getTreeTableModel();
	treeModel.showWaitCursor = true;
	tree.setTableModel(treeModel);
	tree.restoreOrResetCursor(null);
	boolean wide = Gui.screenIs(Gui.WIDE_SCREEN);
	sp = new SplittablePanel(wide ? SplittablePanel.Vertical :SplittablePanel.Horizontal);
	CellPanel cp = sp.getNextPanel();
	ScrollBarPanel sbp = new ScrollBarPanel(tree,Gui.isSmartPhone ? ScrollBarPanel.OPTION_INDICATOR_ONLY : 0);
	cp.addLast(sbp);
	cp = sp.getNextPanel();
	table = new TableControl(){
		public void onKeyEvent(KeyEvent ev){
			super.onKeyEvent(ev);
			if (ev.type == ev.KEY_PRESS && ev.key == IKeys.TAB)
				switchPane();
		}
	};
	table.setClickMode(true);
	//table.clickClearsItself = false;
	table.setTableModel(tableModel = new RegistryTableModel());
	tableModel.setKey(null);
	//tableModel.setDataAndHeaders(new Grid(),null,null);
	//tableModel.calculateSizes(getFontMetrics());
	cp.addLast(new ScrollBarPanel(table,Gui.isSmartPhone ? ScrollBarPanel.OPTION_INDICATOR_ONLY : 0));
	addLast(sp);
	addField(tree,"regTree");
	addField(table,"regTable");
	firstFocus = tree;
	if (Gui.hasSoftKeys){
		if (!autoUpdate) tableModel.needAction();
		sp.theSplitter.thickness = 2;
		SoftKeyBar sb = new SoftKeyBar();
		sb.setKey(2,switchPane,loadImage(wide ? "eve/LeftRight.png" : "eve/UpDown.png"),null);
		//sb.setKey(1,searchButton,null,null);
		sb.setKey(1,"Menu",null,new Menu(){
			protected boolean prepareForDisplay(){
				removeAll();
				addItem(exitButton);
				if (amMaxed) addItem(splitButton);
				else addItem(maxButton);
				addItem(searchButton);
				MenuItem mi = new MenuItem(autoUpdateButton);
				if (autoUpdate) mi.modifiers |= mi.MODIFIER_CHECKED;
				addItem(mi);
				SoftKeyBar.fixMenu(this);
				return true;
			}
		});
		setSoftKeyBarFor(null,sb);
	}
}
void max()
{
	amMaxed = true;
	if (oldSize == -1) oldSize = sp.theSplitter.getControlSize(true);
	if (onTree)
		sp.theSplitter.doOpenClose(true);
	else
		sp.theSplitter.doOpenClose(false);
}
void switchPane()
{
	if (onTree){
		onTree = false;
		selectionToTable();
		treeCursor = tree.hideCursor();
		if (amMaxed) max();
		table.restoreOrResetCursor(tableCursor);
		Gui.takeFocus(table,ByRequest);
	}else{
		onTree = true;
		tableCursor = table.hideCursor();
		if (amMaxed) max();
		tree.restoreOrResetCursor(treeCursor);
		Gui.takeFocus(tree,ByRequest);
		int sel = treeModel.selectedLine;
		if (sel != -1) tree.scrollToVisible(sel);
	}
}
SearchData search;
Editor searchEditor;
public void search_action(Editor ed)
{
	if (search == null) {
		search = new SearchData(this);
		searchEditor = search.getEditor(0);
	}
	search.found.setLength(0);
	int line = treeModel.selectedLine;
	int[] addr = line == -1 ? new int[0] : treeModel.addressOf(line);
	search.found.append(addr,0,addr.length);
	if (searchEditor.execute() == IDOK){
		jumpTo(search.found.toIntArray());
	}
}
public void onSoftKey(int key, String action, MenuItem selected)
{
	if (action.equals(switchPane)){
		switchPane();
	}else if (action.equals(exitButton)){
		exit(0);
	}else if (action.equals(maxButton)){
		max();
	}else if (action.equals(splitButton)){
		amMaxed = false;
		sp.theSplitter.setControlSize(true,oldSize);
	}else if (action.equals(autoUpdateButton)){
		autoUpdate = !autoUpdate;
	}else if (action.equals(searchButton)){
		search_action(this);
		/*
		int line = treeModel.selectedLine;
		if (line == -1) return;
		int[] addr = treeModel.addressOf(line);
		SearchData sd = new SearchData();
		System.out.println("Searching!");
		if (sd.search(null,treeModel,addr)){
			System.out.println("Found!");
			treeModel.expandTo(sd.found.toIntArray(),true);	
		}else
			System.out.println("Not found!");
		*/
	}
}
public void jumpTo(int[] address)
{
	treeModel.expandTo(address,true);
	if (!onTree) switchPane();
	selectionToTable();
}
//===================================================================
public void fieldEvent(FieldTransfer ft,Editor f,Object ev)
//===================================================================
{
	if (ft.fieldName.equals("regTree")){
		if (ev instanceof TreeEvent){
			TreeEvent te = (TreeEvent)ev;
			selectionToTable();
			tableModel.setKey(treeModel.getKeyAt(te.selectedLine));
		}else if (ev instanceof TableEvent){
			if (((TableEvent)ev).type == TableEvent.SELECTION_CHANGED && autoUpdate){
				selectionToTable();
			}
		}
	}else if (ft.fieldName.equals("regTable")){
		if (ev instanceof TableEvent){
			TableEvent te = (TableEvent)ev;
			if (te.type == te.CELL_CLICKED && ((te.flags & te.FLAG_SELECTED_BY_ARROWKEY) == 0)){
				String [] both = tableModel.getDataAt(te.row,te.col);
				if (both != null){
					MessageBox mb = new MessageBox("Key Value");
					mb.type = mb.MBOK;
					TextPad ma = new TextPad(5,50);
					ma.wrapToScreenSize = true;
					ma.setText("Name="+both[0]+"\nValue="+both[1]);
					mb.displayInstead(ma.setAsStaticDisplay(true)).setCell(STRETCH);
					mb.doBeep = false;
					mb.execute();
				}
			}
		}
	}
}
//##################################################################
}
//##################################################################

