/*
 * Created on Feb 23, 2007
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui.table.data.registry;

import eve.sys.Countdown;
import eve.sys.Handle;
import eve.sys.IRegistryKey;
import eve.sys.Vm;
import eve.ui.Button;
import eve.ui.ButtonBar;
import eve.ui.CellPanel;
import eve.ui.Gui;
import eve.ui.Label;
import eve.ui.ProgressBar;
import eve.ui.SoftKeyBar;
import eve.ui.Window;
import eve.ui.data.Editor;
import eve.ui.data.EditorStack;
import eve.ui.data.LiveObject;
import eve.util.IntArray;
import eve.util.OperationTimer;
import eve.util.SubString;

/**
 * @author Michael L Brereton
 *
 */
//####################################################
public class SearchData extends LiveObject{

	RegistryViewForm rvf;
	public SearchData(RegistryViewForm rvf)
	{
		this.rvf = rvf;
	}
	int searchRoot = 0;
	/** A possible searchFrom value. **/
	public static final int FROM_CURRENT = 0;
	/** A possible searchFrom value. **/
	public static final int FROM_TOP = 1;
	public int searchFrom = 0;
	/** A possible searchWhere value. **/
	//public static final int WHERE_CURRENT_IMMEDIATE_CHILDREN = 0;
	/** A possible searchWhere value. **/
	//public static final int WHERE_CURRENT_ALL_DESCENDANTS = 1;
	/** A possible searchWhere value. **/
	//public static final int WHERE_ENTIRE_REGISTRY = 2;
	/** A possible searchWhere value. **/
	//public static final int WHERE_CURRENT_SIBLINGS_ONLY = 3;
	//public int searchWhere = 1;
	/** A possible searchWhat value. **/
	public static final int WHAT_KEY_NAMES = 0;
	/** A possible searchWhat value. **/
	//public static final int WHAT_KEY_AND_VALUE_NAMES = 1;
	/** A possible searchWhat value. **/
	//public static final int WHAT_ALL_DATA = 2;
	public int searchWhat = 0;
	
	public int maxDepth = 0;
	
	public String searchFor = "EveFile";
	public static final int THAT_STARTS_WITH = 0;
	public static final int THAT_EQUALS = 1;
	public static final int THAT_CONTAINS = 2;
	public int searchThat = 0;
	
	public boolean ignoreCLSID = true;
	IntArray found = new IntArray();
	
	Handle searching;
	/*
	protected Editor makeNewEditor(int which)
	{
		return new Editor(){
			public void resizeTo(int w, int h)
			{
				new Exception("Size: "+w+", "+h).printStackTrace();
				super.resizeTo(w,h);
			}
		};
	}
	*/
	public void addToPanel(CellPanel cp, final Editor ed, int which)
	{
		try{
		EditorStack es = new EditorStack();
		ed.title = "Search Registry";
		boolean doubleTop = Gui.isSmartPhone && false;
		if (doubleTop){
			es.doubleLined = true;
			ed.addField(es.addInput("Search for:",""),"searchFor");
			cp.addLast(es).setCell(cp.HSTRETCH);
			es = new EditorStack();
			es.doubleLined = false;
		}
		if (!doubleTop)
			ed.addField(es.addInput("For:",""),"searchFor");
		ed.addField(es.addChoice("From:",new String[]{"Current Selection","Start of Registry"},0),"searchFrom");
		ed.addField(es.addChoice("In:",new String[]{"Key Names"},0),"searchWhat");
		ed.addField(es.addChoice("Which:",new String[]{"Starts with text","Equals Text"},0),"searchThat");
		//ed.addField(es.addInput("Deepest:",""),"maxDepth");
		cp.addLast(es).setCell(cp.HSTRETCH);
		es = new EditorStack();
		es.doubleLined = false;
		ed.addField(es.addCheckBox("Dont Search CLSID"),"ignoreCLSID");
		cp.addLast(es).setCell(cp.HSTRETCH);
		
		ProgressBar pb = new ProgressBar();
		pb.showPercent = false;
		pb.doing = "";
		pb.text = "Searching...";
		cp.addLast(new Label("Progress:")).setCell(cp.HSTRETCH);
		cp.addLast(pb).setCell(cp.HSTRETCH);
		ed.getProperties().set("progress",pb);
		//ed.addField(es.addChoice("Start from:",new String[]{"Current Selection","Start of Registry"},0),"searchFrom");
		/*
		EventListener el = new EventListener(){

			public void onEvent(Event ev) {
				System.out.println(ev.getClass());
				if (ev instanceof SoftKeyEvent){
					SoftKeyEvent se = (SoftKeyEvent)ev;
					if (se.action.equals("Cancel")) cancelAction(ed);
					if (se.action.equals("Search")) searchAction(ed);
				}
				
			}
			
		};
		*/
		Button cb = new Button("Cancel"); cb.action = "cancel";
		Button sb = new Button("Search"); sb.action = "search";
		//ed.addField(cb,"cancel",null,Gui.hasSoftKeys);
		//ed.addField(sb,"search",null,Gui.hasSoftKeys);
		if (Gui.hasSoftKeys){
			SoftKeyBar skb = new SoftKeyBar();//el);
			/*
			Menu m2 = new Menu();
			m2.text = "More...";
			m2.addItem(skb.createMenuItem("Search","search",null));
			Menu m = new Menu();
			m.addItem(m2);
			skb.setKey(2,"Search Options",m);
			*/
			skb.setKey(2,sb);
			skb.setKey(1,cb);
			ed.setSoftKeyBarFor(null,skb);
			ed.addSoftKeysAsActions(skb);
		}else{
			ButtonBar bb = new ButtonBar();
			ed.addField(bb.addNext(sb),sb.action);
			ed.addField(bb.addNext(cb),cb.action);
			cp.addLast(bb).setCell(cp.HSTRETCH);
		}
		}catch(Throwable t){
			t.printStackTrace();
		}
	}
	
	public synchronized void cancel_action(Editor ed)
	{
		if (searching != null) {
			searching.stop(0);
		}else
			ed.exit(ed.IDCANCEL);
	}
	public synchronized void search_action(final Editor ed)
	{
		final int []addr = (searchFrom == FROM_TOP) ? new int[0] : found.toIntArray();
		final ProgressBar pb = (ProgressBar)ed.getProperties().getValue("progress",null);
		if (searching == null) {
			searching = new Handle();
			new Thread(){
				public void run(){
					try{
						pb.set(-1,"Searching...");
						boolean found = search(searching,addr,pb);
						if (!found)
							pb.set(0,"Not found.");
						else{
							pb.set(0,"");//Found!");
							ed.exit(ed.IDOK);
						}
					}finally{
						synchronized(SearchData.this){
							searching = null;
						}
					}
				}
			}.start();
		}		
	}
	class searchEntry {
		searchEntry parent;
		IRegistryKey key;
		String[] values;
		int childIndex;
		
		public searchEntry(IRegistryKey key)
		{
			this.key = key;
			values = (String[])key.getSubKeys(0);
			if (values == null) values = new String[0];
		}
		
		public searchEntry getSubKey(int index)
		{
			searchEntry s = new searchEntry(key.getSubKey(values[index]));
			s.parent = this;
			childIndex = index;
			return s;
		}
	}
	searchEntry createForAddress(int[] addr)
	{
		searchEntry s = new searchEntry(rvf.treeModel.root);
		for (int i = 0; i<addr.length; i++)
			s = s.getSubKey(addr[i]);
		return s;
	}
	
	boolean search(Handle h, int[] startAddress, ProgressBar pb)
	{
		OperationTimer ot = new OperationTimer();
		if (h == null) h = new Handle();
		String sf = searchFor;
		int so = SubString.IGNORE_CASE;
		if (searchThat == THAT_STARTS_WITH) so |= SubString.STARTS_WITH;
		char[] ss = Vm.getStringChars(sf);
		int searchLen = ss.length;
		char[] clsid = Vm.getStringChars("CLSID");
		int cidLen = clsid.length;
		//
		//IntArray locs = new IntArray();
		found.setLength(0);
		found.append(startAddress,0,startAddress.length);
		int depth = found.length+1;
		Countdown cd = new Countdown();
		int tick = 250;
		cd.start(tick);
		//int nextIdx = 0;
		//long[] idxes = null;
		//Object key = null;
		//CharArray ca = new CharArray();
		//
		searchEntry se = createForAddress(startAddress);
		while(depth > 0 && !h.shouldStop){
			if (se.childIndex >= se.values.length || (maxDepth > 0 && depth > maxDepth)){
				// Can't go any further on this depth.
				// Go back up.
				if (depth <= 1) break;
				found.length--;
				depth--;
				se = se.parent;
				if (se == null) break;
				se.childIndex++;
				if (cd.hasExpired()){
					cd.start(tick);
					pb.set(-1,"Searching...");
					Window.handleWindowEvents();
				}
				continue;
			}
			//
			char[] ch = Vm.getStringChars(se.values[se.childIndex]);
			boolean eq = //= searchThat == THAT_CONTAINS ?
				SubString.equals(ch,0,ch.length,ss,0,searchLen,so);
			if (eq){
				found.append(se.childIndex);
				return true;
			}
			//
			// Not that subkey, so go into it.
			//
			if ((maxDepth > 0 && depth >= maxDepth) || (ignoreCLSID && SubString.equals(ch,0,ch.length,clsid,0,cidLen,SubString.IGNORE_CASE))){
				//
				// Too deep or is CLSID, don't go into it.
				//
				se.childIndex++;
				continue;
			}
			found.append(se.childIndex);
			se = se.getSubKey(se.childIndex);
			depth++;
		}
		/*
		Hashtable si = new Hashtable();
		boolean recurse = true;
		while(depth > 0 && !h.shouldStop){
			if (key == null){
				//h.setProgress(-1);
				if (false && pb != null){
					pb.doing = "Searching...";
					pb.set(-1);
				}
				
				//Thread.yield();
				ot.start("GOA");
				key = (Object)rtm.getObjectAtAddress(found.toIntArray());
				ot.start("GID");
				idxes = (long[])si.get(key);
				if (idxes == null){
					idxes = rtm.getKeyIndexes(key);
					if (idxes == null)idxes = new long[0];
					si.put(key,idxes);
				}
			}
			ca.setLength(0);
			ot.start("GSK");
			if ((maxDepth > 0 && depth > maxDepth) || nextIdx >= idxes.length || !rtm.getSubKeyName(key,idxes[nextIdx],ca)){
					//key.getSubKey((int)idxes[nextIdx],ca)){
				//Go up one level.
				if (!recurse) break;
				if (depth <= 1) break;
				key = null; // Get key again when going up.
				depth--;
				nextIdx = found.data[depth-1]+1;
				found.length--;
				ot.end();
				System.out.println(ot);
				continue;
			}
			//System.out.println(ca);// mThread.nap(100);
			ot.start("EQ");
			boolean eq = //= searchThat == THAT_CONTAINS ?
					SubString.equals(ca.data,0,ca.length,ss,0,searchLen,so);
			if (eq){
				found.append(nextIdx);
				return true;
			}
			ot.end();
			System.out.println(ot);
			
			if (!recurse || (maxDepth > 0 && depth >= maxDepth) || (ignoreCLSID && SubString.equals(ca.data,0,ca.length,clsid,0,cidLen,SubString.IGNORE_CASE))){
				nextIdx++;
				continue;
			}
			//
			// Go down one level.
			//
			//System.out.println("->"+nextIdx+"")
			found.append(nextIdx);
			depth++;
			nextIdx = 0;
			key = null;
		}
		*/
		found.setLength(0);
		found.append(startAddress,0,startAddress.length);
		return false;
	}
	/*
	boolean search(Handle h, RegistryKey start, RegistryTreeModel tm, IntArray found, int so, char[] ss, boolean recurse)
	{
		//System.out.println(start);
		char[] clsid = Vm.getStringChars("CLSID");
		int cidLen = clsid.length;
		long[] idx = (long[])start.getSubKeys(RegistryKey.GET_INDEXES_AS_LONGS);
		int searchLen = ss.length;
		CharArray ca = new CharArray();
		if (idx != null){
			for (int i = 0; i<idx.length; i++){
				found.append(i);
				ca.setLength(0);
				if (start.getSubKey((int)idx[i],ca)){
					if (SubString.equals(ca.data,0,ca.length,ss,0,searchLen,so)){
						return true;
					}
					if (recurse && !SubString.equals(ca.data,0,ca.length,clsid,0,cidLen,SubString.IGNORE_CASE)){
						RegistryKey rk = (RegistryKey)tm.getSubKey(start,idx[i]);
						if (search(h,rk,tm,found,so,ss,true))
							return true;
					}
				}
				found.setLength(found.length-1);
			}
		}
		return false;
	}
	*/
	//public 
}

//####################################################
