package eve.util;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.NoSuchElementException;
/**
* This is an object that can be used to implement an object that is both an
* Iterator and an Enumeration. To do this simply override and provide an
* implementation for Iterator.hasNext() and Iterator.next() and (optionally)
* Iterator.remove(). By default remove() will throw an UnsupportedOperationException.
**/
//##################################################################
public class IteratorEnumerator implements Enumeration, Iterator{
//##################################################################

//-------------------------------------------------------------------
protected IteratorEnumerator(){}
//-------------------------------------------------------------------

private Enumeration enumer;
private Iterator it;
/**
Using this constructor will convert the Iterator to an Enumeration.
**/
//===================================================================
IteratorEnumerator(Iterator it)
//===================================================================
{
	this.it = it;	
}
/**
Using this constructor will convert the Enumeration to an Iterator;
**/
//===================================================================
IteratorEnumerator(Enumeration enumer)
//===================================================================
{
	this.enumer = enumer;	
}
/**
Override this to implement your own hasNext().
**/
//===================================================================
public boolean hasNext()
//===================================================================
{
	if (it != null) return it.hasNext();
	else if (enumer != null) return enumer.hasMoreElements();
	else return false;//throw new IllegalStateException("Cannot do hasNext()");
}
/**
Override this to implement your own next().
**/
//===================================================================
public Object next()
//===================================================================
{
	if (it != null) return it.next();
	else if (enumer != null) return enumer.nextElement();
	else throw new NoSuchElementException();
}
/**
* This returns the hasNext() method.
**/
//===================================================================
public final boolean hasMoreElements() {return hasNext();}
//===================================================================
/**
* This returns the next() method.
**/
//===================================================================
public final Object nextElement() throws NoSuchElementException {return next();}
//===================================================================

//-------------------------------------------------------------------
private void unsupported()
//-------------------------------------------------------------------
{
	throw new UnsupportedOperationException("remove()");
}
/**
 * The default version of this will throw an UnsupportedOperationException.
 */
//===================================================================
public void remove() 
//===================================================================
{
	if (it != null) it.remove();
	else unsupported();
}
/*
//=================================================================
public static void main(String[] args)
//=================================================================
{
	java.util.Vector v = new java.util.Vector();
	for (int i = 0; i<3; i++)
		v.addElement(""+i);
	//Method m = Reflection.getMethod(v.getClass(),"iterator()Ljava/util/Iterator;",false);
	//Object
	Wrapper w = Reflection.invokeMethod(v,"iterator()Ljava/util/Iterator;",Reflection.emptyParameters);
	java.util.Iterator jit = (java.util.Iterator)w.getObject();
	for (Iterator it = Utils.toIterator(w.getObject()); it.hasNext();)
		System.out.println(it.next());
}
*/
//##################################################################
}
//##################################################################

