package eve.util;

import eve.sys.Device;

/**
 * This Collator compares Strings based on the Unicode value. This
 * may not produce a sorting that is culturally or Locale correct
 * but it will be consistent across all Java/Win32/Unix platforms.
 */
public class SimpleCollator implements StringCollator{

	private static native int compareChar(char one, char two, int options);
	private static native int compareChars(char[] one, int oneOffset, int oneLength, char[] two,
			int twoOffset, int twoLength, int options);
	private static boolean hasNative = true;
	
	//
	public int compare(char[] one, int oneOffset, int oneLength, char[] two,
			int twoOffset, int twoLength, int options) {
		if (hasNative) try{
			return compareChars(one,oneOffset,oneLength,two,twoOffset,twoLength,options);
		}catch(Throwable t){
			Device.checkNoNativeMethod(t);
			hasNative = false;
		}
		boolean ignoreCase = (options & OPTION_IGNORE_CASE) != 0;
		int oi = oneOffset, ti = twoOffset;
		for (int i = 0; i<oneLength; i++, oi++, ti++){
			if (i >= twoLength) return 1;
			char o = one[oi];
			char t = two[ti];
			if (ignoreCase){
				o = Character.toUpperCase(o);
				t = Character.toUpperCase(t);
			}
			if (o != t) return o-t;
		}
		if (oneLength == twoLength || ((options & OPTION_STARTS_WITH) != 0))
			return 0;
		return -1;
	}
	//
	public int compare(char one, char two, int options) {
		if (hasNative) try{
			return compareChar(one,two,options);
		}catch(Throwable t){
			Device.checkNoNativeMethod(t);
			hasNative = false;
		}
		if ((options & OPTION_IGNORE_CASE) != 0){
			one = Character.toUpperCase(one);
			two = Character.toUpperCase(two);
		}
		return one-two;
	}

}
