/*
 * Created on Mar 26, 2005
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.util;

/**
 * This is a special version of a SubString that can be used to progressively
 * split an array of chars into component substrings one at a time.
 * <p>
 * It allows for more advanced splitting options than a SubString alone, and it 
 * can be reused using calls to getFirst().
 * 
 * @author Michael L Brereton
 *
 */
//####################################################
public class StringSplitter extends SubString {
//####################################################
/**
 * This is the maximum index allowed into the source data string. This is set
 * in the getFirst() method.
 */
public int max;
/**
 * This tells the splitter to ignore separators in quotes. It is false by default.
 */
public boolean ignoreSeparatorInQuotes;
/**
 * This tells is the separator being used to split the String.
 */
public char separator;

//public char options;

//public final static int 

public StringSplitter(char separator,boolean ignoreSeparatorInQuotes)
{
	this.separator = separator;
	this.ignoreSeparatorInQuotes = ignoreSeparatorInQuotes;
}

public Object getNew()
{
	return new StringSplitter(separator,ignoreSeparatorInQuotes);
}

/**
 * Get the first substring in the String. If the string is empty, this will return
 * true and this SubString will be set to an empty String. If it is null it will
 * return false;
 * @param data the data to start spliting.
 * @param offset the offset into the data.
 * @param length the maximum length of the data.
 * @return false only if data is null.
 */
public boolean getFirst(char[] data, int offset, int length)
{
	this.data = data;
	this.max = offset+length;
	start = offset;
	length = 0;
	if (data == null) return false;
	return getNext(true);
}
/**
 * Get the next substring in the String as specified in getFirst().
 * @return true if another SubString was found, false if there are no more SubStrings.
 */
public boolean getNext()
{
	return getNext(false);
}
private boolean getNext(boolean isFirst)
{
	int startFrom = start+length;
	//
	// Get past the current separator.
	//
	if (!isFirst){
		if (startFrom >= max) return false;
		startFrom++;
		if (separator == ' ') 
			for (;startFrom<max; startFrom++)
				if (data[startFrom] != ' ' && data[startFrom] != '\t') break;
	}
	//
	// startFrom may be >= max at this point.
	// in which case a substring of zero length will be returned.
	//
	this.start = startFrom;
	int e = 0;
	char inQuotes = 0;
	for (e = this.start; e < max; e++){
		char ch = data[e];
		if (inQuotes != 0){
			if (ch == inQuotes) inQuotes = 0;
		}else if (ignoreSeparatorInQuotes){
			if (ch == '\'' || ch == '"') inQuotes = ch;
		}else if (ch == separator){
			break;
		}
	}
	//
	this.length = e-this.start;
	return true;
}
//####################################################
}
//####################################################
