package eve.util;
import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.Vector;

import eve.sys.Convert;
import eve.sys.Handle;
import eve.sys.Reflection;
import eve.sys.Vm;
import eve.sys.Wrapper;
//import java.lang.reflect.Field;
//import eve.sys.Handle;

//##################################################################
public class Utils{
//##################################################################
/**
* This attempts to compare two objects which may or may not implement
* the Comparable interface. If they do, then it is used, otherwise the equals
* method is used.
* @param one The first object.
* @param two The second object
* @return less than 1 if one is considered less than two, greater than 1 if one is
* considered greater than two, 0 if they are considered equal.
*/
//===================================================================
public static int compare(Object one,Object two)
//===================================================================
{
	if (one == two) return 0;
	if (one == null) return -1;
	else if (two == null) return 1;
	else if (one instanceof Comparable) return ((Comparable)one).compareTo(two);
	else if (two instanceof Comparable) return -((Comparable)two).compareTo(one);
	else if (one.equals(two)) return 0;
	int o = one.hashCode(), t = two.hashCode();
	if (o > t) return 1;
	else if (o < t) return -1;
	else return 0;
}
/**
* This attempts to get a copy of an object. If the object implements Copyable then
* a copy of that object is returned, otherwise the object itself is returned.
* @param toCopy The object to copy.
*/
//===================================================================
public static Object getCopy(Object toCopy)
//===================================================================
{
		if (toCopy instanceof Copyable) return ((Copyable)toCopy).getCopy();
		return toCopy;
}
//===================================================================
public static void zeroArrayRegion(Object array, int offset, int length)
//===================================================================
{
	Class cType = array.getClass().getComponentType();
	if (cType == null) throw new IllegalArgumentException();
	//Do this in the native one.
	if (length == 0) return;
	int max = offset+length;
	if (array instanceof byte[]){
		byte[] a = (byte[])array;
		for (int i = offset; i<max; i++) a[i] = 0;
	}else if (array instanceof boolean[]){
		boolean[] a = (boolean[])array;
		for (int i = offset; i<max; i++) a[i] = false;
	}else if (array instanceof char[]){
		char[] a = (char[])array;
		for (int i = offset; i<max; i++) a[i] = 0;
	}else if (array instanceof short[]){
		short[] a = (short[])array;
		for (int i = offset; i<max; i++) a[i] = 0;
	}else if (array instanceof int[]){
		int[] a = (int[])array;
		for (int i = offset; i<max; i++) a[i] = 0;
	}else if (array instanceof float[]){
		float[] a = (float[])array;
		for (int i = offset; i<max; i++) a[i] = 0;
	}else if (array instanceof double[]){
		double[] a = (double[])array;
		for (int i = offset; i<max; i++) a[i] = 0;
	}else if (array instanceof long[]){
		long[] a = (long[])array;
		for (int i = offset; i<max; i++) a[i] = 0;
	}else if (array instanceof Object[]){
		Object[] a = (Object[])array;
		for (int i = offset; i<max; i++) a[i] = null;
	}
}
/**
* Creates a hashCode from a sequence of bytes.
**/
//===================================================================
public static int makeHashCode(byte [] data,int start,int length)
//===================================================================
{
	return new String(data,start,length).hashCode();
}
/**
* Creates a hashCode from a sequence of chars.
**/
//===================================================================
public static int makeHashCode(char [] data,int start,int length)
//===================================================================
{
	return new String(data,start,length).hashCode();
}
/**
 * Convert a Java Utf8 byte encoded string into a string of characters.
 * @param data The encoded data bytes.
 * @param start The start index of the data bytes.
 * @param numberOfBytes The number of bytes of encoded data. 
 * @param chars The destination array.
 * @param offset The start index of the destination to place the characters.
 * @return The destination array.
 */
public static final char []
//===================================================================
	decodeJavaUtf8String(byte [] data,int start,int numberOfBytes,char [] buffer,int offset)
//===================================================================
{
	int size = sizeofJavaUtf8String(data,start,numberOfBytes);
	if (buffer != null)
		if (buffer.length < size) buffer = null;
	if (buffer == null) buffer = new char[size];
	int i = 0, t = offset;
	for (i = 0; i<numberOfBytes; i++){
		byte c = data[i+start];
		if ((c & 0x80) == 0) 
			buffer[t] = (char)c;
		else if ((c & 0xe0) == 0xc0) {
			buffer[t] = (char)((((char)c & 0x1f)<<6) + ((char)data[i+start+1] & 0x3f));
			i++;
		}else if ((c & 0xf0) == 0xe0) {
			buffer[t] = (char)((((char)c & 0x0f)<<12) + (((char)data[i+start+1] & 0x3f)<<6)+((char)data[i+start+2] & 0x3f));
			i += 2;
		}
		t++;
		//ewe.sys.Vm.debug("<"+(int)buffer[t]);
	}
	return buffer;
}
/**
 * Encode a String of characters a Java Utf8 encoded bytes.
 * @param toEncode The characters to encode.
 * @param offset The start index of the character.
 * @param length The number of characters to encode.
 * @param destination The destination for the bytes.
 * @param destOffset The offset for the destination bytes 
 * @return the number of bytes used to encode the String.
 */
public static  int 
//===================================================================
encodeJavaUtf8String(char[] toEncode,int offset,int length,byte[] destination,int destOffset)
//===================================================================
{
	if (toEncode == null) return 0;
	int max = length;
	int size = destOffset;
	for (int i = 0; i<max; i++){
		char c = toEncode[i+offset];
		if (c >= 0x1 && c <= 0x7f) destination[size++] = (byte)c;
		else if (c == 0 || (c >= 0x80 && c <= 0x7ff)) {
			destination[size++] = (byte)(0xc0 | ((c >> 6) & 0x1f));
			destination[size++] = (byte) (0x80 | (c & 0x3f));
		}else{
			destination[size++] = (byte)(0xe0 | ((c >> 12) & 0xf));
			destination[size++] = (byte) (0x80 | ((c >> 6) & 0x3f));
			destination[size++] = (byte) (0x80 | (c  & 0x3f));
		}
	}
	return size-destOffset;
}

/**
 * Encodes a String as Java Utf8 bytes.
 * @param str the String to encode.
 * @return the encoded bytes.
 */
//===================================================================
public static byte [] encodeJavaUtf8String(String str)
//===================================================================
{
	char [] chars = eve.sys.Vm.getStringChars(str);
	int size = sizeofJavaUtf8String(chars,0,chars.length);
	byte [] got = new byte[size];
	encodeJavaUtf8String(chars,0,chars.length,got,0);
	return got;
}
/**
* Returns the number of characters needed to store a string which has been
* encoded in the Java UTF8 format.
* @param data the encoded data.
* @param start the start location in the data.
* @param numberOfBytes the number of bytes of encoded data.
* @return the number of characters for the String.
*/
public static  final int
//===================================================================
	sizeofJavaUtf8String(byte[] data,int start,int numberOfBytes)
//===================================================================
{
	int size = 0;
	for (int i = 0; i<numberOfBytes; i++){
		size++;
		byte c = data[i+start];
		if ((c & 0x80) == 0) continue;
		else if ((c & 0xe0) == 0xc0) i++;
		else if ((c & 0xf0) == 0xe0) i+=2;
	}
	return size;
}
/**
* Returns the number of bytes needed to encode a String in the Java UTF8 format.
* @param toEncode The characters to encode.
* @param start The start index of the character.
* @param length The number of characters to encode.
* @return the number of bytes needed to encode a String in the Java UTF8 format.
*/
public static final int
//===================================================================
	sizeofJavaUtf8String(char[] toEncode,int start,int length)
//===================================================================

{
	if (toEncode == null) return 0;
	int max = length;
	int size = 0;
	for (int i = 0; i<max; i++){
		size++;
		char c = toEncode[i+start];
		if (c >= 0x1 && c <= 0x7f) continue;
		else if (c == 0 || (c >= 0x80 && c <= 0x7ff)) size++;
		else size += 2;
	}
	return size;
}
/**
 * Decode a String encoded as Java Utf8 bytes.
 * @param bytes The encoded bytes.
 * @param start The start index of the encoded bytes.
 * @param length The number of encoded bytes.
 * @return The decoded String.
 */
//===================================================================
public static String decodeJavaUtf8String(byte[] bytes,int start,int length)
//===================================================================
{
	int size = sizeofJavaUtf8String(bytes,start,length);
	char [] got = new char [size];
	decodeJavaUtf8String(bytes,start,length,got,0);
	return new String(got);
}
/**
* This writes an integer value to a byte array with the specified number
* of bytes. It always writes out the high byte first.
* @param val The byte, short or integer value to write.
* @param dest The destination array.
* @param offset The location in the destination to write to.
* @param numBytes The number of bytes 1, 2 or 4 to write out.
*/
public static final void
	//============================================================
	writeInt(int val,byte[] dest,int offset,int numBytes)// throws IOException
//============================================================
{
	for (int i = offset+numBytes-1; i>=offset; i--) {
		dest[i] = (byte)(val & 0xff);
		val = val>>8;
	}
}
/**
* This reads an integer value from a byte array with the specified number
* of bytes. It always reads in the high byte first.
* @param source The source of the bytes.
* @param offset The location in the source.
* @param numBytes The number of bytes 1, 2 or 4 to read in.
* @return the integer value read in.
*/
public static final int
//============================================================
	readInt(byte [] source,int offset,int numBytes)// throws IOException
//============================================================
{
	int ret = 0;
	for (int i = offset; i<offset+numBytes; i++) {
		ret = (ret<<8) & 0xffffff00;
		ret |= ((int)source[i])&0xff;
	}
	return ret;
}

/**
* This reads a long value from a byte array as an 8 byte sequence - high byte first.
* @param source The source of the bytes.
* @param offset The location in the source.
* @return The long value read in.
*/
//===================================================================
public static final long readLong(byte[] source,int offset)
//===================================================================
{
	long val = (long)readInt(source,offset,4) << 32;
	val |= (readInt(source,offset+4,4) & 0x0ffffffffL);
	return val;
}
/**
* This writes a long value to a byte array as an 8 byte sequence - high byte first.
* @param value The long value to write.
* @param dest The destination for the bytes.
* @param offset The location in the destination to write to.
*/
//===================================================================
public static final void writeLong(long value,byte[] dest,int offset)
//===================================================================
{
	writeInt((int)((value >> 32) & 0x0ffffffffL),dest,offset,4);
	writeInt((int)(value & 0x0ffffffffL),dest,offset+4,4);
}
/**
* This puts a sequence of integers into a int array.
* @param sequence The destination for the integers.
* @param destOffset The index in the destination to start placing the values.
* @param first The first value.
* @param step The value to add to each value to get the next value.
* @param length The number of values to place.
*/
//===================================================================
public static  void getIntSequence(int[] sequence,int destOffset,int first,int step,int length)
//===================================================================
{
	for (int i = first, n = 0; n < length; n++){
		sequence[destOffset+n] = i;
		i += step;
	}
}
/**
* This fills the int array with an increasing sequence of integers starting with first. This calls:
* getIntSequence(sequence,0,first,1,sequence.length);
**/
//===================================================================
public static void getIntSequence(int [] sequence,int first)
//===================================================================
{
	getIntSequence(sequence,0,first,1,sequence.length);
}
/**
 * Sort an array of integers using the specified CompareInts comparer.
 * @param h an optional handle to track the progress and possible stop the sorting.
 * @param values the values to sort.
 * @param offset the start index to sort.
 * @param length the number of values to sort.
 * @param comparer the comparer to use. If it is null then a simple integer comparison is used.
 * @param descending true if the sort order should be descending.
 * @return true if the sort completed false if it was stopped because the handle was stopped
 * or because an error occured when calling the CompareInts method.
 */
/*
//===================================================================
public static native boolean sort(Handle h, int[] values, CompareInts comparer, boolean descending);
//===================================================================
 */
/*
//===================================================================
public static boolean sort(Handle h,int[] what,int offset,int length,CompareInts comparer,boolean descending)
//===================================================================
{
	int[] toSort = (offset == 0 && length == what.length) ? what : new int[length];
	if (toSort != what)
		System.arraycopy(what,offset,toSort,0,length);
	if (!sort(h,toSort,comparer,descending)) return false;
	if (toSort != what)
		System.arraycopy(toSort,0,what,offset,length);
	return true;
}

//===================================================================
public static boolean sort(Handle h,Object[] values,Comparer comparer,boolean descending)
//===================================================================
{
	int [] vals = new int[values.length];
	getIntSequence(vals,0,0,1,values.length);
	if (!sort(h,vals,new CompareArrayElements(values,comparer),descending))
		return false;
	Object [] newVals = new Object[vals.length];
	for (int i = 0; i<vals.length; i++) newVals[i] = values[vals[i]];
	System.arraycopy(newVals,0,values,0,values.length);
	return true;
}
//===================================================================
public static boolean sort(Object[] values,Comparer comparer,boolean descending)
//===================================================================
{
	return sort(null,values,comparer,descending);
}
*/
/**
 * Returns if the object can be used to provide an Iterator or Enumeration. The criteria for this
 * is if dataSource is a Vector or Enumeration or Iterator.
 * @param dataSource the data source to check.
 * @return true if the dataSource can be converted to an Iterator or Enumeration.
 */
public static boolean canIterate(Object dataSource)
{
	return (dataSource instanceof Iterator || dataSource instanceof Enumeration || dataSource instanceof java.util.Vector);
}
/**
This provides an Enumeration of items provided in dataSource, which may itself be
a Vector, Iterator, or an Enumeration itself. If it is not any of these, a Enumeration
will be returned which enumerates through zero elements.
 * @param dataSource the data source to convert.
 * @return an Enumeration for the dataSource.
*/
//===================================================================
public static Enumeration toEnumeration(Object dataSource)
//===================================================================
{
	if (dataSource instanceof Enumeration) return (Enumeration)dataSource;
	if (dataSource instanceof Iterator) return new IteratorEnumerator((Iterator)dataSource);
	if (dataSource instanceof java.util.Vector) return ((java.util.Vector)dataSource).elements();
	return new ObjectIterator(null);
}
/**
This provides an Iterator of items provided in dataSource, which may itself be
a java.util.Vector, java.util.Iterator, java.util.Enumeration or an eve.util.Iterator itself.
*/
//===================================================================
public static Iterator toIterator(Object dataSource)
//===================================================================
{
	if (dataSource instanceof Iterator) return (Iterator)dataSource;
	if (dataSource instanceof Enumeration) return new IteratorEnumerator((Enumeration)dataSource);
	if (dataSource instanceof java.util.Vector) return ((java.util.Vector)dataSource).iterator();
	return new ObjectIterator(null);
}
/**
 * Join two arrays together to form a new array which contains the elements of the first followed by the elements of the second.
 * @param original The first array. Can be null, in which case only the elements of toAdd will go into the new array.
 * @param toAppend The second array. Can be null, in which case only the elements of original will go into the new array.
 * @return a new array representing original + toAppend. If both original and toAppend are null, this will return null.
 */
//===================================================================
public static Object appendArray(Object original,Object toAppend)
//===================================================================
{
	if (original == null && toAppend == null) return null;
	Class c = original == null ? toAppend.getClass() : original.getClass();
	if (!c.isArray()) throw new IllegalArgumentException("Only arrays can be appended.");
	int oldLen = original == null ? 0 : Array.getLength(original), 
		newLen = toAppend == null ? 0 : Array.getLength(toAppend);
	Object got = Array.newInstance(c.getComponentType(),newLen+oldLen);
	if (oldLen != 0) System.arraycopy(original,0,got,0,oldLen);
	if (newLen != 0) System.arraycopy(toAppend,0,got,oldLen,newLen);
	return got;
}
/**
 * Encode the array of primitive Java values as a String. The length of the array is not encoded
 * along with this data, nor is the type of the data. You will have to store these yourself.
 * <p>See the TextEncoder/TextDecoder objects for how to do this.
 * @param obj An array of any Java type except Object or array.
 * @return The String encoded data.
 */
//===================================================================
public static String primitiveArrayToString(Object obj)
//===================================================================
{
	byte [] toSave = null;
	if (obj instanceof byte []){
		byte [] a = (byte [])obj;
		toSave = new byte[a.length];
		for (int i = 0; i<a.length; i++) toSave[i] = a[i];
	}else if (obj instanceof boolean []){
		boolean [] a = (boolean [])obj;
		toSave = new byte[a.length];
		for (int i = 0; i<a.length; i++) toSave[i] = a[i] ? (byte)1 : (byte)0;
	}else if (obj instanceof short []){
		short [] a = (short [])obj;
		toSave = new byte[a.length*2];
		for (int i = 0; i<a.length; i++)
			writeInt(a[i],toSave,i*2,2);
	}else if (obj instanceof char []){
		char [] a = (char [])obj;
		toSave = new byte[a.length*2];
		for (int i = 0; i<a.length; i++)
			writeInt(a[i],toSave,i*2,2);
	}else if (obj instanceof int []){
		int [] a = (int [])obj;
		toSave = new byte[a.length*4];
		for (int i = 0; i<a.length; i++)
			writeInt(a[i],toSave,i*4,4);
	}else if (obj instanceof long []){
		long [] a = (long [])obj;
		toSave = new byte[a.length*8];
		for (int i = 0; i<a.length; i++)
			writeLong(a[i],toSave,i*8);
	}else if (obj instanceof float []){
		float [] a = (float [])obj;
		toSave = new byte[a.length*8];
		for (int i = 0; i<a.length; i++)
			writeLong(Convert.toLongBitwise(a[i]),toSave,i*8);
	}else if (obj instanceof double []){
		double [] a = (double [])obj;
		toSave = new byte[a.length*8];
		for (int i = 0; i<a.length; i++)
			writeLong(Convert.toLongBitwise(a[i]),toSave,i*8);
	}
	if (toSave == null) return "";
	char [] ret = new char[toSave.length*2];
	for (int i = 0; i<toSave.length; i++){
		ret[i*2] = (char)(((toSave[i] >> 4) & 0xf)+'0');
		if (ret[i*2] > '9') ret[i*2] -= '9'-('A'-1);
		ret[i*2+1] = (char)(((toSave[i]) & 0xf)+'0');
		if (ret[i*2+1] > '9') ret[i*2+1] -= '9'-('A'-1);
	}
	return new String(ret);
}
/**
 * Decode an array of primitive values, encoded as a String.
 * @param obj The array of the correct type and length.
 * @param data The encoded data.
 */
//===================================================================
public static void primitiveArrayFromString(Object obj,String data)
//===================================================================
{
	char [] from = Vm.getStringChars(data);
	byte [] toSave = new byte[from.length/2];
	for (int i = 0; i<toSave.length; i++){
		int one = 0; char c = from[i*2];
		if (c <= '9') one = c-'0';
		else if (c <= 'F') one = c-'A'+10;
		else if (c <= 'f') one = c-'a'+10;
		int two = 0; c = from[i*2+1];
		if (c <= '9') two = c-'0';
		else if (c <= 'F') two = c-'A'+10;
		else if (c <= 'f') two = c-'a'+10;
		toSave[i] = (byte)(one << 4 | two);
	}
	if (obj instanceof byte []){
		byte [] a = (byte [])obj;
		System.arraycopy(toSave,0,a,0,toSave.length);
	}else if (obj instanceof boolean []){
		boolean [] a = (boolean [])obj;
		for (int i = 0; i<a.length; i++) a[i] = toSave[i] == 0 ? false : true;
	}else if (obj instanceof short []){
		short [] a = (short [])obj;
		for (int i = 0; i<a.length; i++)
			a[i] = (short)readInt(toSave,i*2,2);
	}else if (obj instanceof char []){
		char [] a = (char [])obj;
		for (int i = 0; i<a.length; i++)
			a[i] = (char)readInt(toSave,i*2,2);
	}else if (obj instanceof int []){
		int [] a = (int [])obj;
		for (int i = 0; i<a.length; i++)
			a[i] = readInt(toSave,i*4,4);
	}else if (obj instanceof long []){
		long [] a = (long [])obj;
		for (int i = 0; i<a.length; i++)
			a[i] = readLong(toSave,i*8);
	}else if (obj instanceof float []){
		float [] a = (float [])obj;
		for (int i = 0; i<a.length; i++)
			a[i] = (float)Convert.toDoubleBitwise(readLong(toSave,i*8));
	}else if (obj instanceof double []){
		double [] a = (double [])obj;
		for (int i = 0; i<a.length; i++)
			a[i] = Convert.toDoubleBitwise(readLong(toSave,i*8));
	}
}
//-------------------------------------------------------------------
static Field [] fieldsFor(Object obj)
//-------------------------------------------------------------------
{
	if (obj == null) return null;
	Class r = obj.getClass();
	Field [] allFields = null;
	Field f = Reflection.getField(r,"_fields",false);
	if (f != null && f.getType().equals(String.class)) {
		try{
			String [] s = mString.split((String)f.get(obj),',');
			allFields = new Field[s.length];
			for (int i = 0; i<s.length; i++)
				allFields[i] = Reflection.getField(r,s[i],false);
		}catch(Exception e){}
	}else{
		try{
			allFields = r.getFields();
		}catch(Exception e){}
	}
	return allFields;
}
/**
* Encode the fields of the Encodable object.
* @param obj The object to encode.
* @return A simple encoded string for the object.
*/
//===================================================================
public static String textEncode(Encodable obj)
//===================================================================
{
	Field [] allFields = fieldsFor(obj);
	if (allFields == null) return null;
	TextEncoder te = new TextEncoder();
	Wrapper w = null;
	for (int i = 0; i<allFields.length; i++){
		if (allFields[i] == null) continue;
		if (Modifier.isStatic(allFields[i].getModifiers())) continue;
		Class c = allFields[i].getType();
		String toSave = null;
		if (!c.isPrimitive()){
			try{
				TextEncoder t2 = new TextEncoder();
				TextEncoder.saveFullObject(allFields[i].get(obj),t2);
				toSave = t2.toString();
			}catch(Exception e){}
		}else{
			if (w == null) w = new Wrapper();
			toSave = TextEncoder.toString(allFields[i],obj,w);
		}
		te.addValue(allFields[i].getName(),toSave);
	}
	return te.toString();
}
/**
 * Encode the fields of the Encodable object.
 * @param obj The object to encode.
 * @return A simple encoded string for the object.
 */
//===================================================================
public static void textDecode(Encodable obj,String data)
//===================================================================
{
	Field [] allFields = fieldsFor(obj);
	if (allFields == null) return;
	TextDecoder td = new TextDecoder(data);
	Wrapper w = null;
	for (int i = 0; i<allFields.length; i++){
		if (allFields[i] == null) continue;
		if (Modifier.isStatic(allFields[i].getModifiers())) continue;
		Class c = allFields[i].getType();
		String s = td.getValue(allFields[i].getName());
		if (s != null)
			try{
				if (!c.isPrimitive()){
					allFields[i].set(obj,TextEncoder.getFullObject(s,obj.getClass()));
				}else{
					if (w == null) w = new Wrapper();
					TextEncoder.fromString(allFields[i],obj,s,w);	
				}
			}catch(Exception e){}
	}
}
//-------------------------------------------------------------------
protected static boolean merge(Handle h,int [] source,int sourceLength,int one,int two,int length,int [] dest,boolean descending,CompareInts comparer)
//-------------------------------------------------------------------
{
	int o = one, t = two, d = one;
	int omax = one+length, tmax = two+length;
	if (omax > sourceLength) omax = sourceLength;
	if (tmax > sourceLength) tmax = sourceLength;
	
	while(true) {
		if (h != null)
			if (h.shouldStop) return false;
			//else ewe.sys.mThread.yield(250);
		if (o >= omax) {
			if (t >= tmax) return true;
			dest[d++] = source[t++];
		}else {
			if (t >= tmax) dest[d++] = source[o++];
			else {
				int c = 0;
				if (comparer != null) c = comparer.compare(source[o],source[t]);
				else if (source[o] < source[t]) c = -1;
				else if (source[o] > source[t]) c = +1;
				if (((c <= 0) && !descending) || ((c > 0) && descending))
				 dest[d++] = source[o++];
				else dest[d++] = source[t++];
			}
		}
	}
}
/**
* This sorts the array in place. 
* If the handle is not null then the sort will
* yield occasionally to let other threads operate. The handle will be updated to
* show the progress but the handle Handle.Stopped bit will never be set.
* If this returns false this indicates that stop() was called on the handle.
*/
//===================================================================
public static boolean sort(Handle h,int[] what,int length,CompareInts comparer,boolean descending)
//===================================================================
{
	if (length <= 0) return true;
	int len = length;
	int needPasses = 0;
	int l2 = len;
	do{
		needPasses++;
		l2 >>= 1;
	}while(l2 != 0);
	
	int [] source = what, dest = new int[len], temp;
	int mergeLength = 1, passes = 0;
	if (h != null){
		h.startDoing("Sorting");
	}
	while(true) {
		int mergesDid = 0, one = 0, two = 0;
		while(true) {
			if (one >= len) break;
			two = one+mergeLength;
			if ((two >= len) && (mergesDid == 0)) break;
			mergesDid++;
			if (!merge(h,source,length,one,two,mergeLength,dest,descending,comparer))
				return false;
			one += mergeLength*2;
		}
		if (mergesDid == 0) break;
		temp = dest; dest = source; source = temp;
		mergeLength *= 2;
		passes++;
		if (h != null) {
			h.progress = (float)(((double)passes)/(needPasses));
			h.changed();
		}
	}
	if (source != what) System.arraycopy(source,0,what,0,len);
	return true;
}
/**
 * This method sorts the indexes of the values array and returns the indexes
 * as a new array of ints. It does not modify the values array.
 * @param h a handle that can be used to monitor and stop the sorting.
 * @param values the values to compare.
 * @param comparer a Comparer to compare the Objects in values. If this is
 * null a StandardComparer is used.
 * @param descending set true to sort in descending order.
 * @return an int[] on success, or null if the handle was stopped.
 */
public static int[] sortIndexes(Handle h,Object[] values,Comparer comparer,boolean descending)
{
	int [] vals = new int[values.length];
	getIntSequence(vals,0);
	if (h == null) 
		sort(vals,vals.length,new CompareArrayElements(values,comparer),descending);
	else 
		if (!sort(h,vals,vals.length,new CompareArrayElements(values,comparer),descending))
			return null;
	return vals;
}
/**
* This sorts the objects in place.
* If the handle is not null then the sort will
* yield occasionally to let other threads operate. The handle will be updated to
* show the progress but the handle Handle.Stopped bit will never be set.
**/
//===================================================================
public static boolean sort(Handle h,Object [] values,Comparer comparer,boolean descending)
//===================================================================
{
	int [] vals = sortIndexes(h,values,comparer,descending);
	if (vals == null) return false;
	Object [] newVals = new Object[vals.length];
	for (int i = 0; i<vals.length; i++) newVals[i] = values[vals[i]];
	System.arraycopy(newVals,0,values,0,values.length);
	return true;
}

/**
* This sorts the array in place.
**/
//===================================================================
public static void sort(int [] what,int length,CompareInts comparer,boolean descending)
//===================================================================
{
	sort(null,what,length,comparer,descending);
}
/**
* This sorts the objects in place.
**/
//===================================================================
public static void sort(Object [] values,Comparer comparer,boolean descending)
//===================================================================
{
	sort(null,values,comparer,descending);
}
/**
Find the index of a value within an array. The value should be converted to an appropriate
integer/long value and passed to the method. The type of the array will determine the how
the lookingFor value will be used.<p>
Searching begins from the startIndex and will continue either forwards or backwards as
specified. A return value of -1 indicates that the value was not found.
**/
public static int indexOf(Object array, long lookingFor, int minIndex, int maxIndex, boolean backwards)
{
	int i;
	int len = Array.getLength(array);
	//
	if (len == -1) throw new IllegalArgumentException();
	if (maxIndex > len) maxIndex = len;
	//
	if (backwards){
		if (minIndex >= maxIndex) minIndex = maxIndex-1;
		if (minIndex < 0) return -1;
	}else{
		if (minIndex < 0) minIndex = 0;
		if (minIndex >= maxIndex) return -1;
	}
	//
	if (array instanceof int[]){
		int[] s = (int[])array;
		int look = (int)lookingFor;
		if (backwards) for (i = maxIndex-1; i >= minIndex; i--) if (s[i] == look) return i;
		else for (i = minIndex; i < maxIndex; i++) if (s[i] == look) return i;
	}
	return -1;
}

private static void convert(int[] src, int offset, int length, byte[] dest, int doff)
{
	for (int i = 0; i<length; i++){
		writeInt(src[offset+i],dest,doff,4);
		doff += 4;
	}
}
private static void convert(byte[] src, int offset, int length, int[] dest, int doff)
{
	length /= 4;
	for (int i = 0; i<length; i++)
		dest[doff++] = readInt(src,offset+i*4,4);
}
/**
 * Convert a sequence of ints to a sequence of bytes where each integer is represented
 * as a sequence of 4 bytes - high order byte first. The bytes are appended to
 * a ByteArray.
 * @param src the source of the int values.
 * @param offset the start of the int values.
 * @param length the number of int values.
 * @param dest an optional destination ByteArray. If it is null a new one is allocated
 * and returned.
 * @return the destination ByteArray or a new one if the destination is null.
 */
public static ByteArray toBytes(int[] src, int offset, int length, ByteArray dest)
{
	if (dest == null) dest = new ByteArray();
	if (src == null) throw new NullPointerException();
	if (offset < 0 || length < 0 || offset+length > src.length) throw new ArrayIndexOutOfBoundsException();
	dest.ensureCapacity(dest.length+length*4);
	dest.length += length*4;
	convert(src,offset,length,dest.data,dest.length-length*4);
	return dest;
}
/**
 * Convert a sequence of bytes to a sequence of ints where each integer is represented
 * as a sequence of 4 bytes - high order byte first. The ints are appended to
 * an IntArray.
 * @param src the source of the bytes values.
 * @param offset the start of the byte values.
 * @param length the number of byte values.
 * @param dest an optional destination IntArray. If it is null a new one is allocated
 * and returned.
 * @return the destination IntArray or a new one if the destination is null.
 */
public static IntArray toInts(byte[] src, int offset, int length, IntArray dest)
{
	if (dest == null) dest = new IntArray();
	if (src == null) throw new NullPointerException();
	if ((length % 4) != 0) throw new IllegalArgumentException();
	if (offset < 0 || length < 0 || offset+length > src.length) throw new ArrayIndexOutOfBoundsException();
	dest.ensureCapacity(dest.length+length/4);
	dest.length += length/4;
	convert(src,offset,length,dest.data,dest.length-length/4);
	return dest;
}
public static void sort(Handle h, Vector v, Comparer comparer, boolean descending)
{
	Object[] all = new Object[v.size()];
	v.copyInto(all);
	sort(h,all,comparer,descending);
	v.removeAllElements();
	mVector.addAll(v,all);
}
public static void sort(Vector v, boolean descending)
{
	sort(null,v,StandardComparer.comparer,descending);
}
public static void reverse(int[] data, int offset,int length)
{
	int max = length/2;
	int t = 0;
	int off = offset;
	for (int j = 0; j<max; j++){
		t = data[off+j];
		data[off+j] = data[off+length-1-j];
		data[off+length-1-j] = t;
	}
}

//##################################################################
}
//##################################################################


